--
-- Fran lifted images.  Also lifts pickimage.hs.
-- 
-- Last modified Thu Nov 07 14:37:00 1996

module ImageB(
	ImageB, Image,
        emptyImage,
        line, polyline, bezier, 
	circle,	square,	polygon, ellipse, rectangle, star, regularPolygon,
	squareMin, squareMax, circleRadius,
        renderedText, 
	importBitmap,
        over,
        bboxed2, unitBBoxed2,
	withColor, 
	pick2) where        

import qualified Image as I
import Image(Image)
import qualified PickImage as PI
import Fuzzy
import Behavior
import Point2B(Point2B)
import Vector2B(Vector2B)
import Vector2(Vector2)
import TextB(TextB)
import ColorB(ColorB)
import Win32(HBITMAP)
import BaseTypes
	
type ImageB = Behavior Image

-- Hugs bug (?) workaround.  See comment in VectorSpaceB.
-- infixl 6 `over`

emptyImage   :: ImageB

line         :: Point2B -> Point2B -> ImageB  
polyline     :: [Point2B] -> ImageB
bezier       :: Point2B -> Point2B -> Point2B -> Point2B -> ImageB

circle       :: ImageB    -- Center (0,0) radius 1
square       :: ImageB    -- Within unit circle
polygon      :: [Point2B] -> ImageB
ellipse      :: Vector2B -> ImageB
rectangle    :: Vector2B -> ImageB
star         :: Behavior Int -> Behavior Int -> ImageB
regularPolygon :: Behavior Int -> ImageB

squareMin    :: Point2B  -- upper right of square in unit circle
squareMax    :: Point2B  -- lower left of square in unit circle
circleRadius :: Behavior RealVal -- radius of unit cirle

renderedText :: TextB -> ImageB    -- Height is about 0.1

importBitmap :: String -> ImageB  -- String is file name

over         :: ImageB -> ImageB -> ImageB

bboxed2      :: Point2B -> Point2B -> ImageB -> ImageB
unitBBoxed2  :: ImageB -> ImageB

withColor    :: ColorB -> ImageB -> ImageB

pick2        :: ImageB -> Point2B -> BoolB


emptyImage   = lift0 I.emptyImage
circle       = lift0 I.circle
square       = lift0 I.square
line         = lift2 I.line (noI "line")  -- jcp: added noI
polyline ls  = lift1 I.polyline (noI "polyline") (liftLs ls)
polygon ls   = lift1 I.polygon	(noI "polygon" ) (liftLs ls)
bezier       = lift4 I.bezier (noI "bezier")
renderedText = lift1 I.renderedText (noI "renderedText")
withColor    = lift2 I.withColor (noI "withColor")
over         = lift2 I.over (noI "over")
bboxed2      = lift3 I.bboxed2 (noI "bboxed2")
ellipse      = lift1 I.ellipse (noI "ellipse")
rectangle    = lift1 I.rectangle (noI "rectangle")
regularPolygon = lift1 I.regularPolygon (noI "regularPolygon")
star         = lift2 I.star (noI "star")
unitBBoxed2  = lift1 I.unitBBoxed2 (noI "unitBBoxed2")

pick2        = lift2 PI.pick2 (noI "pick2")

-- Importation still acts on strings -- not string behaviors
importBitmap fileName  = lift0 (I.importBitmap fileName)

squareMin     = lift0 I.squareMin
squareMax     = lift0 I.squareMax
circleRadius  = lift0 I.circleRadius

-- *% handled in general in Transform2B.hs

