################################################################
# Files
################################################################

%synonym : AccessMode : DWORD

# Choose any combination of these
%const AccessMode gENERIC_NONE = 0 # non-standard name == can only be queried
%const AccessMode GENERIC_READ
%const AccessMode GENERIC_WRITE
%const AccessMode GENERIC_EXECUTE
%const AccessMode GENERIC_ALL

# Maybe combine with the following after consulting Win32 docs

%const AccessMode DELETE          
%const AccessMode READ_CONTROL    
%const AccessMode WRITE_DAC       
%const AccessMode WRITE_OWNER     
%const AccessMode SYNCHRONIZE     
%const AccessMode STANDARD_RIGHTS_REQUIRED 
%const AccessMode STANDARD_RIGHTS_READ     
%const AccessMode STANDARD_RIGHTS_WRITE    
%const AccessMode STANDARD_RIGHTS_EXECUTE  
%const AccessMode STANDARD_RIGHTS_ALL  
%const AccessMode SPECIFIC_RIGHTS_ALL
%const AccessMode ACCESS_SYSTEM_SECURITY
%const AccessMode MAXIMUM_ALLOWED        # MaximumAllowed access type


%synonym : ShareMode : DWORD

# Choose any combination of these
%const ShareMode fILE_SHARE_NONE = 0 # non-standard name
%const ShareMode FILE_SHARE_READ
%const ShareMode FILE_SHARE_WRITE

%newtype : CreateMode : DWORD : ()

# Choose one of these
%const CreateMode CREATE_NEW          
%const CreateMode CREATE_ALWAYS       
%const CreateMode OPEN_EXISTING       
%const CreateMode OPEN_ALWAYS         
%const CreateMode TRUNCATE_EXISTING   


%synonym : FileAttributeOrFlag : DWORD

# Choose any combination of these
%const FileAttributeOrFlag FILE_ATTRIBUTE_READONLY   
%const FileAttributeOrFlag FILE_ATTRIBUTE_HIDDEN     
%const FileAttributeOrFlag FILE_ATTRIBUTE_SYSTEM     
%const FileAttributeOrFlag FILE_ATTRIBUTE_DIRECTORY  
%const FileAttributeOrFlag FILE_ATTRIBUTE_ARCHIVE    
%const FileAttributeOrFlag FILE_ATTRIBUTE_NORMAL     # overridden by all other attributes 
%const FileAttributeOrFlag FILE_ATTRIBUTE_TEMPORARY  
%const FileAttributeOrFlag FILE_ATTRIBUTE_COMPRESSED 

# And combine with any combination of these
%const FileAttributeOrFlag FILE_FLAG_WRITE_THROUGH     
%const FileAttributeOrFlag FILE_FLAG_OVERLAPPED        
%const FileAttributeOrFlag FILE_FLAG_NO_BUFFERING      
%const FileAttributeOrFlag FILE_FLAG_RANDOM_ACCESS     
%const FileAttributeOrFlag FILE_FLAG_SEQUENTIAL_SCAN   
%const FileAttributeOrFlag FILE_FLAG_DELETE_ON_CLOSE   
%const FileAttributeOrFlag FILE_FLAG_BACKUP_SEMANTICS  
%const FileAttributeOrFlag FILE_FLAG_POSIX_SEMANTICS   

# And with any combination of these - CLIENT SIDE OF PIPE ONLY
# Combination must include SECURITY_SQOS_PRESENT!

%const FileAttributeOrFlag SECURITY_ANONYMOUS         
%const FileAttributeOrFlag SECURITY_IDENTIFICATION    
%const FileAttributeOrFlag SECURITY_IMPERSONATION     
%const FileAttributeOrFlag SECURITY_DELEGATION        
%const FileAttributeOrFlag SECURITY_CONTEXT_TRACKING  
%const FileAttributeOrFlag SECURITY_EFFECTIVE_ONLY    
%const FileAttributeOrFlag SECURITY_SQOS_PRESENT      
%const FileAttributeOrFlag SECURITY_VALID_SQOS_FLAGS  


# ToDo: define all the security attributes stuff
%newtype : LPSECURITY_ATTRIBUTES : Ptr : () : notypedef

%maybe : MbLPSECURITY_ATTRIBUTES : LPSECURITY_ATTRIBUTES : $0 : (LPSECURITY_ATTRIBUTES) 0

%errfun NullHANDLE CreateFile :: String -> AccessMode -> ShareMode -> MbLPSECURITY_ATTRIBUTES -> CreateMode -> FileAttributeOrFlag -> MbHANDLE -> IO HANDLE

%errfun False CloseHandle :: HANDLE -> IO ()

%errfun False DeleteFile :: String -> IO ()
%errfun False CopyFile   :: String -> String -> Bool -> IO ()
%errfun False MoveFile   :: String -> String -> IO ()

%synonym : MoveFileFlag : DWORD

# Use any combination of these
# except MOVEFILE_DELAY_UNTIL_REBOOT and MOVEFILE_COPY_ALLOWED
%const MoveFileFlag MOVEFILE_REPLACE_EXISTING   
%const MoveFileFlag MOVEFILE_COPY_ALLOWED       
%const MoveFileFlag MOVEFILE_DELAY_UNTIL_REBOOT 

%errfun False MoveFileEx :: String -> String -> MoveFileFlag -> IO ()


%newtype : FilePtrDirection : DWORD : ()

%const FilePtrDirection FILE_BEGIN  
%const FilePtrDirection FILE_CURRENT
%const FilePtrDirection FILE_END    

# ToDo: no support for 64 bit numbers just yet
# ToDo: In 64 bit mode, we need to check "GetLastError" as well to
#       confirm an error.
%prim SetFilePointer :: HANDLE -> Word64 -> FilePtrDirection -> IO Word64
DWORD r = SetFilePointer($0,$1,NULL,$2);
%errreturn(0xFFFFFFFF, r, r);

%struct : OVERLAPPED : OVERLAPPED $0
    DWORD   : Internal
    DWORD   : InternalHigh
    DWORD   : Offset
    DWORD   : OffsetHigh
    HANDLE  : hEvent

%ptr   : LPOVERLAPPED   : OVERLAPPED
%maybe : MbLPOVERLAPPED : LPOVERLAPPED : $0 : (LPOVERLAPPED) 0

%errfun False ReadFile :: HANDLE -> LPVOID -> DWORD -> MbLPOVERLAPPED -> IO DWORD using $err = ReadFile($0,$1,$2,&$r0,$3)

%errfun False WriteFile :: HANDLE -> LPVOID -> DWORD -> MbLPOVERLAPPED -> IO DWORD using $err = WriteFile($0,$1,$2,&$r0,$3);

# ToDo: ReadFileEx, WriteFileEx

%errfun False SetEndOfFile     :: HANDLE -> IO ()
%errfun False FlushFileBuffers :: HANDLE -> IO ()

# ToDo: %errfun False UnlockFile :: HANDLE -> Word64 -> Word64 -> IO ()
# ToDo: %errfun False LockFile   :: HANDLE -> Word64 -> Word64 -> IO ()
# ToDo: LockFileEx
# ToDo: UnlockFileEx

# Can I use INT64 instead???
# %struct : FILETIME : FILETIME $0
#   DWORD : dwLowDateTime
#   DWORD : dwHighDateTime

# ToDo: FindFirstFile, FindNextFile, FindClose
# ToDo: SearchPath

# ToDo: GetTempFileName, GetTempPath

%errfun 0xFFFFFFFF GetFileAttributes :: String -> IO FileAttributeOrFlag
%errfun False     SetFileAttributes :: String -> FileAttributeOrFlag -> IO ()


%newtype : FileType : DWORD : (Eq) 

%const FileType	FILE_TYPE_UNKNOWN   
%const FileType	FILE_TYPE_DISK      
%const FileType	FILE_TYPE_CHAR      
%const FileType	FILE_TYPE_PIPE      
%const FileType	FILE_TYPE_REMOTE    


%fun GetFileType :: HANDLE -> IO FileType
# ToDo: GetFileSize, GetFileTime, GetFullPathName, GetShortPathName
# ToDo: GetFileInformationByHandle

%errfun False CreateDirectory :: String -> MbLPSECURITY_ATTRIBUTES -> IO ()
%errfun False CreateDirectoryEx :: String -> String -> MbLPSECURITY_ATTRIBUTES -> IO ()
%errfun False RemoveDirectory :: String -> IO ()
# ToDo: GetCurrentDirectory
%errfun False SetCurrentDirectory :: String -> IO ()

%newtype : FileNotificationFlag : DWORD : (Eq)
{{%
instance BitSet FileNotificationFlag where
  toWord32   = \ (FileNotificationFlag x) -> x
  fromWord32 = FileNotificationFlag
%}}

%const FileNotificationFlag FILE_NOTIFY_CHANGE_FILE_NAME  
%const FileNotificationFlag FILE_NOTIFY_CHANGE_DIR_NAME   
%const FileNotificationFlag FILE_NOTIFY_CHANGE_ATTRIBUTES 
%const FileNotificationFlag FILE_NOTIFY_CHANGE_SIZE       
%const FileNotificationFlag FILE_NOTIFY_CHANGE_LAST_WRITE 
%const FileNotificationFlag FILE_NOTIFY_CHANGE_SECURITY   

%errfun INVALID_HANDLE_VALUE FindFirstChangeNotification :: String -> Bool -> FileNotificationFlag -> IO HANDLE

%errfun False FindNextChangeNotification  :: HANDLE -> IO ()
%errfun False FindCloseChangeNotification :: HANDLE -> IO ()

%errfun False SetVolumeLabel :: String -> String -> IO ()
# ToDo: GetVolumeInformation
# ToDo: GetSystemDirectory, GetWindowsDirectory

%errfun False GetDiskFreeSpace :: MbString -> IO (DWORD, DWORD, DWORD, DWORD) using $err = GetDiskFreeSpace($0,&$r0,&$r1,&$r2,&$r3)

%newtype : DriveType : UINT : (Eq)

%const DriveType DRIVE_UNKNOWN     
%const DriveType DRIVE_NO_ROOT_DIR 
%const DriveType DRIVE_REMOVABLE   
%const DriveType DRIVE_FIXED       
%const DriveType DRIVE_REMOTE      
%const DriveType DRIVE_CDROM       
%const DriveType DRIVE_RAMDISK     

%fun GetDriveType :: MbString -> IO DriveType

%errfun Zero GetLogicalDrives :: IO DWORD
# ToDo: GetLogicalDriveStrings

%fun AreFileApisANSI   :: IO Bool
%fun SetFileApisToOEM  :: IO ()
%fun SetFileApisToANSI :: IO ()
%fun SetHandleCount    :: UINT -> IO UINT

# ToDo: CreateIoCompletionPort
# ToDo: FileIoCompletionRoutine
# ToDo: GetQueuedCompletionStatus

%newtype : DefineDosDeviceFlags : DWORD : ()
{{%
instance BitSet DefineDosDeviceFlags where
  toWord32   = \ (DefineDosDeviceFlags x) -> x
  fromWord32 = DefineDosDeviceFlags
%}}

# One or more of the following
%const DefineDosDeviceFlags DDD_RAW_TARGET_PATH         
%const DefineDosDeviceFlags DDD_REMOVE_DEFINITION       
%const DefineDosDeviceFlags DDD_EXACT_MATCH_ON_REMOVE   

%errfun False DefineDosDevice :: DefineDosDeviceFlags -> String -> String -> IO ()
# ToDo: QueryDosDevice

%newtype : BinaryType : DWORD : (Eq)

%const BinaryType SCS_32BIT_BINARY    
%const BinaryType SCS_DOS_BINARY      
%const BinaryType SCS_WOW_BINARY      
%const BinaryType SCS_PIF_BINARY      
%const BinaryType SCS_POSIX_BINARY    
%const BinaryType SCS_OS216_BINARY    

%errfun False GetBinaryType :: String -> IO BinaryType using $err = GetBinaryType($0,&$r0)

