################################################################
# Fonts and text operations
################################################################

# ToDo: Should Height and Width be Maybes, Eithers or something weirder?

%synonym : FontWeight : INT
# Not abstract: use 0 or Int in range 1..1000

%const FontWeight FW_DONTCARE         # Should this be Nothing?
%const FontWeight FW_THIN             # == 100
%const FontWeight FW_EXTRALIGHT       # == 200
%const FontWeight FW_LIGHT            # == 300
%const FontWeight FW_NORMAL           # == 400
%const FontWeight FW_MEDIUM           # == 500
%const FontWeight FW_SEMIBOLD         # == 600
%const FontWeight FW_BOLD             # == 700
%const FontWeight FW_EXTRABOLD        # == 800
%const FontWeight FW_HEAVY            # == 900 

%const FontWeight FW_ULTRALIGHT       # == FW_EXTRALIGHT
%const FontWeight FW_REGULAR          # == FW_NORMAL
%const FontWeight FW_DEMIBOLD         # == FW_SEMIBOLD
%const FontWeight FW_ULTRABOLD        # == FW_EXTRABOLD
%const FontWeight FW_BLACK            # == FW_HEAVY

%newtype : CharSet : BYTE : (Eq)

# Choose one of these
%const CharSet ANSI_CHARSET        
%const CharSet DEFAULT_CHARSET     
%const CharSet SYMBOL_CHARSET      
%const CharSet SHIFTJIS_CHARSET    
%const CharSet HANGEUL_CHARSET     
%const CharSet CHINESEBIG5_CHARSET 
%const CharSet OEM_CHARSET         

%newtype : OutPrecision : BYTE : (Eq)

# Choose one of these
%const OutPrecision OUT_DEFAULT_PRECIS      
%const OutPrecision OUT_STRING_PRECIS       
%const OutPrecision OUT_CHARACTER_PRECIS    
%const OutPrecision OUT_STROKE_PRECIS       
%const OutPrecision OUT_TT_PRECIS           
%const OutPrecision OUT_DEVICE_PRECIS       
%const OutPrecision OUT_RASTER_PRECIS       
%const OutPrecision OUT_TT_ONLY_PRECIS      

%newtype : ClipPrecision : BYTE : (Eq)
{{%
instance BitSet ClipPrecision where
  toWord32   = \ (ClipPrecision x) -> x
  fromWord32 = ClipPrecision
%}}

# Or some of these together
%const ClipPrecision CLIP_DEFAULT_PRECIS     
%const ClipPrecision CLIP_CHARACTER_PRECIS   
%const ClipPrecision CLIP_STROKE_PRECIS      
%const ClipPrecision CLIP_MASK               
%const ClipPrecision CLIP_LH_ANGLES          
%const ClipPrecision CLIP_TT_ALWAYS          
%const ClipPrecision CLIP_EMBEDDED           

%newtype : FontQuality : BYTE : (Eq)

# Choose one of these
%const FontQuality DEFAULT_QUALITY     
%const FontQuality DRAFT_QUALITY       
%const FontQuality PROOF_QUALITY       


%newtype : PitchAndFamily : BYTE : (Eq)

# Choose one of these pitches
%const PitchAndFamily DEFAULT_PITCH       
%const PitchAndFamily FIXED_PITCH         
%const PitchAndFamily VARIABLE_PITCH      

# And one of these families
%const PitchAndFamily FF_DONTCARE         
%const PitchAndFamily FF_ROMAN            
%const PitchAndFamily FF_SWISS            
%const PitchAndFamily FF_MODERN           
%const PitchAndFamily FF_SCRIPT           
%const PitchAndFamily FF_DECORATIVE       

%const PitchAndFamily familyMask = 0xf0
%const PitchAndFamily pitchMask  = 0x0f

# A FaceName is a string no more that LF_FACESIZE in length
# (including null terminator).
%const Int LF_FACESIZE         # == 32
%sentinel_array : FaceName : CHAR : char : $0 = '\0' : ('\0' == $0) : LF_FACESIZE
{{%
type FaceName = [Char]
%}}

%struct : LOGFONT : LOGFONT $0
    INT	            : lfHeight
    INT	            : lfWidth
    INT	            : lfEscapement
    INT	            : lfOrientation
    FontWeight      : lfWeight
    Bool	    : lfItalic
    Bool	    : lfUnderline
    Bool	    : lfStrikeOut
    CharSet         : lfCharSet
    OutPrecision    : lfOutPrecision
    ClipPrecision   : lfClipPrecision
    FontQuality     : lfQuality
    PitchAndFamily  : lfPitchAndFamily
    FaceName        : lfFaceName

%ptr   : LPLOGFONT : LOGFONT
%maybe : MbLPLOGFONT : LPLOGFONT : $0 : (LPLOGFONT) 0

%errfun NullHANDLE CreateFont :: INT -> INT -> INT -> INT -> FontWeight -> Bool -> Bool -> Bool -> CharSet -> OutPrecision -> ClipPrecision -> FontQuality -> PitchAndFamily -> FaceName -> IO HFONT

%errfun NullHANDLE CreateFontIndirect :: LPLOGFONT -> IO HFONT

# ToDo: EnumFonts
# ToDo: EnumFontFamilies


################################################################
# Text formatting attributes
################################################################

################ Background/Foreground colour ################

%errfun BadColor  SetBkColor :: HDC -> COLORREF -> IO COLORREF
%errfun BadColor  GetBkColor :: HDC ->             IO COLORREF

%errfun BadColor  SetTextColor :: HDC -> COLORREF -> IO COLORREF
%errfun BadColor  GetTextColor :: HDC ->             IO COLORREF

################ Background mode ################

%newtype : BackgroundMode : INT : (Eq)

%const BackgroundMode TRANSPARENT     
%const BackgroundMode OPAQUE          

%errfun Zero SetBkMode  :: HDC -> BackgroundMode -> IO BackgroundMode
%errfun Zero GetBkMode  :: HDC ->                   IO BackgroundMode

################ Text Alignment ################

%newtype : TextAlignment : UINT : (Eq)
{{%
instance BitSet TextAlignment where
  toWord32   = \ (TextAlignment x) -> x
  fromWord32 = TextAlignment
%}}

# Choose (at most) one of these;
%const TextAlignment TA_NOUPDATECP 
%const TextAlignment TA_UPDATECP   
				   
# and (at most) one of these;	   
%const TextAlignment TA_LEFT       
%const TextAlignment TA_RIGHT      
%const TextAlignment TA_CENTER     
				   
# and (at most) one of these.	   
%const TextAlignment TA_TOP        
%const TextAlignment TA_BOTTOM     
%const TextAlignment TA_BASELINE   

# ToDo: the docs mention VTA_BASELINE and VTA_CENTER - but they're not
# in windows.h

%errfun GDI_ERROR SetTextAlign :: HDC -> TextAlignment -> IO TextAlignment
%errfun GDI_ERROR GetTextAlign :: HDC ->                  IO TextAlignment

%constructor CharSpacingFailed :: IOError

%error : MinInt     : INT $0      : (0x80000000 == $0)  : %constructor(CharSpacingFailed);

%errfun MinInt   SetTextCharacterExtra :: HDC -> INT -> IO INT
%errfun MinInt   GetTextCharacterExtra :: HDC ->        IO INT

################ Text justification ################

%errfun False getTextExtentPoint32 :: HDC -> String -> IO SIZE using $err = GetTextExtentPoint32($0,$1,$1_size,&$r0);

# ToDo: Change list of tab stops to a Maybe
# Result is (width,height)
%prim getTabbedTextExtent :: HDC -> String -> ListINT -> IO (WORD,WORD)
DWORD r = GetTabbedTextExtent($0,$1,$1_size,$2_size,$2);
%errreturn(Zero,r,LOWORD(r),HIWORD(r));

# ToDo: I can't express the necessary maybes correctly
# %prim getTextExtentExPoint :: HDC -> String -> MbINT -> IO (INT,INT,SIZE)
# INT  fit;
# INT dx[???];
# SIZE sz;
# Bool r = GetTextExtentExPoint($0,$1,$1_size,&fit,&dx,&sz);
# %errreturn(False,r,fit,dx,sz);


# Use "Nothing" as second argument at start/end of line to clear
# accumulated error value.
%errfun False SetTextJustification :: HDC -> MbINT -> INT -> IO ()


################ Character widths ################

# %prim getCharWidth32 :: HDC -> INT -> INT -> IO ListINT
# INT buffer_size = MAX(0,$2-$1);
# INT buffer[buffer_size];
# Bool r = GetCharWidth32($0,$1,$2,buffer);
# %errreturn(False,r,buffer);
# 
# %prim getCharWidthFloat :: HDC -> INT -> INT -> IO [FLOAT]
# INT   buffer_size = MAX(0,$2-$1);
# FLOAT buffer[buffer_size];
# Bool  r = GetCharWidthFloat($0,$1,$2,buffer);
# %errreturn(False,r,buffer);
# 
# %struct : ABC : ABC $0
#   INT  : abcA
#   UINT : abcB
#   INT  : abcC
# 
# %struct : ABCFLOAT : ABC $0
#   FLOAT : abcfA
#   FLOAT : abcfB
#   FLOAT : abcfC
# 
# %prim getCharABCWidths :: HDC -> INT -> INT -> IO [ABC]
# INT   buffer_size = MAX(0,$2-$1);
# ABC   buffer[buffer_size];
# Bool  r = GetCharABCWidths($0,$1,$2,buffer);
# %errreturn(False,r,buffer);
# 
# %prim getCharABCWidthsFloat :: HDC -> INT -> INT -> IO [ABCFLOAT]
# INT      buffer_size = MAX(0,$2-$1);
# ABCFLOAT buffer[buffer_size];
# Bool     r = GetCharABCWidthsFloat($0,$1,$2,buffer);
# %errreturn(False,r,buffer);


################ Text Metrics ################

%synonym : TechAndFamily : PitchAndFamily 

# See PitchAndFamily for Family-related constants

%const TechAndFamily techMask = 0x0f

%const TechAndFamily TMPF_FIXED_PITCH # Set for variable width fonts!!
%const TechAndFamily TMPF_VECTOR     
%const TechAndFamily TMPF_DEVICE     
%const TechAndFamily TMPF_TRUETYPE   

%struct : TEXTMETRIC : TEXTMETRIC $0
    INT            : tmHeight
    INT            : tmAscent
    INT            : tmDescent
    INT            : tmInternalLeading
    INT            : tmExternalLeading
    INT            : tmAveCharWidth
    INT            : tmMaxCharWidth
    FontWeight     : tmWeight
    Bool           : tmItalic
    Bool           : tmUnderlined
    Bool           : tmStruckOut
    Char           : tmFirstChar
    Char           : tmLastChar
    Char           : tmDefaultChar
    Char           : tmBreakChar
    TechAndFamily  : tmPitchAndFamily
    BYTE           : tmCharSet
    INT	           : tmOverhang
    INT	           : tmDigitizedAspectX
    INT	           : tmDigitizedAspectY

%errfun False GetTextMetrics :: HDC -> IO TEXTMETRIC using $err = GetTextMetrics($0,&$r0)

# ToDo: NEWTEXTMETRIC - for use with EnumFonts and TrueType fonts
# ToDo: GetTextFace
# ToDo: GetOutlineTextMetrics/ OUTLINETEXTMETRIC (Huge struct!!)

################ DeviceCaps ################

# ToDo: GetDeviceCaps
# Many different versions with different types plus lots of new types and constants

################ TextOut (and friends) ################

%errfun False TextOut :: HDC -> INT -> INT -> String -> IO () using $err = TextOut($0,$1,$2,$3,$3_size)

# ToDo: disnae work - doesn't set size to zero
# %maybe : MbListINT : ListINT : $0 : (INT*) 0

# ToDo: not clear from docs what the error condition should be!
# ToDo: tablist should have type MbListINT.
# returns (width,height)
%prim TabbedTextOut :: HDC -> INT -> INT -> String -> ListINT -> INT -> IO (WORD,WORD)
Bool r = TabbedTextOut($0,$1,$2,$3,$3_size,$4_size,$4,$5);
%errreturn(Zero,r,LOWORD(r),HIWORD(r));




# ToDo: ExtTextOut, TabbedTextOut


################ DrawText ################

%newtype : DrawTextFormat : UINT : (Eq)
{{%
instance BitSet DrawTextFormat where
  toWord32   = \ (DrawTextFormat x) -> x
  fromWord32 = DrawTextFormat
%}}

# ToDo: magic needed for tabstops

%const DrawTextFormat DT_TOP              
%const DrawTextFormat DT_LEFT             
%const DrawTextFormat DT_CENTER           
%const DrawTextFormat DT_RIGHT            
%const DrawTextFormat DT_VCENTER          
%const DrawTextFormat DT_BOTTOM           
%const DrawTextFormat DT_WORDBREAK        
%const DrawTextFormat DT_SINGLELINE       
%const DrawTextFormat DT_EXPANDTABS       
%const DrawTextFormat DT_TABSTOP          # Bits 15-8 specify tabsize
%const DrawTextFormat DT_NOCLIP           # Can't be used with DT_TABSTOP   
%const DrawTextFormat DT_EXTERNALLEADING  # Can't be used with DT_TABSTOP   
%const DrawTextFormat DT_CALCRECT         # Can't be used with DT_TABSTOP   
%const DrawTextFormat DT_NOPREFIX         # Can't be used with DT_TABSTOP   
%const DrawTextFormat DT_INTERNAL         # Can't be used with DT_TABSTOP

# ToDo: Provide version which takes CString argument??
# ToDo: The docs talk of success/failure but don't say how to detect failure.

%fun DrawText :: HDC -> String -> RECT -> DrawTextFormat -> IO INT using $r0 = DrawText($0,$1,$1_size,&$2,$3)


