################################################################
# MIDI API
#
# Copyright (c) The University of Nottingham and Yale University, 1994-1997.
# All rights reserved. See NOTICE for details and conditions of use etc...
# Hugs version 1.4, March 1997
#
# $RCSfile: WinMIDI.ss,v $
# $Revision: 1.4 $
# $Date: 1997/03/17 18:40:57 $
################################################################

{%
#include <mmsystem.h>
%}

%newtype : VERSION : WORD : (Eq, Ord, Show) : notypedef

################ manufacturer IDs ################

%newtype : Manufacturer : UINT : (Eq)

#I can see it in the .h file - but it's not coming through
#%const Manufacturer MM_MICROSOFT    # Microsoft Corp.


################ product IDs ################

%newtype : Product : UINT : (Eq)

#%const Product MM_MIDI_MAPPER          # MIDI Mapper 
#%const Product MM_WAVE_MAPPER          # Wave Mapper 
#
#%const Product MM_SNDBLST_MIDIOUT      # Sound Blaster MIDI output port 
#%const Product MM_SNDBLST_MIDIIN       # Sound Blaster MIDI input port  
#%const Product MM_SNDBLST_SYNTH        # Sound Blaster internal synthesizer 
#%const Product MM_SNDBLST_WAVEOUT      # Sound Blaster waveform output 
#%const Product MM_SNDBLST_WAVEIN       # Sound Blaster waveform input 
#
#%const Product MM_ADLIB                # Ad Lib-compatible synthesizer 
#
#%const Product MM_MPU401_MIDIOUT       # MPU401-compatible MIDI output port 
#%const Product MM_MPU401_MIDIIN        # MPU401-compatible MIDI input port 
#
#%const Product MM_PC_JOYSTICK          # Joystick adapter */


################ MIDI error return values ################

%const WindowsError MIDIERR_UNPREPARED    # header not prepared 
%const WindowsError MIDIERR_STILLPLAYING  # still something playing 
%const WindowsError MIDIERR_NOMAP         # no current map 
%const WindowsError MIDIERR_NOTREADY      # hardware is still busy 
%const WindowsError MIDIERR_NODEVICE      # port no longer connected 
%const WindowsError MIDIERR_INVALIDSETUP  # invalid setup 
%const WindowsError MIDIERR_LASTERROR     # last error in range 

################ Multimedia Result values ################

%newtype : MMRESULT : UINT : (Eq) : notypedef

%const MMRESULT	MMSYSERR_NOERROR      # no error 
%const MMRESULT	MMSYSERR_ERROR        # unspecified error 
%const MMRESULT	MMSYSERR_BADDEVICEID  # device ID out of range 
%const MMRESULT	MMSYSERR_NOTENABLED   # driver failed enable 
%const MMRESULT	MMSYSERR_ALLOCATED    # device already allocated 
%const MMRESULT	MMSYSERR_INVALHANDLE  # device handle is invalid 
%const MMRESULT	MMSYSERR_NODRIVER     # no device driver present 
%const MMRESULT	MMSYSERR_NOMEM        # memory allocation error 
%const MMRESULT	MMSYSERR_NOTSUPPORTED # function isn't supported 
%const MMRESULT	MMSYSERR_BADERRNUM    # error value out of range 
%const MMRESULT	MMSYSERR_INVALFLAG    # invalid flag passed
%const MMRESULT	MMSYSERR_INVALPARAM   # invalid parameter passed 
%const MMRESULT	MMSYSERR_HANDLEBUSY   # handle being used 
			      	      #   simultaneously on another 
			      	      #   thread (eg callback) 
%const MMRESULT	MMSYSERR_INVALIDALIAS # specified alias not found 
%const MMRESULT	MMSYSERR_BADDB        # bad registry database
%const MMRESULT	MMSYSERR_KEYNOTFOUND  # registry key not found 
%const MMRESULT	MMSYSERR_READERROR    # registry read error 
%const MMRESULT	MMSYSERR_WRITEERROR   # registry write error 
%const MMRESULT	MMSYSERR_DELETEERROR  # registry delete error 
%const MMRESULT	MMSYSERR_VALNOTFOUND  # registry value not found
%const MMRESULT	MMSYSERR_NODRIVERCB   # driver does not call DriverCallback 
%const MMRESULT	MMSYSERR_LASTERROR    # last error in range 



################ MIDI audio data types ################

%newtype : HMIDI    : HANDLE : () : notypedef
%newtype : HMIDIIN  : HANDLE : () : notypedef
%newtype : HMIDIOUT : HANDLE : () : notypedef

################ MIDI callback messages ################

%const WindowMessage MIM_OPEN        
%const WindowMessage MIM_CLOSE       
%const WindowMessage MIM_DATA        
%const WindowMessage MIM_LONGDATA    
%const WindowMessage MIM_ERROR       
%const WindowMessage MIM_LONGERROR   
%const WindowMessage MOM_OPEN        
%const WindowMessage MOM_CLOSE       
%const WindowMessage MOM_DONE        

################
# flags for wFlags parm of midiOutCachePatches(), midiOutCacheDrumPatches()
################

%newtype : MidiCachePolicy : UINT : (Eq)

%const MidiCachePolicy MIDI_CACHE_ALL      
%const MidiCachePolicy MIDI_CACHE_BESTFIT  
%const MidiCachePolicy MIDI_CACHE_QUERY    
%const MidiCachePolicy MIDI_UNCACHE        


################ flags for wTechnology field of MIDIOUTCAPS structure ####

%newtype : MidiTechnology : UINT : (Eq)

%const MidiTechnology MOD_MIDIPORT    #	output port
%const MidiTechnology MOD_SYNTH       #	generic internal synth 
%const MidiTechnology MOD_SQSYNTH     #	square wave internal synth 
%const MidiTechnology MOD_FMSYNTH     #	FM internal synth 
%const MidiTechnology MOD_MAPPER      #	MIDI mapper 


################ flags for dwSupport field of MIDIOUTCAPS structure ####

%newtype : MidiOutSupport : UINT : ()
{{%
instance BitSet MidiOutSupport where
  toWord32   = \ (MidiOutSupport x) -> x
  fromWord32 = MidiOutSupport
%}}


%const MidiOutSupport MIDICAPS_VOLUME   # supports volume control 
%const MidiOutSupport MIDICAPS_LRVOLUME # separate left-right volume control 
%const MidiOutSupport MIDICAPS_CACHE


################ MIDIOUTCAPS ################

%struct : MIDIOUTCAPS : MIDIOUTCAPS $0
  Manufacturer   : wMid           # manufacturer ID                      
  Product        : wPid	          # product ID                            
  VERSION        : vDriverVersion # version of the driver                
  String         : szPname	  # product name (NULL terminated string) 
  MidiTechnology : wTechnology    # type of device                       
  UINT           : wVoices	  # # of voices (internal synth only)     
  UINT           : wNotes	  # max # of notes (internal synth only)  
  UINT           : wChannelMask   # channels used (internal synth only)   
  MidiOutSupport : dwSupport	  # functionality supported by driver     

# ToDo: %ptr : LPMIDIOUTCAPS : MIDIOUTCAPS : MIDIOUTCAPS

################ MIDIINCAPS ################

%struct : MIDIINCAPS : MIDIINCAPS
    Manufacturer : wMid             # manufacturer ID 
    Product      : wPid             # product ID 
    VERSION      : vDriverVersion   # version of the driver 
    String       : szPname          # product name (NULL terminated string) 

# ToDo: %ptr : LPMIDIINCAPS : MIDIINCAPS : MIDIINCAPS

################ MIDIHDR ################

# Not supported yet (looks tricky)


################ MIDI device Id ################

%synonym : MidiDevice : UINT : (Eq,Enum)

# device ID for MIDI mapper
%const MidiDevice MIDIMAPPER     
%const MidiDevice MIDI_MAPPER    

# All other devices numbered from 0.. (use toEnum)


################################################################
# Midi functions
################################################################

%constructor MidiError :: MMRESULT -> IOError
%error : MMError : MMRESULT $0 : (MMSYSERR_NOERROR != $0) : %constructor(MidiError,$0);


# %type	: a 	: Cell 	$0	: $0		: $1=$0
# 
# %prim succIO :: a -> IO a
# %return($0);
# 
# %prim failIO :: IO ()
# %failWith(MMError,MMSYSERR_NOERROR);

%fun midiOutGetNumDevs :: IO UINT

%prim midiOutGetDevCaps :: MidiDevice -> IO MIDIOUTCAPS
MIDIOUTCAPS caps;
MMRESULT r = midiOutGetDevCaps($0,&caps,sizeof(MIDIOUTCAPS));
%errreturn(MMError,r,caps);

# Note: type of first arg was UINT in early versions of Windows
%prim midiOutGetVolume :: HMIDIOUT -> IO (WORD, WORD)
DWORD vol;
MMRESULT r = midiOutGetVolume($0,&vol);
%errreturn(MMError,r,vol&0xffff, vol>>16);

%prim midiOutSetVolume :: HMIDIOUT -> WORD -> WORD -> IO ()
MMRESULT r = midiOutSetVolume($0,$1 | ($2<<16));
%errreturn(MMError,r);

# %prim midiOutGetErrorText :: MMRESULT -> UINT -> IO (MMRESULT,String)
#

# This is causing a link error on windows 95
#%prim midiOutGetId :: HMIDIOUT -> IO (MidiDevice)
#MidiDevice dev;
#MMRESULT r = midiOutGetId($0,&dev);
#%errreturn(MMError,r,dev);


# This has a deliberately horrible name to encourage me to 
# replace it with something more general real soon now.

%prim midiOutOpenNoCB :: MidiDevice -> IO HMIDIOUT
HMIDIOUT hMidi;
MMRESULT r;
r = midiOutOpen(&hMidi, $0, (DWORD) NULL, 0, CALLBACK_NULL);
%errreturn(MMError,r,hMidi);

# %prim midiOutOpen :: MidiCallback -> MidiDevice -> IO (YHMIDIOUT)
# HMIDIOUT hMidi;
# MMRESULT r;
# DWORD callback;
# DWORD data;
# DWORD type;
# if () {
#   ... decode the callback ...
# }
# r = midiOutOpen(&hMidi, $1, callback, data, type);
# %errreturn(MMError,r,hMidi);


%errfun MMError midiOutClose :: HMIDIOUT -> IO ()

# UINT WINAPI midiOutPrepareHeader(HMIDIOUT hMidiOut,
#     MIDIHDR FAR* lpMidiOutHdr, UINT uSize);
# UINT WINAPI midiOutUnprepareHeader(HMIDIOUT hMidiOut,
#     MIDIHDR FAR* lpMidiOutHdr, UINT uSize);

%errfun MMError midiOutShortMsg :: HMIDIOUT -> DWORD -> IO ()

# ToDo: when is it safe to deallocate midiHdr?
# %prim midiOutLongMsg :: HMIDIOUT -> (LPMIDIHDR,UINT) -> IO ()
# MMRESULT r = midiOutLongMsg($0,$1);
# %return(r);

%errfun MMError midiOutReset :: HMIDIOUT -> IO ()

# UINT WINAPI midiOutCachePatches(HMIDIOUT hMidiOut,
#     UINT uBank, WORD FAR* lpwPatchArray, UINT uFlags);
# 
# UINT WINAPI midiOutCacheDrumPatches(HMIDIOUT hMidiOut,
#     UINT uPatch, WORD FAR* lpwKeyArray, UINT uFlags);

# ToDo: cleaner wrappers?
%errfun MMError midiOutMessage :: HMIDIOUT -> UINT -> DWORD -> DWORD -> IO ()

%fun midiInGetNumDevs :: IO ()

%prim midiInGetDevCaps :: MidiDevice -> IO MIDIINCAPS
MIDIINCAPS caps;
MMRESULT r = midiInGetDevCaps($0,&caps,sizeof(MIDIINCAPS));
%errreturn(MMError,r,caps);

# %prim midiOutGetErrorText :: MMRESULT -> UINT -> IO String
#

# UINT WINAPI midiInOpen(HMIDIIN FAR* lphMidiIn, UINT uDeviceID,
#     DWORD dwCallback, DWORD dwInstance, DWORD dwFlags);

%errfun MMError midiInClose :: HMIDIIN -> IO ()

# UINT WINAPI midiInPrepareHeader(HMIDIIN hMidiIn,
#     MIDIHDR FAR* lpMidiInHdr, UINT uSize);
# UINT WINAPI midiInUnprepareHeader(HMIDIIN hMidiIn,
#     MIDIHDR FAR* lpMidiInHdr, UINT uSize);
# UINT WINAPI midiInAddBuffer(HMIDIIN hMidiIn,
#     MIDIHDR FAR* lpMidiInHdr, UINT uSize);

%errfun MMError midiInStart :: HMIDIIN -> IO ()
%errfun MMError midiInStop  :: HMIDIIN -> IO ()
%errfun MMError midiInReset :: HMIDIIN -> IO ()

# This causes a link error on windows 95
# %prim midiInGetID :: HMIDIIN -> IO (MMRESULT, MidiDevice)
# MidiDevice dev;
# MMRESULT r = midiInGetId($0,&dev);
# %return(r,dev);

%errfun MMError midiInMessage :: HMIDIIN -> UINT -> DWORD -> DWORD -> IO ()


################################################################
# Delay loop hack
################################################################

%prim delay :: Int -> Int -> IO ()
Int i, j;
for (i = 0; i<$0; ++i) {
  for (j = 0; j<$1; ++j) {
  }
}
%return();


################################################################
# End of midi.ss
################################################################


