################################################################
# Menus
################################################################

%newtype : MenuName : LPCSTR : ()

%synonym : MenuID    : UINT
%maybe   : MbMenuName : MenuName : $0 : (MenuName) 0

# ToDo: there's no way to recover the space used in the string
%prim mkMenuName :: PString -> MenuName
%return($0);

%prim atomToMenuName :: ATOM -> MenuName
%return((MenuName)$0);

%constructor NoMenuItem :: IOError
%error : NoMenuItem : INT $0 : (-1 == $0) : %constructor(NoMenuItem);

%constructor NoMenuID :: IOError
%error : NoMenuID : INT $0 : (-1 == $0) : %constructor(NoMenuID);

%constructor BadMenu :: IOError
%error : BadMenu : INT $0 : (-1 == $0) : %constructor(BadMenu);

################################################################

%synonym : GMDIFlag : UINT

%const GMDIFlag GMDI_USEDISABLED   
%const GMDIFlag GMDI_GOINTOPOPUPS  

################################################################

%synonym : MenuFlag  : UINT

%const MenuFlag MF_BYCOMMAND
%const MenuFlag MF_BYPOSITION

################################################################

%newtype : MenuItem : UINT : ()

%const MenuItem	MF_INSERT           
%const MenuItem	MF_CHANGE           
%const MenuItem	MF_APPEND           
%const MenuItem	MF_DELETE           
%const MenuItem	MF_REMOVE           
%const MenuItem	MF_USECHECKBITMAPS  
%const MenuItem	MF_POPUP            
%const MenuItem	MF_SYSMENU          
%const MenuItem	MF_HELP             
%const MenuItem	MF_MOUSESELECT      
%const MenuItem	MF_END              # Obsolete -- only used by old RES files

################################################################

%synonym : MenuFlagT : MenuFlag

%const MenuFlagT MFT_STRING          
%const MenuFlagT MFT_BITMAP          
%const MenuFlagT MFT_MENUBARBREAK    
%const MenuFlagT MFT_MENUBREAK       
%const MenuFlagT MFT_OWNERDRAW       
%const MenuFlagT MFT_RADIOCHECK      
%const MenuFlagT MFT_SEPARATOR       
%const MenuFlagT MFT_RIGHTORDER      
%const MenuFlagT MFT_RIGHTJUSTIFY    

################################################################

%synonym : MenuState : MenuFlag

%const MenuState MFS_GRAYED          
%const MenuState MFS_DISABLED        # == MFS_GRAYED
%const MenuState MFS_CHECKED         
%const MenuState MFS_HILITE          
%const MenuState MFS_ENABLED         
%const MenuState MFS_UNCHECKED       
%const MenuState MFS_UNHILITE        
%const MenuState MFS_DEFAULT         

################################################################

%synonym : TrackMenuFlag : UINT

%const TrackMenuFlag TPM_LEFTBUTTON  
%const TrackMenuFlag TPM_RIGHTBUTTON 
%const TrackMenuFlag TPM_LEFTALIGN   
%const TrackMenuFlag TPM_CENTERALIGN 
%const TrackMenuFlag TPM_RIGHTALIGN  
%const TrackMenuFlag TPM_TOPALIGN    
%const TrackMenuFlag TPM_VCENTERALIGN
%const TrackMenuFlag TPM_BOTTOMALIGN 
%const TrackMenuFlag TPM_HORIZONTAL     # Horz alignment matters more 
%const TrackMenuFlag TPM_VERTICAL       # Vert alignment matters more
%const TrackMenuFlag TPM_NONOTIFY       # Don't send any notification msgs
%const TrackMenuFlag TPM_RETURNCMD   

################################################################

%newtype : SystemMenuCommand : UINT : (Eq)

%const SystemMenuCommand SC_SIZE         
%const SystemMenuCommand SC_MOVE         
%const SystemMenuCommand SC_MINIMIZE     
%const SystemMenuCommand SC_MAXIMIZE     
%const SystemMenuCommand SC_NEXTWINDOW   
%const SystemMenuCommand SC_PREVWINDOW   
%const SystemMenuCommand SC_CLOSE        
%const SystemMenuCommand SC_VSCROLL      
%const SystemMenuCommand SC_HSCROLL      
%const SystemMenuCommand SC_MOUSEMENU    
%const SystemMenuCommand SC_KEYMENU      
%const SystemMenuCommand SC_ARRANGE      
%const SystemMenuCommand SC_RESTORE      
%const SystemMenuCommand SC_TASKLIST     
%const SystemMenuCommand SC_SCREENSAVE   
%const SystemMenuCommand SC_HOTKEY       
%const SystemMenuCommand SC_DEFAULT      
%const SystemMenuCommand SC_MONITORPOWER 
%const SystemMenuCommand SC_CONTEXTHELP  
%const SystemMenuCommand SC_SEPARATOR    

# Obsolete names 
%const SystemMenuCommand SC_ICON         # == SC_MINIMIZE
%const SystemMenuCommand SC_ZOOM         # == SC_MAXIMIZE

################################################################

%fun IsMenu        :: HMENU -> IO Bool       	   
%fun GetSystemMenu :: HWND  -> Bool ->     IO MbHMENU
%fun GetMenu       :: HWND  ->             IO MbHMENU
%fun GetSubMenu    :: HMENU -> MenuItem -> IO MbHMENU

%errfun False      SetMenu            :: HWND -> HMENU -> IO ()

%errfun BadMenu    GetMenuItemCount   :: HMENU -> IO Int
%errfun NoMenuID   GetMenuItemID      :: HMENU -> MenuItem -> IO MenuID
# %errfun False    GetMenuItemInfo    :: HMENU -> MenuItem -> Bool -> IO MENUITEMINFO
%errfun BadMenu    GetMenuState       :: HMENU -> MenuItem -> MenuFlag -> IO MenuState
# %errfun ...      GetMenuString      :: HMENU -> MenuItem -> Bool -> IO String
# %errfun False    SetMenuItemInfo    :: HMENU -> MenuItem -> Bool -> MENUITEMINFO -> IO ()

%errfun NoMenuItem GetMenuDefaultItem :: HMENU -> Bool -> GMDIFlag -> IO MenuItem
%errfun False      SetMenuDefaultItem :: HMENU -> MenuItem -> Bool -> IO ()

%errfun BadMenu    CheckMenuItem      :: HMENU -> MenuItem -> MenuFlag -> IO DWORD
%errfun False      CheckMenuRadioItem :: HMENU -> MenuItem -> MenuItem -> MenuItem -> MenuFlag -> IO DWORD
%errfun BadMenu    EnableMenuItem     :: HMENU -> MenuItem -> MenuFlag -> IO MenuState
# %fun             GetMenuCheckMarkDimensions :: IO (UINT,UINT)
%fun               HiliteMenuItem     :: HWND  -> HMENU -> MenuItem -> MenuFlag -> IO Bool
%errfun False SetMenuItemBitmaps      :: HMENU -> MenuItem -> MenuFlag -> MbHBITMAP -> MbHBITMAP -> IO ()


# ToDo: if you use MF_POPUP flag, MenuID should be HMENU
# ToDo: if you use MF_BITMAP or MF_OWNERDRAW lpNewItem should be a bitmap 
#       or itemData
%errfun NullHANDLE CreateMenu      :: IO HMENU
%errfun NullHANDLE CreatePopupMenu :: IO HMENU
%errfun False DestroyMenu :: HMENU -> IO ()
%errfun False DeleteMenu  :: HMENU -> MenuItem -> MenuFlag -> IO ()
%errfun False RemoveMenu  :: HMENU -> MenuItem -> MenuFlag -> IO ()
%errfun False AppendMenu  :: HMENU -> MenuFlag -> MenuID -> String -> IO ()
%errfun False InsertMenu  :: HMENU -> MenuItem -> MenuFlag -> MenuID -> String -> IO ()
# %errfun False InsertMenuItem :: HMENU -> MenuItem -> Bool -> MENUITEMINFO -> IO ()
%errfun False ModifyMenu  :: HMENU -> MenuItem -> MenuFlag -> MenuID -> String -> IO ()
%errfun False DrawMenuBar :: HWND -> IO ()

%errfun False TrackPopupMenu :: HMENU -> TrackMenuFlag -> Int -> Int -> HWND -> RECT -> IO () using $err = TrackPopupMenu($0,$1,$2,$3,0,$4,&$5)

# Not yet:
# %errfun ... LoadMenuIndirect :: ... -> IO HMENU
# %errfun ... TrackPopupMenuEx :: ...

# Obsolete:


