################################################################
# Windows
#
# Copyright (c) The University of Nottingham and Yale University, 1994-1997.
# All rights reserved. See NOTICE for details and conditions of use etc...
# Hugs version 1.4, March 1997
#
# $RCSfile: Windows.ss,v $
# $Revision: 1.10 $
# $Date: 1997/03/24 04:47:12 $
################################################################

################ Handles ################

%include Handles.ss

################ 32 bit Handles ################

%newtype : HHOOK    : FarPtr : () : notypedef
%maybe   : MbHANDLE : HANDLE : $0 : (HANDLE) 0

################################################################
# Error Handling
################################################################

%newtype : WindowsError : DWORD : (Show,Eq)

# If we have a specific error number
%constructor WindowsError :: Int -> IOError

# If all we know is that something bad happened
# (ToDo: replace all uses of these errors with something more useful.)
%constructor AnonWindowsError :: IOError
%constructor TrueError     :: IOError
%constructor FalseError    :: IOError
%constructor FFError       :: IOError
%constructor ZeroError     :: IOError
%constructor NullHANDLE    :: IOError
%constructor InvalidHANDLE :: IOError
%constructor NullPtr       :: IOError
%constructor NullAtom      :: IOError
%constructor BadColor      :: IOError
%constructor GDIError      :: IOError
%constructor WinError      :: LONG -> IOError

# ToDo: it's not clear whether every function does set LastError.

%error : NullHANDLE : HANDLE $0 : !($0) : %constructor(NullHANDLE);
%error : NullPtr    : LPVOID $0 : !($0) : %constructor(NullPtr);
%error : INVALID_HANDLE_VALUE : HANDLE $0 : (INVALID_HANDLE_VALUE == $0) : %constructor(InvalidHANDLE);
%error : NullATOM   : ATOM $0   : !($0) : %constructor(NullAtom);
%error : NullPtr    : LPVOID $0 : !($0) : %constructor(NullPtr);
%error : True       : BOOL $0 : ($0)    : %constructor(TrueError);
%error : False      : BOOL $0 : !($0)   : %constructor(FalseError);
%error : Zero       : Int  $0 : !($0)   : %constructor(ZeroError);
%error : 0xFFFFFFFF : Int  $0 : (0xFFFFFFFF == $0) : %constructor(FFError);

%error : BadColor  : COLORREF $0 : (CLR_INVALID == $0)   : %constructor(BadColor);
%error : GDI_ERROR : UINT $0     : (GDI_ERROR == $0)     : %constructor(GDIError); 
%error : WinError  : LONG $0     : (ERROR_SUCCESS != $0) : %constructor(WinError,$0);

################################################################
# Points, Rects
################################################################

%struct : POINT : POINT $0
INT	: x
INT 	: y

%array : ListPOINT : POINT : POINT
{{%
type ListPOINT = [POINT]
%}}

%ptr : LPPOINT : POINT

%struct : RECT : RECT $0
    LONG : left
    LONG : top
    LONG : right
    LONG : bottom

%ptr : LPRECT : RECT

%maybe : MbLPRECT : LPRECT : $0	: (LPRECT) 0

%struct : SIZE : SIZE $0
    LONG : cx
    LONG : cy

#%ptr : LPSIZE : SIZE
#%maybe : MbLPSIZE : LPSIZE : $0	: (LPSIZE) 0

################################################################
# Resource Ids
################################################################

%newtype : Accelerator : LPCSTR : ()
%newtype : Bitmap      : LPCSTR : ()
%newtype : Cursor      : LPCSTR : ()
%newtype : Icon        : LPCSTR : ()
%newtype : Menu        : LPCSTR : ()

{{%
mkAccelerator :: PString -> Accelerator
mkAccelerator = Accelerator . toLPCSTR

mkBitmap :: PString -> Bitmap
mkBitmap = Bitmap . toLPCSTR

mkCursor :: PString -> Cursor
mkCursor = Cursor . toLPCSTR

mkIcon :: PString -> Icon
mkIcon = Icon . toLPCSTR

mkMenu :: PString -> Menu
mkMenu = Menu . toLPCSTR
%}}

%asm XS_MAKEINTRESOURCE :: WORD -> LPCSTR
WORD w;
LPCSTR r;
%getArg(0,w);
r = MAKEINTRESOURCE(w);
%return(r);

{{%
primitive intToAccelerator "XS_MAKEINTRESOURCE" :: Int -> Accelerator
primitive intToBitmap      "XS_MAKEINTRESOURCE" :: Int -> Bitmap
primitive intToCursor      "XS_MAKEINTRESOURCE" :: Int -> Cursor
primitive intToIcon        "XS_MAKEINTRESOURCE" :: Int -> Icon
primitive intToMenu        "XS_MAKEINTRESOURCE" :: Int -> Menu
%}}

# ToDo: can these be made pure?
%errfun NullHANDLE LoadAccelerators :: MbHINSTANCE -> Accelerator -> IO HACCEL
%errfun NullHANDLE LoadBitmap       :: MbHINSTANCE -> Bitmap      -> IO HBITMAP
%errfun NullHANDLE LoadCursor       :: MbHINSTANCE -> Cursor      -> IO HCURSOR
%errfun NullHANDLE LoadIcon         :: MbHINSTANCE -> Icon        -> IO HICON
%errfun NullHANDLE LoadMenu         :: MbHINSTANCE -> Menu        -> IO HMENU

# ToDo: LoadCursorFromFile
# ToDo: LoadKeyboardLayout
# ToDo: LoadMenuIndirect
# ToDo: LoadString

# ToDo: How to make LoadResource and FindResource typesafe?

%errfun NullHANDLE LoadLibrary :: String -> IO HINSTANCE
# ToDo: LoadLibraryEx
# ToDo: LoadModule

################################################################
# Standard cursor resource IDs
################################################################

# ToDo: the win32 docs list several other "standard cursors"
# (IDC_APPSTARTING, IDC_NO, IDC_SIZEALL, UDC_UPARROW) but they're
# not in windows.h.  What's the deal?

%const Cursor IDC_ARROW           
%const Cursor IDC_IBEAM           
%const Cursor IDC_WAIT            
%const Cursor IDC_CROSS           
%const Cursor IDC_UPARROW         
%const Cursor IDC_SIZE            
%const Cursor IDC_ICON            
%const Cursor IDC_SIZENWSE        
%const Cursor IDC_SIZENESW        
%const Cursor IDC_SIZEWE          
%const Cursor IDC_SIZENS          

################################################################
# Standard icon resource IDs
################################################################

%const Icon IDI_APPLICATION
%const Icon IDI_HAND
%const Icon IDI_QUESTION
%const Icon IDI_EXCLAMATION
%const Icon IDI_ASTERISK


################################################################
# MessageBoxes
################################################################

%newtype : MBStyle : UINT : ()
{{%
instance BitSet MBStyle where
  toWord32   = \ (MBStyle x) -> x
  fromWord32 = MBStyle
%}}


################ MBStyle ################

%const MBStyle MB_OK
%const MBStyle MB_OKCANCEL
%const MBStyle MB_ABORTRETRYIGNORE
%const MBStyle MB_YESNOCANCEL
%const MBStyle MB_YESNO
%const MBStyle MB_RETRYCANCEL
%const MBStyle MB_TYPEMASK         # all the above

%const MBStyle MB_ICONHAND
%const MBStyle MB_ICONQUESTION
%const MBStyle MB_ICONEXCLAMATION
%const MBStyle MB_ICONASTERISK
%const MBStyle MB_ICONINFORMATION  # MB_ICONASTERISK
%const MBStyle MB_ICONSTOP         # MB_ICONHAND

%const MBStyle MB_DEFBUTTON1
%const MBStyle MB_DEFBUTTON2
%const MBStyle MB_DEFBUTTON3
%const MBStyle MB_DEFMASK          # MB_DEFBUTTON1 | MB_DEFBUTTON2 | MB_DEFBUTTON3

%const MBStyle MB_APPLMODAL
%const MBStyle MB_SYSTEMMODAL
%const MBStyle MB_TASKMODAL

%const MBStyle MB_SETFOREGROUND
#%const MBStyle MB_DESKTOP_ONLY
%const MBStyle MB_NOFOCUS

################ MBStatus ################

%newtype : MBStatus : INT : (Eq)

%error : MBFail : MBStatus $0 : !($0) : %constructor(WindowsError,$0);

%const MBStatus IDABORT
%const MBStatus IDCANCEL
%const MBStatus IDIGNORE
%const MBStatus IDNO
%const MBStatus IDOK
%const MBStatus IDRETRY
%const MBStatus IDYES


# ToDo: it may make more sense to split the style argument into
# several components: icon, buttons, button defaults, modality, etc
# - but then we lose some of the (convenient??) defaulting behaviour.

%errfun MBFail MessageBox :: MbHWND -> String -> MbString -> MBStyle -> IO MBStatus using $r0 = MessageBox($0,$1,$2,$3)


################################################################
# Icons
################################################################


# ToDo: CreateIcon, CreateIconFromResource, CreateIconIndirect

%errfun NullHANDLE CopyIcon :: HICON -> IO HICON
%errfun False DrawIcon :: HDC -> Int -> Int -> HICON -> IO Bool

# ToDo: can I do this with weak pointers??
%errfun False DestroyIcon :: HICON -> IO Bool




################################################################
# Class styles
################################################################

%newtype : ClassStyle : UINT : ()
{{%
instance BitSet ClassStyle where
  toWord32   = \ (ClassStyle x) -> x
  fromWord32 = ClassStyle
%}}

# Use any combination of these
%const ClassStyle CS_VREDRAW          
%const ClassStyle CS_HREDRAW          

%const ClassStyle CS_OWNDC            
%const ClassStyle CS_CLASSDC          
%const ClassStyle CS_PARENTDC         
				      
%const ClassStyle CS_SAVEBITS         
				      
%const ClassStyle CS_DBLCLKS          
				      
%const ClassStyle CS_BYTEALIGNCLIENT  
%const ClassStyle CS_BYTEALIGNWINDOW  
				      
%const ClassStyle CS_NOCLOSE          
				      
%const ClassStyle CS_KEYCVTWINDOW     
%const ClassStyle CS_NOKEYCVT         
				      
%const ClassStyle CS_GLOBALCLASS      


################################################################
# System Colors
################################################################

%newtype : SystemColor : INT : (Eq)

%const SystemColor COLOR_SCROLLBAR           
%const SystemColor COLOR_BACKGROUND          
%const SystemColor COLOR_ACTIVECAPTION       
%const SystemColor COLOR_INACTIVECAPTION     
%const SystemColor COLOR_MENU                
%const SystemColor COLOR_WINDOW              
%const SystemColor COLOR_WINDOWFRAME         
%const SystemColor COLOR_MENUTEXT            
%const SystemColor COLOR_WINDOWTEXT          
%const SystemColor COLOR_CAPTIONTEXT         
%const SystemColor COLOR_ACTIVEBORDER        
%const SystemColor COLOR_INACTIVEBORDER      
%const SystemColor COLOR_APPWORKSPACE        
%const SystemColor COLOR_HIGHLIGHT           
%const SystemColor COLOR_HIGHLIGHTTEXT       
%const SystemColor COLOR_BTNFACE             
%const SystemColor COLOR_BTNSHADOW           
%const SystemColor COLOR_GRAYTEXT            
%const SystemColor COLOR_BTNTEXT 

# These last two aren't listed as valid colours in the WNDCLASS docs
%const SystemColor COLOR_INACTIVECAPTIONTEXT 
%const SystemColor COLOR_BTNHIGHLIGHT        


################################################################
# Window Messages
################################################################

%newtype : WindowMessage : UINT : (Eq)

%const WindowMessage WM_COMPACTING       # Low system memory notification message
%const WindowMessage WM_WININICHANGE     
%const WindowMessage WM_SYSCOLORCHANGE   

# Palette window messages
%const WindowMessage WM_QUERYNEWPALETTE  
%const WindowMessage WM_PALETTEISCHANGING
%const WindowMessage WM_PALETTECHANGED   

%const WindowMessage WM_FONTCHANGE       

%const WindowMessage WM_SPOOLERSTATUS    # Spooler status notification message

%const WindowMessage WM_DEVMODECHANGE    
%const WindowMessage WM_TIMECHANGE       

# Standard window messages 
# PenWindows specific messages
%const WindowMessage WM_PENWINFIRST      
%const WindowMessage WM_PENWINLAST       

# Coalescing messages 
#%const WindowMessage WM_COALESCE_FIRST   
#%const WindowMessage WM_COALESCE_LAST    

%const WindowMessage WM_POWER            

# Application termination
%const WindowMessage WM_QUERYENDSESSION  
%const WindowMessage WM_ENDSESSION       
%const WindowMessage WM_QUIT             
#%const WindowMessage WM_SYSTEMERROR      


%const WindowMessage WM_CREATE           
%const WindowMessage WM_NCCREATE         
%const WindowMessage WM_DESTROY          
%const WindowMessage WM_NCDESTROY        

%const WindowMessage WM_SHOWWINDOW       

%const WindowMessage WM_SETREDRAW        

%const WindowMessage WM_ENABLE           

%const WindowMessage WM_SETTEXT          
%const WindowMessage WM_GETTEXT          
%const WindowMessage WM_GETTEXTLENGTH    

%const WindowMessage WM_WINDOWPOSCHANGING
%const WindowMessage WM_WINDOWPOSCHANGED 

%const WindowMessage WM_MOVE             
%const WindowMessage WM_SIZE             

%const WindowMessage WM_QUERYOPEN        
%const WindowMessage WM_CLOSE            

%const WindowMessage WM_GETMINMAXINFO    

%const WindowMessage WM_PAINT            
%const WindowMessage WM_ERASEBKGND       
%const WindowMessage WM_ICONERASEBKGND   
			    
			    
%const WindowMessage WM_NCPAINT          
			    
%const WindowMessage WM_NCCALCSIZE       


# Drag-and-drop support

%const WindowMessage WM_QUERYDRAGICON    
%const WindowMessage WM_DROPFILES        
			    
%const WindowMessage WM_ACTIVATE         
%const WindowMessage WM_ACTIVATEAPP      
%const WindowMessage WM_NCACTIVATE       
			    
%const WindowMessage WM_SETFOCUS         
%const WindowMessage WM_KILLFOCUS        
			    
%const WindowMessage WM_KEYDOWN          
%const WindowMessage WM_KEYUP            
			    
%const WindowMessage WM_CHAR             
%const WindowMessage WM_DEADCHAR         
			    
%const WindowMessage WM_SYSKEYDOWN       
%const WindowMessage WM_SYSKEYUP         
			    
%const WindowMessage WM_SYSCHAR          
%const WindowMessage WM_SYSDEADCHAR      


# Keyboard message range
%const WindowMessage WM_KEYFIRST         
%const WindowMessage WM_KEYLAST          

# Mouse input messages 
%const WindowMessage WM_MOUSEMOVE        
%const WindowMessage WM_LBUTTONDOWN      
%const WindowMessage WM_LBUTTONUP        
%const WindowMessage WM_LBUTTONDBLCLK    
%const WindowMessage WM_RBUTTONDOWN      
%const WindowMessage WM_RBUTTONUP        
%const WindowMessage WM_RBUTTONDBLCLK    
%const WindowMessage WM_MBUTTONDOWN      
%const WindowMessage WM_MBUTTONUP        
%const WindowMessage WM_MBUTTONDBLCLK    

# Mouse input message range 
%const WindowMessage WM_MOUSEFIRST       
%const WindowMessage WM_MOUSELAST        

# Non-client mouse messages 
%const WindowMessage WM_NCMOUSEMOVE      
%const WindowMessage WM_NCLBUTTONDOWN    
%const WindowMessage WM_NCLBUTTONUP      
%const WindowMessage WM_NCLBUTTONDBLCLK  
%const WindowMessage WM_NCRBUTTONDOWN    
%const WindowMessage WM_NCRBUTTONUP      
%const WindowMessage WM_NCRBUTTONDBLCLK  
%const WindowMessage WM_NCMBUTTONDOWN    
%const WindowMessage WM_NCMBUTTONUP      
%const WindowMessage WM_NCMBUTTONDBLCLK  

# Mouse click activation support
%const WindowMessage WM_MOUSEACTIVATE    
			    
%const WindowMessage WM_CANCELMODE       
			    
%const WindowMessage WM_TIMER            
			    
# Menu messages
%const WindowMessage WM_INITMENU         
%const WindowMessage WM_INITMENUPOPUP    
			    
%const WindowMessage WM_MENUSELECT       
%const WindowMessage WM_MENUCHAR         

# Menu and control command messages
%const WindowMessage WM_COMMAND          
			    
%const WindowMessage WM_HSCROLL          
%const WindowMessage WM_VSCROLL          

# Clipboard command messages 
%const WindowMessage WM_CUT              
%const WindowMessage WM_COPY             
%const WindowMessage WM_PASTE            
%const WindowMessage WM_CLEAR            
%const WindowMessage WM_UNDO             

# Clipboard owner messages 
%const WindowMessage WM_RENDERFORMAT     
%const WindowMessage WM_RENDERALLFORMATS 
%const WindowMessage WM_DESTROYCLIPBOARD 

# Clipboard viewer messages
%const WindowMessage WM_DRAWCLIPBOARD    
%const WindowMessage WM_PAINTCLIPBOARD   
%const WindowMessage WM_SIZECLIPBOARD    
%const WindowMessage WM_VSCROLLCLIPBOARD 
%const WindowMessage WM_HSCROLLCLIPBOARD 
%const WindowMessage WM_ASKCBFORMATNAME  
%const WindowMessage WM_CHANGECBCHAIN    
			    
%const WindowMessage WM_SETCURSOR        

%const WindowMessage WM_SYSCOMMAND  

# MDI messages 
%const WindowMessage WM_MDICREATE        
%const WindowMessage WM_MDIDESTROY       
%const WindowMessage WM_MDIACTIVATE      
%const WindowMessage WM_MDIRESTORE       
%const WindowMessage WM_MDINEXT          
%const WindowMessage WM_MDIMAXIMIZE      
%const WindowMessage WM_MDITILE          
%const WindowMessage WM_MDICASCADE       
%const WindowMessage WM_MDIICONARRANGE   
%const WindowMessage WM_MDIGETACTIVE     
%const WindowMessage WM_MDISETMENU       
			    
%const WindowMessage WM_CHILDACTIVATE    

# Dialog notification messages 
%const WindowMessage WM_INITDIALOG       
%const WindowMessage WM_NEXTDLGCTL       
			    
%const WindowMessage WM_PARENTNOTIFY     
			    
%const WindowMessage WM_ENTERIDLE        
			    
%const WindowMessage WM_GETDLGCODE       
			    
#%const WindowMessage WM_CTLCOLOR         
			    
%const WindowMessage WM_SETFONT          
%const WindowMessage WM_GETFONT          
			    
%const WindowMessage WM_DRAWITEM         
			    
%const WindowMessage WM_MEASUREITEM      
%const WindowMessage WM_DELETEITEM       
%const WindowMessage WM_COMPAREITEM      

# Listbox notification messages */
%const WindowMessage WM_VKEYTOITEM       
%const WindowMessage WM_CHARTOITEM       

%const WindowMessage WM_QUEUESYNC        

%const WindowMessage WM_COMMNOTIFY       


################################################################
# Virtual keys
################################################################

%newtype : VKey : UINT : (Eq)

%const VKey VK_LBUTTON       
%const VKey VK_RBUTTON       
%const VKey VK_CANCEL        
%const VKey VK_MBUTTON
%const VKey VK_BACK       
%const VKey VK_TAB
%const VKey VK_CLEAR
%const VKey VK_RETURN

%const VKey VK_SHIFT
%const VKey VK_CONTROL       
%const VKey VK_MENU          
%const VKey VK_PAUSE         
%const VKey VK_CAPITAL       

%const VKey VK_ESCAPE        

%const VKey VK_SPACE         
%const VKey VK_PRIOR         
%const VKey VK_NEXT          
%const VKey VK_END           
%const VKey VK_HOME          
%const VKey VK_LEFT          
%const VKey VK_UP            
%const VKey VK_RIGHT         
%const VKey VK_DOWN          
%const VKey VK_SELECT        
%const VKey VK_PRINT         
%const VKey VK_EXECUTE       
%const VKey VK_SNAPSHOT      
%const VKey VK_INSERT        
%const VKey VK_DELETE        
%const VKey VK_HELP          

# VK_0 thru VK_9 are the same as ASCII '0' thru '9' (0x30 - 0x39)
# VK_A thru VK_Z are the same as ASCII 'A' thru 'Z' (0x41 - 0x5A)

%const VKey VK_LWIN          
%const VKey VK_RWIN          
%const VKey VK_APPS          

%const VKey VK_NUMPAD0       
%const VKey VK_NUMPAD1       
%const VKey VK_NUMPAD2       
%const VKey VK_NUMPAD3       
%const VKey VK_NUMPAD4       
%const VKey VK_NUMPAD5       
%const VKey VK_NUMPAD6       
%const VKey VK_NUMPAD7       
%const VKey VK_NUMPAD8       
%const VKey VK_NUMPAD9       
%const VKey VK_MULTIPLY      
%const VKey VK_ADD           
%const VKey VK_SEPARATOR     
%const VKey VK_SUBTRACT      
%const VKey VK_DECIMAL       
%const VKey VK_DIVIDE        
%const VKey VK_F1            
%const VKey VK_F2            
%const VKey VK_F3            
%const VKey VK_F4            
%const VKey VK_F5            
%const VKey VK_F6            
%const VKey VK_F7            
%const VKey VK_F8            
%const VKey VK_F9            
%const VKey VK_F10           
%const VKey VK_F11           
%const VKey VK_F12           
%const VKey VK_F13           
%const VKey VK_F14           
%const VKey VK_F15           
%const VKey VK_F16           
%const VKey VK_F17           
%const VKey VK_F18           
%const VKey VK_F19           
%const VKey VK_F20           
%const VKey VK_F21           
%const VKey VK_F22           
%const VKey VK_F23           
%const VKey VK_F24           

%const VKey VK_NUMLOCK
%const VKey VK_SCROLL

################################################################
# Window Creation
################################################################


%newtype : WindowStyle : DWORD : ()
{{%
instance BitSet WindowStyle where
  toWord32   = \ (WindowStyle x) -> x
  fromWord32 = WindowStyle
%}}

# Basic window types
%const WindowStyle WS_OVERLAPPED       
%const WindowStyle WS_POPUP            
%const WindowStyle WS_CHILD            
			    
# Clipping styles 	    
%const WindowStyle WS_CLIPSIBLINGS     
%const WindowStyle WS_CLIPCHILDREN     
			    
# Generic window states	    
%const WindowStyle WS_VISIBLE          
%const WindowStyle WS_DISABLED         
			    
# Main window states	    
%const WindowStyle WS_MINIMIZE         
%const WindowStyle WS_MAXIMIZE         
			    
# Main window styles	    
%const WindowStyle WS_CAPTION          # WS_BORDER | WS_DLGFRAME
%const WindowStyle WS_BORDER           
%const WindowStyle WS_DLGFRAME         
%const WindowStyle WS_VSCROLL          
%const WindowStyle WS_HSCROLL          
%const WindowStyle WS_SYSMENU          
%const WindowStyle WS_THICKFRAME       
%const WindowStyle WS_MINIMIZEBOX      
%const WindowStyle WS_MAXIMIZEBOX      
			    
# Control window styles	    
%const WindowStyle WS_GROUP            
%const WindowStyle WS_TABSTOP          

# Common Window Styles
%const WindowStyle WS_OVERLAPPEDWINDOW # (WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX)
%const WindowStyle WS_POPUPWINDOW      # (WS_POPUP | WS_BORDER | WS_SYSMENU)
%const WindowStyle WS_CHILDWINDOW      # (WS_CHILD)

# Extended Window Styles - should probably be a separate type!
%const WindowStyle WS_EX_DLGMODALFRAME  
%const WindowStyle WS_EX_NOPARENTNOTIFY 

%const WindowStyle WS_EX_TOPMOST        
%const WindowStyle WS_EX_ACCEPTFILES    
%const WindowStyle WS_EX_TRANSPARENT    

# Obsolete style names
%const WindowStyle WS_TILED            # WS_OVERLAPPED
%const WindowStyle WS_ICONIC           # WS_MINIMIZE
%const WindowStyle WS_SIZEBOX          # WS_THICKFRAME
%const WindowStyle WS_TILEDWINDOW      # WS_OVERLAPPEDWINDOW


################################################################
# Class Names
################################################################

%newtype : ClassName : LPCSTR : ()

%prim mkClassName :: PString -> ClassName	
%return($0);

%prim atomToClassName :: ATOM -> ClassName
%return((ClassName)$0);


################################################################
# WindowProcs
################################################################

# Only used for creating plain windows.
# Special versions exist for creating BUTTONs, COMBOBOXes, EDIT windows,
# LISTBOXes, MDICLIENTs, SCROLLBARs and STATIC windows.

%synonym : Pos : Int
%maybe   : MbPos : Pos : (CW_USEDEFAULT != $0) : CW_USEDEFAULT

%fun DefWindowProc :: MbHWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT

%stableptr : WindowClosure
{%
typedef Int WindowClosure;
%}
{{%
-- Note: these are only called when hwnd is non-null - so first arg
-- has type HWND instead of Maybe HWND.
type WindowClosure = HWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT

{- Note: the final parameter of createWindow is the "callback routine".
   In standard Windows programming, this is specified when the window
   class is registered - but it's much more convenient to specify it
   when the window is created.
-}
%}}

# ToDo: I couldn't figure out the type of the final parameter to CreateWindow
# ToDo: The Win32 docs say the HINSTANCE should be a HANDLE - but I don't
#       believe them.
%prim createWindow :: ClassName -> String -> WindowStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HINSTANCE -> WindowClosure -> IO HWND
MbHWND r = CreateWindow($0,$1,$2,$3,$4,$5,$6,$7,$8,$9,NULL);
if (!r) {%failWith(NullHANDLE);}
SetWindowLong(r, GWL_USERDATA, (LONG) $10);
%return(r);

%prim createWindowEx :: WindowStyle -> ClassName -> String -> WindowStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HINSTANCE -> WindowClosure -> IO HWND
MbHWND r = CreateWindowEx($0,$1,$2,$3,$4,$5,$6,$7,$8,$9,$10,NULL);
if (!r) {%failWith(NullHANDLE);}
SetWindowLong(r, GWL_USERDATA, (LONG) $11);
%return(r);

{%
/* Note that we use a generic WindowProc.  All this does is fetch the 
 * "USERDATA" field from the window and (assuming it to be a stable
 * pointer to a Haskell callback routine) enter it.
 */

LRESULT CALLBACK genericWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    if (hwnd) {
	Int wndprocptr = GetWindowLong(hwnd,GWL_USERDATA);
	if (wndprocptr) {
	    hugsStablePtr(wndprocptr);
	    hugsINT((Int)hwnd);   hugsRAP;
	    hugsINT((Int)msg);    hugsRAP;
	    hugsINT((Int)wParam); hugsRAP;
	    hugsINT((Int)lParam); hugsRAP;
	    return hugsEvalInt(hugsRunIO(hugs_pop()));
	}
    }
    return DefWindowProc(hwnd, msg, wParam, lParam);
}
%}

%errfun False DestroyWindow :: HWND -> IO ()
%errfun False BringWindowToTop :: HWND -> IO ()

# %errfun False AdjustWindowRectEx :: RECT -> WindowStyle -> Bool -> WindowStyleEx -> IO RECT using $err = AdjustWindowRectEx(&$0, $1, $2, $3) -> ($0)

%errfun False AdjustWindowRect :: RECT -> WindowStyle -> Bool -> IO RECT using $err = AdjustWindowRect(&$0, $1, $2) -> ($0)

# Note: I've omitted the following fields:
# WNDPROC     	lpfnWndProc
# INT         	cbClsExtra 
# INT         	cbWndExtra 

# To avoid confusion with NULL, WNDCLASS requires you to add 1 to a SystemColor
# (which can be NULL)
%prim mkMbHBRUSH :: SystemColor -> MbHBRUSH
%return((HBRUSH)($0+1));

%struct : WNDCLASS : WNDCLASS $0
ClassStyle     	: style
HINSTANCE   	: hInstance
MbHICON       	: hIcon
MbHCURSOR     	: hCursor
MbHBRUSH      	: hbrBackground
MbLPCSTR      	: lpszMenuName
ClassName      	: lpszClassName

%maybe : MbATOM : ATOM : $0 : 0

{{%
primitive registerClass "XS_RegisterClass" :: WNDCLASS -> IO ATOM
%}}

%asm XS_RegisterClass :: WNDCLASS -> IO ATOM
ATOM r;
WNDCLASS *wndclass = malloc(sizeof(WNDCLASS));
if (!wndclass) {%failWith(HeapExhaustion);}
%getArg(0,*wndclass);
/* fill in omitted fields */
wndclass->lpfnWndProc = genericWndProc;
wndclass->cbClsExtra  = 0;
wndclass->cbWndExtra  = 0;
r = RegisterClass(wndclass);
%errreturn(NullATOM,r,r);    

# Note: don't forget to destroy all windows of this class first!
%errfun False UnregisterClass :: ClassName -> HINSTANCE -> IO ()

################################################################
# Button Creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : ButtonStyle : WindowStyle

# choose one of these
%const ButtonStyle BS_PUSHBUTTON       
%const ButtonStyle BS_DEFPUSHBUTTON    
%const ButtonStyle BS_CHECKBOX         
%const ButtonStyle BS_AUTOCHECKBOX     
%const ButtonStyle BS_RADIOBUTTON      
%const ButtonStyle BS_3STATE           
%const ButtonStyle BS_AUTO3STATE       
%const ButtonStyle BS_GROUPBOX         
%const ButtonStyle BS_AUTORADIOBUTTON  
%const ButtonStyle BS_OWNERDRAW        

# modifier for radio buttons and check boxes
%const ButtonStyle BS_LEFTTEXT         

# obsolete - use BS_OWNERDRAW instead
%const ButtonStyle BS_USERBUTTON       

%prim createButton :: String -> WindowStyle -> ButtonStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HANDLE -> IO HWND
MbHWND r = CreateWindow("BUTTON", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);


################################################################
# Combobox Creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : ComboBoxStyle : WindowStyle

# Choose one of these
%const ComboBoxStyle CBS_SIMPLE            
%const ComboBoxStyle CBS_DROPDOWN          
%const ComboBoxStyle CBS_DROPDOWNLIST      
					   
# and or it with combinations of these	   
%const ComboBoxStyle CBS_OWNERDRAWFIXED    
%const ComboBoxStyle CBS_OWNERDRAWVARIABLE 
%const ComboBoxStyle CBS_AUTOHSCROLL       
%const ComboBoxStyle CBS_OEMCONVERT        
%const ComboBoxStyle CBS_SORT              
%const ComboBoxStyle CBS_HASSTRINGS        
%const ComboBoxStyle CBS_NOINTEGRALHEIGHT  
%const ComboBoxStyle CBS_DISABLENOSCROLL   

%prim createComboBox :: String -> WindowStyle -> ComboBoxStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
HWND r = CreateWindow("COMBOBOX", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);



################################################################
# Edit window creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : EditStyle : WindowStyle

# Edit window Styles

# Any combination of the following
%const EditStyle ES_LEFT             
%const EditStyle ES_CENTER           
%const EditStyle ES_RIGHT            
%const EditStyle ES_MULTILINE        
%const EditStyle ES_UPPERCASE        
%const EditStyle ES_LOWERCASE        
%const EditStyle ES_PASSWORD         
%const EditStyle ES_AUTOVSCROLL      
%const EditStyle ES_AUTOHSCROLL      
%const EditStyle ES_NOHIDESEL        
%const EditStyle ES_OEMCONVERT       
%const EditStyle ES_READONLY         
%const EditStyle ES_WANTRETURN       


%prim createEditWindow :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
HWND r = CreateWindow("EDIT", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);



################################################################
# List box creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : ListBoxStyle : WindowStyle

# List box Styles

# Any combination of the following
%const ListBoxStyle LBS_NOTIFY            
%const ListBoxStyle LBS_SORT              
%const ListBoxStyle LBS_NOREDRAW          
%const ListBoxStyle LBS_MULTIPLESEL       
%const ListBoxStyle LBS_OWNERDRAWFIXED    
%const ListBoxStyle LBS_OWNERDRAWVARIABLE 
%const ListBoxStyle LBS_HASSTRINGS        
%const ListBoxStyle LBS_USETABSTOPS       
%const ListBoxStyle LBS_NOINTEGRALHEIGHT  
%const ListBoxStyle LBS_MULTICOLUMN       
%const ListBoxStyle LBS_WANTKEYBOARDINPUT 
%const ListBoxStyle LBS_EXTENDEDSEL       
%const ListBoxStyle LBS_DISABLENOSCROLL   

%const ListBoxStyle LBS_STANDARD          # (LBS_NOTIFY | LBS_SORT | WS_VSCROLL | WS_BORDER)

%prim createListbox :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
HWND r = CreateWindow("LISTBOX", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);




################################################################
# Scrollbar creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : ScrollbarStyle : WindowStyle


################ Horizontal scrollbar styles ################

# Specify this:
%const ScrollbarStyle SBS_HORZ                    

# Combine with one of these (causes height to be ignored)
%const ScrollbarStyle SBS_TOPALIGN                
%const ScrollbarStyle SBS_BOTTOMALIGN             

################ Vertical scrollbar styles ################

# Specify this:
%const ScrollbarStyle SBS_VERT                    

# Combine with one of these (causes width to be ignored)
%const ScrollbarStyle SBS_LEFTALIGN               
%const ScrollbarStyle SBS_RIGHTALIGN              

################ Sizebox styles ################

# Specify this:
%const ScrollbarStyle SBS_SIZEBOX                 

# Combine with one of these
%const ScrollbarStyle SBS_SIZEBOXTOPLEFTALIGN     
%const ScrollbarStyle SBS_SIZEBOXBOTTOMRIGHTALIGN 


%prim createScrollbar :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
HWND r = CreateWindow("SCROLLBAR", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);



%synonym : ScrollbarType : WindowStyle

%const ScrollbarType SB_HORZ
%const ScrollbarType SB_VERT
%const ScrollbarType SB_CTL
%const ScrollbarType SB_BOTH

%synonym : ScrollbarCommand : Int

%const ScrollbarCommand SB_LINEUP
%const ScrollbarCommand SB_LINELEFT
%const ScrollbarCommand SB_LINEDOWN
%const ScrollbarCommand SB_LINERIGHT
%const ScrollbarCommand SB_PAGEUP
%const ScrollbarCommand SB_PAGELEFT
%const ScrollbarCommand SB_PAGEDOWN
%const ScrollbarCommand SB_PAGERIGHT
%const ScrollbarCommand SB_THUMBPOSITION
%const ScrollbarCommand SB_THUMBTRACK
%const ScrollbarCommand SB_TOP
%const ScrollbarCommand SB_LEFT
%const ScrollbarCommand SB_BOTTOM
%const ScrollbarCommand SB_RIGHT
%const ScrollbarCommand SB_ENDSCROLL


%fun ShowScrollBar :: HWND -> ScrollbarType -> Bool -> IO ()

%errfun False   SetScrollRange :: HWND -> ScrollbarType -> Int  -> Int -> Bool -> IO ()

%prim  getScrollPos   :: HWND -> ScrollbarType -> IO Int
int i = GetScrollPos((HWND)$0,SB_HORZ);
%return(i);

%prim setScrollPos   :: HWND -> ScrollbarType -> Int -> Bool -> IO ()
int i = SetScrollPos((HWND)$0,(unsigned)$1,(int)$2,(BOOL)$3);
%return();

################################################################
# Static window creation
################################################################

# Not abstract - so that we can use WindowStyle operators on them
%synonym : StaticControlStyle : WindowStyle

# Static Control Styles
# Choose exactly one of these
%const StaticControlStyle SS_LEFT             
%const StaticControlStyle SS_CENTER           
%const StaticControlStyle SS_RIGHT            
%const StaticControlStyle SS_ICON       # window name interpreted as icon name
%const StaticControlStyle SS_BLACKRECT        
%const StaticControlStyle SS_GRAYRECT         
%const StaticControlStyle SS_WHITERECT        
%const StaticControlStyle SS_BLACKFRAME       
%const StaticControlStyle SS_GRAYFRAME        
%const StaticControlStyle SS_WHITEFRAME       
%const StaticControlStyle SS_SIMPLE           
%const StaticControlStyle SS_LEFTNOWORDWRAP   
%const StaticControlStyle SS_NOPREFIX         


%prim createStaticWindow :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
HWND r = CreateWindow("STATIC", $0, $1|$2, $3,$4,$5,$6, $7, $8, $9, NULL);
%errreturn(NullHANDLE,r,r);


################ PaintStruct ################

# There are 3 other slots - but we're not supposed to modify them
%struct : PAINTSTRUCT : PAINTSTRUCT $0
HDC         : hdc
Bool        : fErase
RECT        : rcPaint

%ptr : LPPAINTSTRUCT : PAINTSTRUCT

%errfun NullHANDLE BeginPaint     :: HWND   -> LPPAINTSTRUCT -> IO HDC
%fun               EndPaint       :: HWND   -> LPPAINTSTRUCT -> IO ()
%errfun False      GetClientRect  :: HWND   -> IO RECT using $err = GetClientRect($0,&$r0)
%errfun False      InvalidateRect :: MbHWND -> MbLPRECT -> Bool -> IO ()

################################################################
# ShowWindow control values
################################################################

%newtype : ShowWindowControl : INT : (Eq)

%const ShowWindowControl SW_HIDE             
%const ShowWindowControl SW_SHOWNORMAL       
%const ShowWindowControl SW_NORMAL           
%const ShowWindowControl SW_SHOWMINIMIZED    
%const ShowWindowControl SW_SHOWMAXIMIZED    
%const ShowWindowControl SW_MAXIMIZE         
%const ShowWindowControl SW_SHOWNOACTIVATE   
%const ShowWindowControl SW_SHOW             
%const ShowWindowControl SW_MINIMIZE         
%const ShowWindowControl SW_SHOWMINNOACTIVE  
%const ShowWindowControl SW_SHOWNA           
%const ShowWindowControl SW_RESTORE          

# Obsolete ShowWindow() command names
%const ShowWindowControl HIDE_WINDOW         
%const ShowWindowControl SHOW_OPENWINDOW     
%const ShowWindowControl SHOW_ICONWINDOW     
%const ShowWindowControl SHOW_FULLSCREEN     
%const ShowWindowControl SHOW_OPENNOACTIVATE 

%fun ShowWindow   :: HWND -> ShowWindowControl -> IO Bool
%errfun False UpdateWindow :: HWND -> IO ()


################################################################
# Program parameters
################################################################

%errfun NullHANDLE GetModuleHandle :: MbString -> IO HMODULE

%newtype : StdHandleId : DWORD : (Eq)

%const StdHandleId STD_INPUT_HANDLE
%const StdHandleId STD_OUTPUT_HANDLE
%const StdHandleId STD_ERROR_HANDLE

%errfun INVALID_HANDLE_VALUE GetStdHandle :: StdHandleId -> IO HANDLE


################################################################
# Timers
################################################################

# ToDo: should this be abstract?
%synonym : TimerId : UINT

# ToDo: support the other two forms of timer initialisation

# Cause WM_TIMER events to be sent to window callback
%prim setWinTimer :: HWND -> TimerId -> UINT -> IO TimerId
TimerId r = SetTimer($0, $1, $2, NULL);
%errreturn(Zero,r,r);

%errfun False KillTimer :: MbHWND -> TimerId -> IO ()

# For documentation purposes:
%synonym : MilliSeconds : DWORD

%fun timeGetTime :: IO MilliSeconds

################################################################
# Error Functions
################################################################


%synonym : Duration : DWORD

%maybe : MbDuration : Duration : (-1 == $0) : -1
%errfun False Beep :: DWORD -> MbDuration -> IO ()


%maybe : MbBeep : MBStyle : (0xFFFFFFFF == $0) : 0xFFFFFFFF
%errfun False MessageBeep :: MbBeep -> IO ()

%fun FlashWindow :: HWND -> Bool -> IO Bool

# ToDo: GetLastError
# ToDo: SetLastError
# ToDo: SetLastErrorEx

%prim ExitWindows :: IO ()
Bool r = ExitWindows(0,0);
%errreturn(False,r);

%newtype : ExitOption : UINT : (Eq)

%const ExitOption EWX_FORCE
%const ExitOption EWX_LOGOFF
%const ExitOption EWX_POWEROFF
%const ExitOption EWX_REBOOT
%const ExitOption EWX_SHUTDOWN

%prim exitWindowsEx :: ExitOption -> IO ()
Bool r = ExitWindowsEx($0,0);
%errreturn(False,r);

################################################################
# Cursor position
################################################################

%errfun False GetCursorPos :: IO POINT using $err = GetCursorPos(&$r0)

%prim SetCursorPos :: POINT -> IO ()
Bool r = SetCursorPos($0.x,$0.y);
%errreturn(False,r);

%errfun False ClipCursor :: RECT -> IO () using $err = ClipCursor(&$0)
%errfun False GetClipCursor :: IO RECT using $err = GetClipCursor(&$r0)


################################################################
# Caret position
################################################################

%errfun False GetCaretPos :: IO POINT using $err = GetCaretPos(&$r0)

%prim SetCaretPos :: POINT -> IO ()
Bool r = SetCaretPos($0.x,$0.y);
%errreturn(False,r);

%errfun False HideCaret    :: HWND -> IO ()
%errfun False ShowCaret    :: HWND -> IO ()
%errfun False CreateCaret  :: HWND -> HBITMAP -> MbINT -> MbINT -> IO ()
%errfun False DestroyCaret :: IO ()

################################################################
#
################################################################

%prim ScreenToClient :: HWND -> POINT -> IO POINT
Bool r = ScreenToClient($0,&$1);
%errreturn(False,r,$1);

%prim ClientToScreen :: HWND -> POINT -> IO POINT
Bool r = ClientToScreen($0,&$1);
%errreturn(False,r,$1);


################################################################
# Event loop
################################################################

%struct : MSG : MSG $0
HWND        : hwnd
UINT        : message
WPARAM      : wParam
LPARAM      : lParam
DWORD	    : time
POINT       : pt

%ptr : LPMSG : MSG

%constructor GetMsgError :: IOError
%error : GetMsgError : BOOL $0 : (-1 == $0) : %constructor(GetMsgError);

%errfun GetMsgError GetMessage :: LPMSG -> MbHWND -> IO Bool using $r0 = GetMessage($0, $1, 0, 0)

%fun PeekMessage :: LPMSG -> MbHWND -> IO Bool using $r0 = PeekMessage($0, $1, 0, 0, PM_REMOVE)

%fun    TranslateMessage :: LPMSG -> IO Bool
%fun    DispatchMessage  :: LPMSG -> IO LONG

%var : hugsQuitFlag : Bool

{{%
eventLoop :: HWND -> IO ()
eventLoop hwnd = do
  set_hugsQuitFlag False
  lpmsg <- alloc
  while (map not get_hugsQuitFlag) (do
    getMessage       lpmsg (Just hwnd)
    translateMessage lpmsg
    dispatchMessage  lpmsg
    )
  free lpmsg
 where
  while :: IO Bool -> IO a -> IO ()
  while p s = do
    c <- p
    if c then s >> while p s else return ()
%}}

# ToDo: no way to send HWND_BROADCAST

%errfun False PostMessage :: MbHWND -> WindowMessage -> WPARAM -> LPARAM -> IO ()

%fun PostQuitMessage :: Int -> IO ()

# ToDo: DWORD -> ThreadID??
%errfun False PostThreadMessage :: DWORD -> WindowMessage -> WPARAM -> LPARAM -> IO ()

%fun SendMessage :: HWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT
%fun InSendMessage :: IO Bool


%errfun NullHANDLE SetFocus :: HWND -> IO HWND

%const LRESULT MA_NOACTIVATE
%const LRESULT MA_NOACTIVATEANDEAT
%const LRESULT MA_ACTIVATE
%const LRESULT MA_ACTIVATEANDEAT

%fun eqHWND :: HWND -> HWND -> Bool using $r0 = ($0 == $1)


################################################################
# Accessing the system's capabilities
################################################################

%newtype : SMSetting : UINT : (Eq)

%fun GetSystemMetrics :: SMSetting -> IO Int

#
# Some of these are 95/NT specific - check your local listings.
#
%const SMSetting SM_ARRANGE
%const SMSetting SM_CLEANBOOT
%const SMSetting SM_CMETRICS
%const SMSetting SM_CMOUSEBUTTONS
%const SMSetting SM_CXBORDER
%const SMSetting SM_CYBORDER
%const SMSetting SM_CXCURSOR
%const SMSetting SM_CYCURSOR
%const SMSetting SM_CXDLGFRAME
%const SMSetting SM_CYDLGFRAME
%const SMSetting SM_CXDOUBLECLK
%const SMSetting SM_CYDOUBLECLK
%const SMSetting SM_CXDRAG
%const SMSetting SM_CYDRAG
%const SMSetting SM_CXEDGE
%const SMSetting SM_CYEDGE
%const SMSetting SM_CXFRAME
%const SMSetting SM_CYFRAME
%const SMSetting SM_CXFULLSCREEN
%const SMSetting SM_CYFULLSCREEN
%const SMSetting SM_CXHSCROLL
%const SMSetting SM_CYVSCROLL
%const SMSetting SM_CXHTHUMB
%const SMSetting SM_CXICON
%const SMSetting SM_CYICON
%const SMSetting SM_CXICONSPACING
%const SMSetting SM_CYICONSPACING
%const SMSetting SM_CXMAXIMIZED
%const SMSetting SM_CYMAXIMIZED
%const SMSetting SM_CXMAXTRACK
%const SMSetting SM_CYMAXTRACK
%const SMSetting SM_CXMENUCHECK
%const SMSetting SM_CYMENUCHECK
%const SMSetting SM_CXMENUSIZE
%const SMSetting SM_CYMENUSIZE
%const SMSetting SM_CXMIN
%const SMSetting SM_CYMIN
%const SMSetting SM_CXMINIMIZED
%const SMSetting SM_CYMINIMIZED
%const SMSetting SM_CXMINSPACING
%const SMSetting SM_CYMINSPACING
%const SMSetting SM_CXMINTRACK
%const SMSetting SM_CYMINTRACK
%const SMSetting SM_CXSCREEN
%const SMSetting SM_CYSCREEN
%const SMSetting SM_CXSIZE
%const SMSetting SM_CYSIZE
%const SMSetting SM_CXSIZEFRAME
%const SMSetting SM_CYSIZEFRAME
%const SMSetting SM_CXSMICON
%const SMSetting SM_CYSMICON
%const SMSetting SM_CXSMSIZE
%const SMSetting SM_CYSMSIZE
%const SMSetting SM_CXVSCROLL
%const SMSetting SM_CYHSCROLL
%const SMSetting SM_CYVTHUMB
%const SMSetting SM_CYCAPTION
%const SMSetting SM_CYKANJIWINDOW
%const SMSetting SM_CYMENU
%const SMSetting SM_CYSMCAPTION
%const SMSetting SM_DBCSENABLED
%const SMSetting SM_DEBUG
%const SMSetting SM_MENUDROPALIGNMENT
%const SMSetting SM_MIDEASTENABLED
%const SMSetting SM_MOUSEPRESENT
%const SMSetting SM_NETWORK
%const SMSetting SM_PENWINDOWS
%const SMSetting SM_SECURE
%const SMSetting SM_SHOWSOUNDS
%const SMSetting SM_SLOWMACHINE
%const SMSetting SM_SWAPBUTTON

################################################################
# End of windows.ss
################################################################
