################################################################
# Standard type conversions for Hugs plugins
#
# Copyright (c) Mark P Jones 1991-1996.   All rights reserved.
# See NOTICE for details and conditions of use etc...
# Hugs version 1.3, August 1996
#
# $RCSfile: StdTypes.ss,v $
# $Revision: 1.4 $
# $Date: 1997/03/17 18:40:35 $
################################################################

%type 	: Int 		: Int 	$0 	: hugsINT($0);	: hugs->eval($0); $1=hugs_whnfInt
%type 	: Char		: Char 	$0	: hugsCHAR($0);	: hugs->eval($0); $1=hugs->CharOf(hugs_whnfHead)
%type 	: Float		: Float	$0	: hugsFLOAT($0);: hugs->eval($0); $1=hugs_whnfFloat
%type 	: Double	: Double $0	: hugsFLOAT($0);: hugs->eval($0); $1=hugs_whnfFloat
%type 	: Bool 		: Bool 	$0 	: if ($0) {hugsTRUE;} else {hugsFALSE;} : hugs->eval($0); $1=(hugs_whnfHead==hugs->nameTrue)

# Note: We define a new type CHAR rather than using Char
# because Char is a synonym for Int - making sizeofChar come out wrong.

%type 	: CHAR		: char 	$0	: hugsCHAR($0);	: hugs->eval($0); $1=hugs->CharOf(hugs_whnfHead)

{%
typedef char  CHAR;
%}
{{%
type CHAR = Char
%}}

#For just now, we'll continue to allocate strings on the heap - but
# we'll try to gradually move these allocations onto the stack
#%sentinel_array : String : CHAR : char : $0 = '\0' : ('\0' == $0)
%sentinel_array : String : CHAR : char : $0 = '\0' : ('\0' == $0) : persistent

%sentinel_array : PString : CHAR : char : $0 = '\0' : ('\0' == $0) : persistent
{{%
type PString = String -- "persistent" string needs to be explicitly freed
%}}

################################################################
# Pointer types
################################################################

# This little wrapper function should be the only place where a 
# dubious int<->pointer conversion occurs.
#
# Q (SOF): a candidate for inclusion in rheap.c + dllexport it?
# A (ADR): yes - once we get the STG stuff in place
  
{{%
-- Note: we do not consider NULL to be a valid pointer value.
-- Anything that requires a NULL pointer of type "T" should be
-- modified to require a "Maybe T" and an appropriate "Maybe"
-- declaration defined.
newtype Ptr = Ptr Int deriving (Eq)

unPtr :: Ptr -> Int
unPtr (Ptr i) = i

plusPtr  :: Ptr -> Int -> Ptr
plusPtr  p x = Ptr (unPtr p + x)

minusPtr :: Ptr -> Ptr -> Int
minusPtr p q = unPtr p - unPtr q
%}}

{%
typedef void* Ptr;

static Void local mkPtr Args((Ptr));
static Void local mkPtr(n) 
Ptr n; {
    hugs_push(hugs->makeInt((Int) n));
}

#define hugs_whnfPtr ((void*) hugs_whnfInt)
%}

%type : Ptr : Ptr $0 : mkPtr((Ptr)$0); : hugs->eval($0); $1=(Ptr)hugs_whnfPtr

################################################################
# The type synonym hack
################################################################

{{%
-- This is used in restricted type synonyms where there are no 
-- Haskell functions which need to "look inside the representation".
-- It overcomes a (normally reasonable) restriction that the type be
-- "open" in a non-empty list of variables.
rtsDummy = error "rtsDummy"
%}}


################################################################
# Types of particular sizes
################################################################

# ToDo: pay more attention to signed vs unsigned types
# and type sizes.

# ToDo: think hard about whether Int8 and friends should
# be abstract, should be members of Num, etc.

# Note: Haskell standard only requires 29 bits but all compilers
# provide 32 bits.

%synonym : Word8 : Int

%synonym : Word16 : Int

# This was causing lots of type warnings - so I macro-expanded and
# hand-patched the result.
# %synonym : Word32 : Int
{{%
type Word32 = Int
%}}
{%
typedef unsigned long Word32;
%}
%type : Word32 : Word32 $0 : %pack(Int,$0);% : %unpack(Int,$0,$1);%

# This was causing lots of type warnings - so I macro-expanded and
# hand-patched the result.
# %synonym : Word32 : Int
{{%
type Word = Int
%}}
{%
typedef unsigned int Word;
%}
%type : Word : Word $0 : %pack(Int,$0);% : %unpack(Int,$0,$1);%

%synonym : Int8 : Int

%synonym : Int16 : Int

%synonym : Int32 : Int

# NB: This hack can't be expected to work for genuinely large Ints.
%synonym : Int64 : Int

# NB: This hack can't be expected to work for genuinely large Ints.
%synonym : Word64 : Int

{%
static Cell hugsRunIO Args((Cell));
static Cell hugsRunIO(x)
Cell x; {
    StackPtr old_sp   = hugs_sp;     
    Cell     result   = NIL;                              
    Cell     temp     = NIL;
    hugs_push(hugs->nameIORun);                             
    hugs_push(x);                                           
    hugsRAP;                  
    temp = hugs->evalWithNoError(hugs_pop());
    hugs_assert(isNull(temp));
    if (hugs_whnfHead == hugs->nameRight) {
	result = hugs_pop();
    } else { /* Called "exit" */
	hugs->evalFails(old_sp);
    }
    hugs_assert(hugs_sp == old_sp); 
    return result;
}    

static Int hugsEvalInt Args((Cell));
static Int hugsEvalInt(x)
Cell x; {
    StackPtr old_sp = hugs_sp;                                   
    Cell     temp = hugs->evalWithNoError(x);                            
    hugs_assert(isNull(temp));
#if 0 
    hugs_assert(isInt(whnfHead)); /* Foo - gets confused by CAFs */
#endif
    hugs_assert(hugs_sp == old_sp); 
    return hugs_whnfInt;                        
}    

static Bool hugsEvalBool Args((Cell));
static Bool hugsEvalBool(x)
Cell x; {
    StackPtr old_sp = hugs_sp;  
    Cell     temp = hugs->evalWithNoError(x);                            
    hugs_assert(isNull(temp));
    hugs_assert(hugs_whnfHead == hugs->nameTrue
                || hugs_whnfHead == hugs->nameFalse
                ); 
    hugs_assert(hugs_sp == old_sp); 
    return hugs_whnfInt;                        
}    
%}