module Win32 where
import Prelude hiding (readFile, writeFile)
type CHAR = Char
type PString = String -- "persistent" string needs to be explicitly freed
-- Note: we do not consider NULL to be a valid pointer value.
-- Anything that requires a NULL pointer of type "T" should be
-- modified to require a "Maybe T" and an appropriate "Maybe"
-- declaration defined.
newtype Ptr = Ptr Int deriving (Eq)

unPtr :: Ptr -> Int
unPtr (Ptr i) = i

plusPtr  :: Ptr -> Int -> Ptr
plusPtr  p x = Ptr (unPtr p + x)

minusPtr :: Ptr -> Ptr -> Int
minusPtr p q = unPtr p - unPtr q
-- This is used in restricted type synonyms where there are no
-- Haskell functions which need to "look inside the representation".
-- It overcomes a (normally reasonable) restriction that the type be
-- "open" in a non-empty list of variables.
rtsDummy = error "rtsDummy"
type Word8 = Int
type Word16 = Int
type Word32 = Int
type Word = Int
type Int8 = Int
type Int16 = Int
type Int32 = Int
type Int64 = Int
type Word64 = Int
class Pointer a where
  toPtr   :: a -> Ptr
  fromPtr :: Ptr -> a
  alloc   :: IO a          -- never returns NULL ptr, may raise error.
  plus    :: a -> Int -> a
  minus   :: a -> a -> Int

free :: Pointer a => a -> IO ()
free p = freePtr (toPtr p)

cast :: (Pointer a, Pointer b) => a -> b
cast = fromPtr . toPtr

malloc :: Pointer a => Int -> IO a
malloc sz = primMalloc sz >>= \ p -> return (fromPtr p)

instance Pointer Ptr where
  toPtr   = id
  fromPtr = id
  alloc   = primMalloc 0
  plus    = plusPtr
  minus   = minusPtr
primitive isHeapExhaustion "XStest_isHeapExhaustion" :: IOError -> Bool
primitive freePtr "XS_freePtr" :: Ptr -> IO ()
primitive primMalloc "XS_primMalloc" :: Int -> IO Ptr
class BitSet a where
 toWord32 :: a -> Word32
 fromWord32 :: Word32 -> a

-- really want a Word32 instance - but that's a synonym for Int so...
instance BitSet Int where
 toWord32   = id
 fromWord32 = id

orb, andb :: BitSet a => a -> a -> a
orb x y = fromWord32 (orWord32 (toWord32 x) (toWord32 y))
andb x y = fromWord32 (andWord32 (toWord32 x) (toWord32 y))

notb :: BitSet a => a -> a
notb x = fromWord32 (notWord32 (toWord32 x))

emptyb, fullb :: BitSet a => a
emptyb = fromWord32 emptyWord32
fullb  = fromWord32 fullWord32

{- Sigh, I ought to know better by now... not legal Haskell
instance BitSet a => Eq a where
  x == y = toWord32 x == toWord32 y
-}

eqb :: BitSet a => a -> a -> Bool
x `eqb` y =  toWord32 x `eqWord32` toWord32 y

isEmptyb :: BitSet a => a -> Bool
isEmptyb x = x `eqb` emptyb

elemb :: BitSet a => Int -> a -> Bool
x `elemb` y = x `elemWord32` toWord32 y

shiftl, shiftr :: BitSet a => a -> Int -> a
shiftl x y = fromWord32 (shiftlWord32 (toWord32 x) y)
shiftr x y = fromWord32 (shiftrWord32 (toWord32 x) y)

orbs, andbs :: BitSet a => [a] -> a
orbs  = foldl orb  emptyb
andbs = foldl andb fullb
primitive orWord32 "XS_orWord32" :: Word32 -> Word32 -> Word32
primitive andWord32 "XS_andWord32" :: Word32 -> Word32 -> Word32
primitive notWord32 "XS_notWord32" :: Word32 -> Word32
emptyWord32 :: Word32
emptyWord32 = consts_Word32 0
fullWord32 :: Word32
fullWord32 = consts_Word32 1
primitive eqWord32 "XS_eqWord32" :: Word32 -> Word32 -> Bool
primitive elemWord32 "XS_elemWord32" :: Int -> Word32 -> Bool
primitive shiftlWord32 "XS_shiftlWord32" :: Word32 -> Int -> Word32
primitive shiftrWord32 "XS_shiftrWord32" :: Word32 -> Int -> Word32
type FLOAT = Float
type BYTE = Word8
type USHORT = Word16
type UINT = Word32
type INT = Int32
type WORD = Word16
type DWORD = Word32
type LONG = Int32
type ATOM = UINT
type WPARAM = UINT
type LPARAM = LONG
type LRESULT = LONG
type LPVOID = Ptr
type LPBYTE = Ptr
type MbString = Maybe String
type MbINT = Maybe INT
type MbFLOAT = Maybe FLOAT
type MbLPVOID = Maybe LPVOID
type MbDWORD = Maybe DWORD
type ListINT   = [INT]
type ListFLOAT = [FLOAT]
newtype NearPtr = NearPtr Ptr deriving (Eq)
newtype FarPtr  = FarPtr  Ptr deriving (Eq)
type LPCHAR = Ptr in
  toLPCHAR   :: Ptr -> LPCHAR,
  fromLPCHAR :: LPCHAR -> Ptr

toLPCHAR   x = x 
fromLPCHAR x = x

instance Pointer LPCHAR where
  toPtr     = fromLPCHAR
  fromPtr   = toLPCHAR
  alloc     = malloc sizeofLPCHAR
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPCHAR))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPCHAR

primitive getLPCHAR        "XS_get_LPCHAR"    :: LPCHAR -> IO CHAR
primitive setLPCHAR        "XS_set_LPCHAR"    :: LPCHAR -> CHAR -> IO ()
sizeofLPCHAR :: Int
sizeofLPCHAR = consts_Int 0
newtype LPCSTR = LPCSTR Ptr
type MbLPCSTR = Maybe LPCSTR
primitive toLPCSTR "XS_toLPCSTR" :: String -> LPCSTR
primitive fromLPCSTR "XS_fromLPCSTR" :: LPCSTR -> IO String
newtype HACCEL = HACCEL NearPtr deriving (Eq)
newtype HBITMAP = HBITMAP NearPtr deriving (Eq)
newtype HBRUSH = HBRUSH NearPtr deriving (Eq)
newtype HCURSOR = HCURSOR NearPtr deriving (Eq)
newtype HDC = HDC NearPtr deriving (Eq)
newtype HDRVR = HDRVR NearPtr deriving (Eq)
newtype HDWP = HDWP NearPtr deriving (Eq)
newtype HFONT = HFONT NearPtr deriving (Eq)
newtype HGDIOBJ = HGDIOBJ NearPtr deriving (Eq)
newtype HMENU = HMENU NearPtr deriving (Eq)
newtype HMETAFILE = HMETAFILE NearPtr deriving (Eq)
newtype HPALETTE = HPALETTE NearPtr deriving (Eq)
newtype HPEN = HPEN NearPtr deriving (Eq)
newtype HRGN = HRGN NearPtr deriving (Eq)
newtype HRSRC = HRSRC NearPtr deriving (Eq)
newtype HSTR = HSTR NearPtr deriving (Eq)
newtype HTASK = HTASK NearPtr deriving (Eq)
newtype HWND = HWND NearPtr deriving (Eq)
newtype HICON = HICON NearPtr deriving (Eq)
newtype HINSTANCE = HINSTANCE NearPtr deriving (Eq)
newtype HANDLE = HANDLE NearPtr deriving (Eq)
type HMODULE = HINSTANCE
type MbHDC = Maybe HDC
type MbHWND = Maybe HWND
type MbHMENU = Maybe HMENU
type MbHINSTANCE = Maybe HINSTANCE
type MbHMODULE = Maybe HMODULE
type MbHFONT = Maybe HFONT
type MbHPEN = Maybe HPEN
type MbHPALETTE = Maybe HPALETTE
type MbHICON = Maybe HICON
type MbHCURSOR = Maybe HCURSOR
type MbHBRUSH = Maybe HBRUSH
type MbHBITMAP = Maybe HBITMAP
type MbHRGN = Maybe HRGN
newtype HHOOK = HHOOK FarPtr deriving ()
type MbHANDLE = Maybe HANDLE
newtype WindowsError = WindowsError DWORD deriving (Show,Eq)
primitive isWindowsError "XStest_isWindowsError" :: IOError -> Maybe (Int )
primitive isAnonWindowsError "XStest_isAnonWindowsError" :: IOError -> Bool
primitive isTrueError "XStest_isTrueError" :: IOError -> Bool
primitive isFalseError "XStest_isFalseError" :: IOError -> Bool
primitive isFFError "XStest_isFFError" :: IOError -> Bool
primitive isZeroError "XStest_isZeroError" :: IOError -> Bool
primitive isNullHANDLE "XStest_isNullHANDLE" :: IOError -> Bool
primitive isInvalidHANDLE "XStest_isInvalidHANDLE" :: IOError -> Bool
primitive isNullPtr "XStest_isNullPtr" :: IOError -> Bool
primitive isNullAtom "XStest_isNullAtom" :: IOError -> Bool
primitive isBadColor "XStest_isBadColor" :: IOError -> Bool
primitive isGDIError "XStest_isGDIError" :: IOError -> Bool
primitive isWinError "XStest_isWinError" :: IOError -> Maybe (LONG )
type POINT = (INT,INT)
type ListPOINT = [POINT]
type LPPOINT = Ptr in
  toLPPOINT   :: Ptr -> LPPOINT,
  fromLPPOINT :: LPPOINT -> Ptr

toLPPOINT   x = x 
fromLPPOINT x = x

instance Pointer LPPOINT where
  toPtr     = fromLPPOINT
  fromPtr   = toLPPOINT
  alloc     = malloc sizeofLPPOINT
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPPOINT))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPPOINT

primitive getLPPOINT        "XS_get_LPPOINT"    :: LPPOINT -> IO POINT
primitive setLPPOINT        "XS_set_LPPOINT"    :: LPPOINT -> POINT -> IO ()
sizeofLPPOINT :: Int
sizeofLPPOINT = consts_Int 1
type RECT = (LONG,LONG,LONG,LONG)
type LPRECT = Ptr in
  toLPRECT   :: Ptr -> LPRECT,
  fromLPRECT :: LPRECT -> Ptr

toLPRECT   x = x 
fromLPRECT x = x

instance Pointer LPRECT where
  toPtr     = fromLPRECT
  fromPtr   = toLPRECT
  alloc     = malloc sizeofLPRECT
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPRECT))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPRECT

primitive getLPRECT        "XS_get_LPRECT"    :: LPRECT -> IO RECT
primitive setLPRECT        "XS_set_LPRECT"    :: LPRECT -> RECT -> IO ()
sizeofLPRECT :: Int
sizeofLPRECT = consts_Int 2
type MbLPRECT = Maybe LPRECT
type SIZE = (LONG,LONG)
newtype Accelerator = Accelerator LPCSTR deriving ()
newtype Bitmap = Bitmap LPCSTR deriving ()
newtype Cursor = Cursor LPCSTR deriving ()
newtype Icon = Icon LPCSTR deriving ()
newtype Menu = Menu LPCSTR deriving ()
mkAccelerator :: PString -> Accelerator
mkAccelerator = Accelerator . toLPCSTR

mkBitmap :: PString -> Bitmap
mkBitmap = Bitmap . toLPCSTR

mkCursor :: PString -> Cursor
mkCursor = Cursor . toLPCSTR

mkIcon :: PString -> Icon
mkIcon = Icon . toLPCSTR

mkMenu :: PString -> Menu
mkMenu = Menu . toLPCSTR
primitive intToAccelerator "XS_MAKEINTRESOURCE" :: Int -> Accelerator
primitive intToBitmap      "XS_MAKEINTRESOURCE" :: Int -> Bitmap
primitive intToCursor      "XS_MAKEINTRESOURCE" :: Int -> Cursor
primitive intToIcon        "XS_MAKEINTRESOURCE" :: Int -> Icon
primitive intToMenu        "XS_MAKEINTRESOURCE" :: Int -> Menu
primitive loadAccelerators "XS_LoadAccelerators" :: MbHINSTANCE -> Accelerator -> IO HACCEL
primitive loadBitmap "XS_LoadBitmap" :: MbHINSTANCE -> Bitmap      -> IO HBITMAP
primitive loadCursor "XS_LoadCursor" :: MbHINSTANCE -> Cursor      -> IO HCURSOR
primitive loadIcon "XS_LoadIcon" :: MbHINSTANCE -> Icon        -> IO HICON
primitive loadMenu "XS_LoadMenu" :: MbHINSTANCE -> Menu        -> IO HMENU
primitive loadLibrary "XS_LoadLibrary" :: String -> IO HINSTANCE
iDC_ARROW :: Cursor
iDC_ARROW = consts_Cursor 0
iDC_IBEAM :: Cursor
iDC_IBEAM = consts_Cursor 1
iDC_WAIT :: Cursor
iDC_WAIT = consts_Cursor 2
iDC_CROSS :: Cursor
iDC_CROSS = consts_Cursor 3
iDC_UPARROW :: Cursor
iDC_UPARROW = consts_Cursor 4
iDC_SIZE :: Cursor
iDC_SIZE = consts_Cursor 5
iDC_ICON :: Cursor
iDC_ICON = consts_Cursor 6
iDC_SIZENWSE :: Cursor
iDC_SIZENWSE = consts_Cursor 7
iDC_SIZENESW :: Cursor
iDC_SIZENESW = consts_Cursor 8
iDC_SIZEWE :: Cursor
iDC_SIZEWE = consts_Cursor 9
iDC_SIZENS :: Cursor
iDC_SIZENS = consts_Cursor 10
iDI_APPLICATION :: Icon
iDI_APPLICATION = consts_Icon 0
iDI_HAND :: Icon
iDI_HAND = consts_Icon 1
iDI_QUESTION :: Icon
iDI_QUESTION = consts_Icon 2
iDI_EXCLAMATION :: Icon
iDI_EXCLAMATION = consts_Icon 3
iDI_ASTERISK :: Icon
iDI_ASTERISK = consts_Icon 4
newtype MBStyle = MBStyle UINT deriving ()
instance BitSet MBStyle where
  toWord32   = \ (MBStyle x) -> x
  fromWord32 = MBStyle
mB_OK :: MBStyle
mB_OK = consts_MBStyle 0
mB_OKCANCEL :: MBStyle
mB_OKCANCEL = consts_MBStyle 1
mB_ABORTRETRYIGNORE :: MBStyle
mB_ABORTRETRYIGNORE = consts_MBStyle 2
mB_YESNOCANCEL :: MBStyle
mB_YESNOCANCEL = consts_MBStyle 3
mB_YESNO :: MBStyle
mB_YESNO = consts_MBStyle 4
mB_RETRYCANCEL :: MBStyle
mB_RETRYCANCEL = consts_MBStyle 5
mB_TYPEMASK :: MBStyle
mB_TYPEMASK = consts_MBStyle 6
mB_ICONHAND :: MBStyle
mB_ICONHAND = consts_MBStyle 7
mB_ICONQUESTION :: MBStyle
mB_ICONQUESTION = consts_MBStyle 8
mB_ICONEXCLAMATION :: MBStyle
mB_ICONEXCLAMATION = consts_MBStyle 9
mB_ICONASTERISK :: MBStyle
mB_ICONASTERISK = consts_MBStyle 10
mB_ICONINFORMATION :: MBStyle
mB_ICONINFORMATION = consts_MBStyle 11
mB_ICONSTOP :: MBStyle
mB_ICONSTOP = consts_MBStyle 12
mB_DEFBUTTON1 :: MBStyle
mB_DEFBUTTON1 = consts_MBStyle 13
mB_DEFBUTTON2 :: MBStyle
mB_DEFBUTTON2 = consts_MBStyle 14
mB_DEFBUTTON3 :: MBStyle
mB_DEFBUTTON3 = consts_MBStyle 15
mB_DEFMASK :: MBStyle
mB_DEFMASK = consts_MBStyle 16
mB_APPLMODAL :: MBStyle
mB_APPLMODAL = consts_MBStyle 17
mB_SYSTEMMODAL :: MBStyle
mB_SYSTEMMODAL = consts_MBStyle 18
mB_TASKMODAL :: MBStyle
mB_TASKMODAL = consts_MBStyle 19
mB_SETFOREGROUND :: MBStyle
mB_SETFOREGROUND = consts_MBStyle 20
mB_NOFOCUS :: MBStyle
mB_NOFOCUS = consts_MBStyle 21
newtype MBStatus = MBStatus INT deriving (Eq)
iDABORT :: MBStatus
iDABORT = consts_MBStatus 0
iDCANCEL :: MBStatus
iDCANCEL = consts_MBStatus 1
iDIGNORE :: MBStatus
iDIGNORE = consts_MBStatus 2
iDNO :: MBStatus
iDNO = consts_MBStatus 3
iDOK :: MBStatus
iDOK = consts_MBStatus 4
iDRETRY :: MBStatus
iDRETRY = consts_MBStatus 5
iDYES :: MBStatus
iDYES = consts_MBStatus 6
primitive messageBox "XS_MessageBox" :: MbHWND -> String -> MbString -> MBStyle -> IO MBStatus 
primitive copyIcon "XS_CopyIcon" :: HICON -> IO HICON
primitive drawIcon "XS_DrawIcon" :: HDC -> Int -> Int -> HICON -> IO Bool
primitive destroyIcon "XS_DestroyIcon" :: HICON -> IO Bool
newtype ClassStyle = ClassStyle UINT deriving ()
instance BitSet ClassStyle where
  toWord32   = \ (ClassStyle x) -> x
  fromWord32 = ClassStyle
cS_VREDRAW :: ClassStyle
cS_VREDRAW = consts_ClassStyle 0
cS_HREDRAW :: ClassStyle
cS_HREDRAW = consts_ClassStyle 1
cS_OWNDC :: ClassStyle
cS_OWNDC = consts_ClassStyle 2
cS_CLASSDC :: ClassStyle
cS_CLASSDC = consts_ClassStyle 3
cS_PARENTDC :: ClassStyle
cS_PARENTDC = consts_ClassStyle 4
cS_SAVEBITS :: ClassStyle
cS_SAVEBITS = consts_ClassStyle 5
cS_DBLCLKS :: ClassStyle
cS_DBLCLKS = consts_ClassStyle 6
cS_BYTEALIGNCLIENT :: ClassStyle
cS_BYTEALIGNCLIENT = consts_ClassStyle 7
cS_BYTEALIGNWINDOW :: ClassStyle
cS_BYTEALIGNWINDOW = consts_ClassStyle 8
cS_NOCLOSE :: ClassStyle
cS_NOCLOSE = consts_ClassStyle 9
cS_KEYCVTWINDOW :: ClassStyle
cS_KEYCVTWINDOW = consts_ClassStyle 10
cS_NOKEYCVT :: ClassStyle
cS_NOKEYCVT = consts_ClassStyle 11
cS_GLOBALCLASS :: ClassStyle
cS_GLOBALCLASS = consts_ClassStyle 12
newtype SystemColor = SystemColor INT deriving (Eq)
cOLOR_SCROLLBAR :: SystemColor
cOLOR_SCROLLBAR = consts_SystemColor 0
cOLOR_BACKGROUND :: SystemColor
cOLOR_BACKGROUND = consts_SystemColor 1
cOLOR_ACTIVECAPTION :: SystemColor
cOLOR_ACTIVECAPTION = consts_SystemColor 2
cOLOR_INACTIVECAPTION :: SystemColor
cOLOR_INACTIVECAPTION = consts_SystemColor 3
cOLOR_MENU :: SystemColor
cOLOR_MENU = consts_SystemColor 4
cOLOR_WINDOW :: SystemColor
cOLOR_WINDOW = consts_SystemColor 5
cOLOR_WINDOWFRAME :: SystemColor
cOLOR_WINDOWFRAME = consts_SystemColor 6
cOLOR_MENUTEXT :: SystemColor
cOLOR_MENUTEXT = consts_SystemColor 7
cOLOR_WINDOWTEXT :: SystemColor
cOLOR_WINDOWTEXT = consts_SystemColor 8
cOLOR_CAPTIONTEXT :: SystemColor
cOLOR_CAPTIONTEXT = consts_SystemColor 9
cOLOR_ACTIVEBORDER :: SystemColor
cOLOR_ACTIVEBORDER = consts_SystemColor 10
cOLOR_INACTIVEBORDER :: SystemColor
cOLOR_INACTIVEBORDER = consts_SystemColor 11
cOLOR_APPWORKSPACE :: SystemColor
cOLOR_APPWORKSPACE = consts_SystemColor 12
cOLOR_HIGHLIGHT :: SystemColor
cOLOR_HIGHLIGHT = consts_SystemColor 13
cOLOR_HIGHLIGHTTEXT :: SystemColor
cOLOR_HIGHLIGHTTEXT = consts_SystemColor 14
cOLOR_BTNFACE :: SystemColor
cOLOR_BTNFACE = consts_SystemColor 15
cOLOR_BTNSHADOW :: SystemColor
cOLOR_BTNSHADOW = consts_SystemColor 16
cOLOR_GRAYTEXT :: SystemColor
cOLOR_GRAYTEXT = consts_SystemColor 17
cOLOR_BTNTEXT :: SystemColor
cOLOR_BTNTEXT = consts_SystemColor 18
cOLOR_INACTIVECAPTIONTEXT :: SystemColor
cOLOR_INACTIVECAPTIONTEXT = consts_SystemColor 19
cOLOR_BTNHIGHLIGHT :: SystemColor
cOLOR_BTNHIGHLIGHT = consts_SystemColor 20
newtype WindowMessage = WindowMessage UINT deriving (Eq)
wM_COMPACTING :: WindowMessage
wM_COMPACTING = consts_WindowMessage 0
wM_WININICHANGE :: WindowMessage
wM_WININICHANGE = consts_WindowMessage 1
wM_SYSCOLORCHANGE :: WindowMessage
wM_SYSCOLORCHANGE = consts_WindowMessage 2
wM_QUERYNEWPALETTE :: WindowMessage
wM_QUERYNEWPALETTE = consts_WindowMessage 3
wM_PALETTEISCHANGING :: WindowMessage
wM_PALETTEISCHANGING = consts_WindowMessage 4
wM_PALETTECHANGED :: WindowMessage
wM_PALETTECHANGED = consts_WindowMessage 5
wM_FONTCHANGE :: WindowMessage
wM_FONTCHANGE = consts_WindowMessage 6
wM_SPOOLERSTATUS :: WindowMessage
wM_SPOOLERSTATUS = consts_WindowMessage 7
wM_DEVMODECHANGE :: WindowMessage
wM_DEVMODECHANGE = consts_WindowMessage 8
wM_TIMECHANGE :: WindowMessage
wM_TIMECHANGE = consts_WindowMessage 9
wM_PENWINFIRST :: WindowMessage
wM_PENWINFIRST = consts_WindowMessage 10
wM_PENWINLAST :: WindowMessage
wM_PENWINLAST = consts_WindowMessage 11
wM_POWER :: WindowMessage
wM_POWER = consts_WindowMessage 12
wM_QUERYENDSESSION :: WindowMessage
wM_QUERYENDSESSION = consts_WindowMessage 13
wM_ENDSESSION :: WindowMessage
wM_ENDSESSION = consts_WindowMessage 14
wM_QUIT :: WindowMessage
wM_QUIT = consts_WindowMessage 15
wM_CREATE :: WindowMessage
wM_CREATE = consts_WindowMessage 16
wM_NCCREATE :: WindowMessage
wM_NCCREATE = consts_WindowMessage 17
wM_DESTROY :: WindowMessage
wM_DESTROY = consts_WindowMessage 18
wM_NCDESTROY :: WindowMessage
wM_NCDESTROY = consts_WindowMessage 19
wM_SHOWWINDOW :: WindowMessage
wM_SHOWWINDOW = consts_WindowMessage 20
wM_SETREDRAW :: WindowMessage
wM_SETREDRAW = consts_WindowMessage 21
wM_ENABLE :: WindowMessage
wM_ENABLE = consts_WindowMessage 22
wM_SETTEXT :: WindowMessage
wM_SETTEXT = consts_WindowMessage 23
wM_GETTEXT :: WindowMessage
wM_GETTEXT = consts_WindowMessage 24
wM_GETTEXTLENGTH :: WindowMessage
wM_GETTEXTLENGTH = consts_WindowMessage 25
wM_WINDOWPOSCHANGING :: WindowMessage
wM_WINDOWPOSCHANGING = consts_WindowMessage 26
wM_WINDOWPOSCHANGED :: WindowMessage
wM_WINDOWPOSCHANGED = consts_WindowMessage 27
wM_MOVE :: WindowMessage
wM_MOVE = consts_WindowMessage 28
wM_SIZE :: WindowMessage
wM_SIZE = consts_WindowMessage 29
wM_QUERYOPEN :: WindowMessage
wM_QUERYOPEN = consts_WindowMessage 30
wM_CLOSE :: WindowMessage
wM_CLOSE = consts_WindowMessage 31
wM_GETMINMAXINFO :: WindowMessage
wM_GETMINMAXINFO = consts_WindowMessage 32
wM_PAINT :: WindowMessage
wM_PAINT = consts_WindowMessage 33
wM_ERASEBKGND :: WindowMessage
wM_ERASEBKGND = consts_WindowMessage 34
wM_ICONERASEBKGND :: WindowMessage
wM_ICONERASEBKGND = consts_WindowMessage 35
wM_NCPAINT :: WindowMessage
wM_NCPAINT = consts_WindowMessage 36
wM_NCCALCSIZE :: WindowMessage
wM_NCCALCSIZE = consts_WindowMessage 37
wM_QUERYDRAGICON :: WindowMessage
wM_QUERYDRAGICON = consts_WindowMessage 38
wM_DROPFILES :: WindowMessage
wM_DROPFILES = consts_WindowMessage 39
wM_ACTIVATE :: WindowMessage
wM_ACTIVATE = consts_WindowMessage 40
wM_ACTIVATEAPP :: WindowMessage
wM_ACTIVATEAPP = consts_WindowMessage 41
wM_NCACTIVATE :: WindowMessage
wM_NCACTIVATE = consts_WindowMessage 42
wM_SETFOCUS :: WindowMessage
wM_SETFOCUS = consts_WindowMessage 43
wM_KILLFOCUS :: WindowMessage
wM_KILLFOCUS = consts_WindowMessage 44
wM_KEYDOWN :: WindowMessage
wM_KEYDOWN = consts_WindowMessage 45
wM_KEYUP :: WindowMessage
wM_KEYUP = consts_WindowMessage 46
wM_CHAR :: WindowMessage
wM_CHAR = consts_WindowMessage 47
wM_DEADCHAR :: WindowMessage
wM_DEADCHAR = consts_WindowMessage 48
wM_SYSKEYDOWN :: WindowMessage
wM_SYSKEYDOWN = consts_WindowMessage 49
wM_SYSKEYUP :: WindowMessage
wM_SYSKEYUP = consts_WindowMessage 50
wM_SYSCHAR :: WindowMessage
wM_SYSCHAR = consts_WindowMessage 51
wM_SYSDEADCHAR :: WindowMessage
wM_SYSDEADCHAR = consts_WindowMessage 52
wM_KEYFIRST :: WindowMessage
wM_KEYFIRST = consts_WindowMessage 53
wM_KEYLAST :: WindowMessage
wM_KEYLAST = consts_WindowMessage 54
wM_MOUSEMOVE :: WindowMessage
wM_MOUSEMOVE = consts_WindowMessage 55
wM_LBUTTONDOWN :: WindowMessage
wM_LBUTTONDOWN = consts_WindowMessage 56
wM_LBUTTONUP :: WindowMessage
wM_LBUTTONUP = consts_WindowMessage 57
wM_LBUTTONDBLCLK :: WindowMessage
wM_LBUTTONDBLCLK = consts_WindowMessage 58
wM_RBUTTONDOWN :: WindowMessage
wM_RBUTTONDOWN = consts_WindowMessage 59
wM_RBUTTONUP :: WindowMessage
wM_RBUTTONUP = consts_WindowMessage 60
wM_RBUTTONDBLCLK :: WindowMessage
wM_RBUTTONDBLCLK = consts_WindowMessage 61
wM_MBUTTONDOWN :: WindowMessage
wM_MBUTTONDOWN = consts_WindowMessage 62
wM_MBUTTONUP :: WindowMessage
wM_MBUTTONUP = consts_WindowMessage 63
wM_MBUTTONDBLCLK :: WindowMessage
wM_MBUTTONDBLCLK = consts_WindowMessage 64
wM_MOUSEFIRST :: WindowMessage
wM_MOUSEFIRST = consts_WindowMessage 65
wM_MOUSELAST :: WindowMessage
wM_MOUSELAST = consts_WindowMessage 66
wM_NCMOUSEMOVE :: WindowMessage
wM_NCMOUSEMOVE = consts_WindowMessage 67
wM_NCLBUTTONDOWN :: WindowMessage
wM_NCLBUTTONDOWN = consts_WindowMessage 68
wM_NCLBUTTONUP :: WindowMessage
wM_NCLBUTTONUP = consts_WindowMessage 69
wM_NCLBUTTONDBLCLK :: WindowMessage
wM_NCLBUTTONDBLCLK = consts_WindowMessage 70
wM_NCRBUTTONDOWN :: WindowMessage
wM_NCRBUTTONDOWN = consts_WindowMessage 71
wM_NCRBUTTONUP :: WindowMessage
wM_NCRBUTTONUP = consts_WindowMessage 72
wM_NCRBUTTONDBLCLK :: WindowMessage
wM_NCRBUTTONDBLCLK = consts_WindowMessage 73
wM_NCMBUTTONDOWN :: WindowMessage
wM_NCMBUTTONDOWN = consts_WindowMessage 74
wM_NCMBUTTONUP :: WindowMessage
wM_NCMBUTTONUP = consts_WindowMessage 75
wM_NCMBUTTONDBLCLK :: WindowMessage
wM_NCMBUTTONDBLCLK = consts_WindowMessage 76
wM_MOUSEACTIVATE :: WindowMessage
wM_MOUSEACTIVATE = consts_WindowMessage 77
wM_CANCELMODE :: WindowMessage
wM_CANCELMODE = consts_WindowMessage 78
wM_TIMER :: WindowMessage
wM_TIMER = consts_WindowMessage 79
wM_INITMENU :: WindowMessage
wM_INITMENU = consts_WindowMessage 80
wM_INITMENUPOPUP :: WindowMessage
wM_INITMENUPOPUP = consts_WindowMessage 81
wM_MENUSELECT :: WindowMessage
wM_MENUSELECT = consts_WindowMessage 82
wM_MENUCHAR :: WindowMessage
wM_MENUCHAR = consts_WindowMessage 83
wM_COMMAND :: WindowMessage
wM_COMMAND = consts_WindowMessage 84
wM_HSCROLL :: WindowMessage
wM_HSCROLL = consts_WindowMessage 85
wM_VSCROLL :: WindowMessage
wM_VSCROLL = consts_WindowMessage 86
wM_CUT :: WindowMessage
wM_CUT = consts_WindowMessage 87
wM_COPY :: WindowMessage
wM_COPY = consts_WindowMessage 88
wM_PASTE :: WindowMessage
wM_PASTE = consts_WindowMessage 89
wM_CLEAR :: WindowMessage
wM_CLEAR = consts_WindowMessage 90
wM_UNDO :: WindowMessage
wM_UNDO = consts_WindowMessage 91
wM_RENDERFORMAT :: WindowMessage
wM_RENDERFORMAT = consts_WindowMessage 92
wM_RENDERALLFORMATS :: WindowMessage
wM_RENDERALLFORMATS = consts_WindowMessage 93
wM_DESTROYCLIPBOARD :: WindowMessage
wM_DESTROYCLIPBOARD = consts_WindowMessage 94
wM_DRAWCLIPBOARD :: WindowMessage
wM_DRAWCLIPBOARD = consts_WindowMessage 95
wM_PAINTCLIPBOARD :: WindowMessage
wM_PAINTCLIPBOARD = consts_WindowMessage 96
wM_SIZECLIPBOARD :: WindowMessage
wM_SIZECLIPBOARD = consts_WindowMessage 97
wM_VSCROLLCLIPBOARD :: WindowMessage
wM_VSCROLLCLIPBOARD = consts_WindowMessage 98
wM_HSCROLLCLIPBOARD :: WindowMessage
wM_HSCROLLCLIPBOARD = consts_WindowMessage 99
wM_ASKCBFORMATNAME :: WindowMessage
wM_ASKCBFORMATNAME = consts_WindowMessage 100
wM_CHANGECBCHAIN :: WindowMessage
wM_CHANGECBCHAIN = consts_WindowMessage 101
wM_SETCURSOR :: WindowMessage
wM_SETCURSOR = consts_WindowMessage 102
wM_SYSCOMMAND :: WindowMessage
wM_SYSCOMMAND = consts_WindowMessage 103
wM_MDICREATE :: WindowMessage
wM_MDICREATE = consts_WindowMessage 104
wM_MDIDESTROY :: WindowMessage
wM_MDIDESTROY = consts_WindowMessage 105
wM_MDIACTIVATE :: WindowMessage
wM_MDIACTIVATE = consts_WindowMessage 106
wM_MDIRESTORE :: WindowMessage
wM_MDIRESTORE = consts_WindowMessage 107
wM_MDINEXT :: WindowMessage
wM_MDINEXT = consts_WindowMessage 108
wM_MDIMAXIMIZE :: WindowMessage
wM_MDIMAXIMIZE = consts_WindowMessage 109
wM_MDITILE :: WindowMessage
wM_MDITILE = consts_WindowMessage 110
wM_MDICASCADE :: WindowMessage
wM_MDICASCADE = consts_WindowMessage 111
wM_MDIICONARRANGE :: WindowMessage
wM_MDIICONARRANGE = consts_WindowMessage 112
wM_MDIGETACTIVE :: WindowMessage
wM_MDIGETACTIVE = consts_WindowMessage 113
wM_MDISETMENU :: WindowMessage
wM_MDISETMENU = consts_WindowMessage 114
wM_CHILDACTIVATE :: WindowMessage
wM_CHILDACTIVATE = consts_WindowMessage 115
wM_INITDIALOG :: WindowMessage
wM_INITDIALOG = consts_WindowMessage 116
wM_NEXTDLGCTL :: WindowMessage
wM_NEXTDLGCTL = consts_WindowMessage 117
wM_PARENTNOTIFY :: WindowMessage
wM_PARENTNOTIFY = consts_WindowMessage 118
wM_ENTERIDLE :: WindowMessage
wM_ENTERIDLE = consts_WindowMessage 119
wM_GETDLGCODE :: WindowMessage
wM_GETDLGCODE = consts_WindowMessage 120
wM_SETFONT :: WindowMessage
wM_SETFONT = consts_WindowMessage 121
wM_GETFONT :: WindowMessage
wM_GETFONT = consts_WindowMessage 122
wM_DRAWITEM :: WindowMessage
wM_DRAWITEM = consts_WindowMessage 123
wM_MEASUREITEM :: WindowMessage
wM_MEASUREITEM = consts_WindowMessage 124
wM_DELETEITEM :: WindowMessage
wM_DELETEITEM = consts_WindowMessage 125
wM_COMPAREITEM :: WindowMessage
wM_COMPAREITEM = consts_WindowMessage 126
wM_VKEYTOITEM :: WindowMessage
wM_VKEYTOITEM = consts_WindowMessage 127
wM_CHARTOITEM :: WindowMessage
wM_CHARTOITEM = consts_WindowMessage 128
wM_QUEUESYNC :: WindowMessage
wM_QUEUESYNC = consts_WindowMessage 129
wM_COMMNOTIFY :: WindowMessage
wM_COMMNOTIFY = consts_WindowMessage 130
newtype VKey = VKey UINT deriving (Eq)
vK_LBUTTON :: VKey
vK_LBUTTON = consts_VKey 0
vK_RBUTTON :: VKey
vK_RBUTTON = consts_VKey 1
vK_CANCEL :: VKey
vK_CANCEL = consts_VKey 2
vK_MBUTTON :: VKey
vK_MBUTTON = consts_VKey 3
vK_BACK :: VKey
vK_BACK = consts_VKey 4
vK_TAB :: VKey
vK_TAB = consts_VKey 5
vK_CLEAR :: VKey
vK_CLEAR = consts_VKey 6
vK_RETURN :: VKey
vK_RETURN = consts_VKey 7
vK_SHIFT :: VKey
vK_SHIFT = consts_VKey 8
vK_CONTROL :: VKey
vK_CONTROL = consts_VKey 9
vK_MENU :: VKey
vK_MENU = consts_VKey 10
vK_PAUSE :: VKey
vK_PAUSE = consts_VKey 11
vK_CAPITAL :: VKey
vK_CAPITAL = consts_VKey 12
vK_ESCAPE :: VKey
vK_ESCAPE = consts_VKey 13
vK_SPACE :: VKey
vK_SPACE = consts_VKey 14
vK_PRIOR :: VKey
vK_PRIOR = consts_VKey 15
vK_NEXT :: VKey
vK_NEXT = consts_VKey 16
vK_END :: VKey
vK_END = consts_VKey 17
vK_HOME :: VKey
vK_HOME = consts_VKey 18
vK_LEFT :: VKey
vK_LEFT = consts_VKey 19
vK_UP :: VKey
vK_UP = consts_VKey 20
vK_RIGHT :: VKey
vK_RIGHT = consts_VKey 21
vK_DOWN :: VKey
vK_DOWN = consts_VKey 22
vK_SELECT :: VKey
vK_SELECT = consts_VKey 23
vK_PRINT :: VKey
vK_PRINT = consts_VKey 24
vK_EXECUTE :: VKey
vK_EXECUTE = consts_VKey 25
vK_SNAPSHOT :: VKey
vK_SNAPSHOT = consts_VKey 26
vK_INSERT :: VKey
vK_INSERT = consts_VKey 27
vK_DELETE :: VKey
vK_DELETE = consts_VKey 28
vK_HELP :: VKey
vK_HELP = consts_VKey 29
vK_LWIN :: VKey
vK_LWIN = consts_VKey 30
vK_RWIN :: VKey
vK_RWIN = consts_VKey 31
vK_APPS :: VKey
vK_APPS = consts_VKey 32
vK_NUMPAD0 :: VKey
vK_NUMPAD0 = consts_VKey 33
vK_NUMPAD1 :: VKey
vK_NUMPAD1 = consts_VKey 34
vK_NUMPAD2 :: VKey
vK_NUMPAD2 = consts_VKey 35
vK_NUMPAD3 :: VKey
vK_NUMPAD3 = consts_VKey 36
vK_NUMPAD4 :: VKey
vK_NUMPAD4 = consts_VKey 37
vK_NUMPAD5 :: VKey
vK_NUMPAD5 = consts_VKey 38
vK_NUMPAD6 :: VKey
vK_NUMPAD6 = consts_VKey 39
vK_NUMPAD7 :: VKey
vK_NUMPAD7 = consts_VKey 40
vK_NUMPAD8 :: VKey
vK_NUMPAD8 = consts_VKey 41
vK_NUMPAD9 :: VKey
vK_NUMPAD9 = consts_VKey 42
vK_MULTIPLY :: VKey
vK_MULTIPLY = consts_VKey 43
vK_ADD :: VKey
vK_ADD = consts_VKey 44
vK_SEPARATOR :: VKey
vK_SEPARATOR = consts_VKey 45
vK_SUBTRACT :: VKey
vK_SUBTRACT = consts_VKey 46
vK_DECIMAL :: VKey
vK_DECIMAL = consts_VKey 47
vK_DIVIDE :: VKey
vK_DIVIDE = consts_VKey 48
vK_F1 :: VKey
vK_F1 = consts_VKey 49
vK_F2 :: VKey
vK_F2 = consts_VKey 50
vK_F3 :: VKey
vK_F3 = consts_VKey 51
vK_F4 :: VKey
vK_F4 = consts_VKey 52
vK_F5 :: VKey
vK_F5 = consts_VKey 53
vK_F6 :: VKey
vK_F6 = consts_VKey 54
vK_F7 :: VKey
vK_F7 = consts_VKey 55
vK_F8 :: VKey
vK_F8 = consts_VKey 56
vK_F9 :: VKey
vK_F9 = consts_VKey 57
vK_F10 :: VKey
vK_F10 = consts_VKey 58
vK_F11 :: VKey
vK_F11 = consts_VKey 59
vK_F12 :: VKey
vK_F12 = consts_VKey 60
vK_F13 :: VKey
vK_F13 = consts_VKey 61
vK_F14 :: VKey
vK_F14 = consts_VKey 62
vK_F15 :: VKey
vK_F15 = consts_VKey 63
vK_F16 :: VKey
vK_F16 = consts_VKey 64
vK_F17 :: VKey
vK_F17 = consts_VKey 65
vK_F18 :: VKey
vK_F18 = consts_VKey 66
vK_F19 :: VKey
vK_F19 = consts_VKey 67
vK_F20 :: VKey
vK_F20 = consts_VKey 68
vK_F21 :: VKey
vK_F21 = consts_VKey 69
vK_F22 :: VKey
vK_F22 = consts_VKey 70
vK_F23 :: VKey
vK_F23 = consts_VKey 71
vK_F24 :: VKey
vK_F24 = consts_VKey 72
vK_NUMLOCK :: VKey
vK_NUMLOCK = consts_VKey 73
vK_SCROLL :: VKey
vK_SCROLL = consts_VKey 74
newtype WindowStyle = WindowStyle DWORD deriving ()
instance BitSet WindowStyle where
  toWord32   = \ (WindowStyle x) -> x
  fromWord32 = WindowStyle
wS_OVERLAPPED :: WindowStyle
wS_OVERLAPPED = consts_WindowStyle 0
wS_POPUP :: WindowStyle
wS_POPUP = consts_WindowStyle 1
wS_CHILD :: WindowStyle
wS_CHILD = consts_WindowStyle 2
wS_CLIPSIBLINGS :: WindowStyle
wS_CLIPSIBLINGS = consts_WindowStyle 3
wS_CLIPCHILDREN :: WindowStyle
wS_CLIPCHILDREN = consts_WindowStyle 4
wS_VISIBLE :: WindowStyle
wS_VISIBLE = consts_WindowStyle 5
wS_DISABLED :: WindowStyle
wS_DISABLED = consts_WindowStyle 6
wS_MINIMIZE :: WindowStyle
wS_MINIMIZE = consts_WindowStyle 7
wS_MAXIMIZE :: WindowStyle
wS_MAXIMIZE = consts_WindowStyle 8
wS_CAPTION :: WindowStyle
wS_CAPTION = consts_WindowStyle 9
wS_BORDER :: WindowStyle
wS_BORDER = consts_WindowStyle 10
wS_DLGFRAME :: WindowStyle
wS_DLGFRAME = consts_WindowStyle 11
wS_VSCROLL :: WindowStyle
wS_VSCROLL = consts_WindowStyle 12
wS_HSCROLL :: WindowStyle
wS_HSCROLL = consts_WindowStyle 13
wS_SYSMENU :: WindowStyle
wS_SYSMENU = consts_WindowStyle 14
wS_THICKFRAME :: WindowStyle
wS_THICKFRAME = consts_WindowStyle 15
wS_MINIMIZEBOX :: WindowStyle
wS_MINIMIZEBOX = consts_WindowStyle 16
wS_MAXIMIZEBOX :: WindowStyle
wS_MAXIMIZEBOX = consts_WindowStyle 17
wS_GROUP :: WindowStyle
wS_GROUP = consts_WindowStyle 18
wS_TABSTOP :: WindowStyle
wS_TABSTOP = consts_WindowStyle 19
wS_OVERLAPPEDWINDOW :: WindowStyle
wS_OVERLAPPEDWINDOW = consts_WindowStyle 20
wS_POPUPWINDOW :: WindowStyle
wS_POPUPWINDOW = consts_WindowStyle 21
wS_CHILDWINDOW :: WindowStyle
wS_CHILDWINDOW = consts_WindowStyle 22
wS_EX_DLGMODALFRAME :: WindowStyle
wS_EX_DLGMODALFRAME = consts_WindowStyle 23
wS_EX_NOPARENTNOTIFY :: WindowStyle
wS_EX_NOPARENTNOTIFY = consts_WindowStyle 24
wS_EX_TOPMOST :: WindowStyle
wS_EX_TOPMOST = consts_WindowStyle 25
wS_EX_ACCEPTFILES :: WindowStyle
wS_EX_ACCEPTFILES = consts_WindowStyle 26
wS_EX_TRANSPARENT :: WindowStyle
wS_EX_TRANSPARENT = consts_WindowStyle 27
wS_TILED :: WindowStyle
wS_TILED = consts_WindowStyle 28
wS_ICONIC :: WindowStyle
wS_ICONIC = consts_WindowStyle 29
wS_SIZEBOX :: WindowStyle
wS_SIZEBOX = consts_WindowStyle 30
wS_TILEDWINDOW :: WindowStyle
wS_TILEDWINDOW = consts_WindowStyle 31
newtype ClassName = ClassName LPCSTR deriving ()
primitive mkClassName "XS_mkClassName" :: PString -> ClassName
primitive atomToClassName "XS_atomToClassName" :: ATOM -> ClassName
type Pos = Int
type MbPos = Maybe Pos
primitive defWindowProc "XS_DefWindowProc" :: MbHWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT
-- Note: these are only called when hwnd is non-null - so first arg
-- has type HWND instead of Maybe HWND.
type WindowClosure = HWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT

{- Note: the final parameter of createWindow is the "callback routine".
   In standard Windows programming, this is specified when the window
   class is registered - but it's much more convenient to specify it
   when the window is created.
-}
primitive createWindow "XS_createWindow" :: ClassName -> String -> WindowStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HINSTANCE -> WindowClosure -> IO HWND
primitive createWindowEx "XS_createWindowEx" :: WindowStyle -> ClassName -> String -> WindowStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HINSTANCE -> WindowClosure -> IO HWND
primitive destroyWindow "XS_DestroyWindow" :: HWND -> IO ()
primitive bringWindowToTop "XS_BringWindowToTop" :: HWND -> IO ()
primitive adjustWindowRect "XS_AdjustWindowRect" :: RECT -> WindowStyle -> Bool -> IO RECT 
primitive mkMbHBRUSH "XS_mkMbHBRUSH" :: SystemColor -> MbHBRUSH
type WNDCLASS = (ClassStyle,HINSTANCE,MbHICON,MbHCURSOR,MbHBRUSH,MbLPCSTR,ClassName)
type MbATOM = Maybe ATOM
primitive registerClass "XS_RegisterClass" :: WNDCLASS -> IO ATOM
primitive unregisterClass "XS_UnregisterClass" :: ClassName -> HINSTANCE -> IO ()
type ButtonStyle = WindowStyle
bS_PUSHBUTTON :: ButtonStyle
bS_PUSHBUTTON = consts_ButtonStyle 0
bS_DEFPUSHBUTTON :: ButtonStyle
bS_DEFPUSHBUTTON = consts_ButtonStyle 1
bS_CHECKBOX :: ButtonStyle
bS_CHECKBOX = consts_ButtonStyle 2
bS_AUTOCHECKBOX :: ButtonStyle
bS_AUTOCHECKBOX = consts_ButtonStyle 3
bS_RADIOBUTTON :: ButtonStyle
bS_RADIOBUTTON = consts_ButtonStyle 4
bS_3STATE :: ButtonStyle
bS_3STATE = consts_ButtonStyle 5
bS_AUTO3STATE :: ButtonStyle
bS_AUTO3STATE = consts_ButtonStyle 6
bS_GROUPBOX :: ButtonStyle
bS_GROUPBOX = consts_ButtonStyle 7
bS_AUTORADIOBUTTON :: ButtonStyle
bS_AUTORADIOBUTTON = consts_ButtonStyle 8
bS_OWNERDRAW :: ButtonStyle
bS_OWNERDRAW = consts_ButtonStyle 9
bS_LEFTTEXT :: ButtonStyle
bS_LEFTTEXT = consts_ButtonStyle 10
bS_USERBUTTON :: ButtonStyle
bS_USERBUTTON = consts_ButtonStyle 11
primitive createButton "XS_createButton" :: String -> WindowStyle -> ButtonStyle -> MbPos -> MbPos -> MbPos -> MbPos -> MbHWND -> MbHMENU -> HANDLE -> IO HWND
type ComboBoxStyle = WindowStyle
cBS_SIMPLE :: ComboBoxStyle
cBS_SIMPLE = consts_ComboBoxStyle 0
cBS_DROPDOWN :: ComboBoxStyle
cBS_DROPDOWN = consts_ComboBoxStyle 1
cBS_DROPDOWNLIST :: ComboBoxStyle
cBS_DROPDOWNLIST = consts_ComboBoxStyle 2
cBS_OWNERDRAWFIXED :: ComboBoxStyle
cBS_OWNERDRAWFIXED = consts_ComboBoxStyle 3
cBS_OWNERDRAWVARIABLE :: ComboBoxStyle
cBS_OWNERDRAWVARIABLE = consts_ComboBoxStyle 4
cBS_AUTOHSCROLL :: ComboBoxStyle
cBS_AUTOHSCROLL = consts_ComboBoxStyle 5
cBS_OEMCONVERT :: ComboBoxStyle
cBS_OEMCONVERT = consts_ComboBoxStyle 6
cBS_SORT :: ComboBoxStyle
cBS_SORT = consts_ComboBoxStyle 7
cBS_HASSTRINGS :: ComboBoxStyle
cBS_HASSTRINGS = consts_ComboBoxStyle 8
cBS_NOINTEGRALHEIGHT :: ComboBoxStyle
cBS_NOINTEGRALHEIGHT = consts_ComboBoxStyle 9
cBS_DISABLENOSCROLL :: ComboBoxStyle
cBS_DISABLENOSCROLL = consts_ComboBoxStyle 10
primitive createComboBox "XS_createComboBox" :: String -> WindowStyle -> ComboBoxStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
type EditStyle = WindowStyle
eS_LEFT :: EditStyle
eS_LEFT = consts_EditStyle 0
eS_CENTER :: EditStyle
eS_CENTER = consts_EditStyle 1
eS_RIGHT :: EditStyle
eS_RIGHT = consts_EditStyle 2
eS_MULTILINE :: EditStyle
eS_MULTILINE = consts_EditStyle 3
eS_UPPERCASE :: EditStyle
eS_UPPERCASE = consts_EditStyle 4
eS_LOWERCASE :: EditStyle
eS_LOWERCASE = consts_EditStyle 5
eS_PASSWORD :: EditStyle
eS_PASSWORD = consts_EditStyle 6
eS_AUTOVSCROLL :: EditStyle
eS_AUTOVSCROLL = consts_EditStyle 7
eS_AUTOHSCROLL :: EditStyle
eS_AUTOHSCROLL = consts_EditStyle 8
eS_NOHIDESEL :: EditStyle
eS_NOHIDESEL = consts_EditStyle 9
eS_OEMCONVERT :: EditStyle
eS_OEMCONVERT = consts_EditStyle 10
eS_READONLY :: EditStyle
eS_READONLY = consts_EditStyle 11
eS_WANTRETURN :: EditStyle
eS_WANTRETURN = consts_EditStyle 12
primitive createEditWindow "XS_createEditWindow" :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
type ListBoxStyle = WindowStyle
lBS_NOTIFY :: ListBoxStyle
lBS_NOTIFY = consts_ListBoxStyle 0
lBS_SORT :: ListBoxStyle
lBS_SORT = consts_ListBoxStyle 1
lBS_NOREDRAW :: ListBoxStyle
lBS_NOREDRAW = consts_ListBoxStyle 2
lBS_MULTIPLESEL :: ListBoxStyle
lBS_MULTIPLESEL = consts_ListBoxStyle 3
lBS_OWNERDRAWFIXED :: ListBoxStyle
lBS_OWNERDRAWFIXED = consts_ListBoxStyle 4
lBS_OWNERDRAWVARIABLE :: ListBoxStyle
lBS_OWNERDRAWVARIABLE = consts_ListBoxStyle 5
lBS_HASSTRINGS :: ListBoxStyle
lBS_HASSTRINGS = consts_ListBoxStyle 6
lBS_USETABSTOPS :: ListBoxStyle
lBS_USETABSTOPS = consts_ListBoxStyle 7
lBS_NOINTEGRALHEIGHT :: ListBoxStyle
lBS_NOINTEGRALHEIGHT = consts_ListBoxStyle 8
lBS_MULTICOLUMN :: ListBoxStyle
lBS_MULTICOLUMN = consts_ListBoxStyle 9
lBS_WANTKEYBOARDINPUT :: ListBoxStyle
lBS_WANTKEYBOARDINPUT = consts_ListBoxStyle 10
lBS_EXTENDEDSEL :: ListBoxStyle
lBS_EXTENDEDSEL = consts_ListBoxStyle 11
lBS_DISABLENOSCROLL :: ListBoxStyle
lBS_DISABLENOSCROLL = consts_ListBoxStyle 12
lBS_STANDARD :: ListBoxStyle
lBS_STANDARD = consts_ListBoxStyle 13
primitive createListbox "XS_createListbox" :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
type ScrollbarStyle = WindowStyle
sBS_HORZ :: ScrollbarStyle
sBS_HORZ = consts_ScrollbarStyle 0
sBS_TOPALIGN :: ScrollbarStyle
sBS_TOPALIGN = consts_ScrollbarStyle 1
sBS_BOTTOMALIGN :: ScrollbarStyle
sBS_BOTTOMALIGN = consts_ScrollbarStyle 2
sBS_VERT :: ScrollbarStyle
sBS_VERT = consts_ScrollbarStyle 3
sBS_LEFTALIGN :: ScrollbarStyle
sBS_LEFTALIGN = consts_ScrollbarStyle 4
sBS_RIGHTALIGN :: ScrollbarStyle
sBS_RIGHTALIGN = consts_ScrollbarStyle 5
sBS_SIZEBOX :: ScrollbarStyle
sBS_SIZEBOX = consts_ScrollbarStyle 6
sBS_SIZEBOXTOPLEFTALIGN :: ScrollbarStyle
sBS_SIZEBOXTOPLEFTALIGN = consts_ScrollbarStyle 7
sBS_SIZEBOXBOTTOMRIGHTALIGN :: ScrollbarStyle
sBS_SIZEBOXBOTTOMRIGHTALIGN = consts_ScrollbarStyle 8
primitive createScrollbar "XS_createScrollbar" :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
type ScrollbarType = WindowStyle
sB_HORZ :: ScrollbarType
sB_HORZ = consts_ScrollbarType 0
sB_VERT :: ScrollbarType
sB_VERT = consts_ScrollbarType 1
sB_CTL :: ScrollbarType
sB_CTL = consts_ScrollbarType 2
sB_BOTH :: ScrollbarType
sB_BOTH = consts_ScrollbarType 3
type ScrollbarCommand = Int
sB_LINEUP :: ScrollbarCommand
sB_LINEUP = consts_ScrollbarCommand 0
sB_LINELEFT :: ScrollbarCommand
sB_LINELEFT = consts_ScrollbarCommand 1
sB_LINEDOWN :: ScrollbarCommand
sB_LINEDOWN = consts_ScrollbarCommand 2
sB_LINERIGHT :: ScrollbarCommand
sB_LINERIGHT = consts_ScrollbarCommand 3
sB_PAGEUP :: ScrollbarCommand
sB_PAGEUP = consts_ScrollbarCommand 4
sB_PAGELEFT :: ScrollbarCommand
sB_PAGELEFT = consts_ScrollbarCommand 5
sB_PAGEDOWN :: ScrollbarCommand
sB_PAGEDOWN = consts_ScrollbarCommand 6
sB_PAGERIGHT :: ScrollbarCommand
sB_PAGERIGHT = consts_ScrollbarCommand 7
sB_THUMBPOSITION :: ScrollbarCommand
sB_THUMBPOSITION = consts_ScrollbarCommand 8
sB_THUMBTRACK :: ScrollbarCommand
sB_THUMBTRACK = consts_ScrollbarCommand 9
sB_TOP :: ScrollbarCommand
sB_TOP = consts_ScrollbarCommand 10
sB_LEFT :: ScrollbarCommand
sB_LEFT = consts_ScrollbarCommand 11
sB_BOTTOM :: ScrollbarCommand
sB_BOTTOM = consts_ScrollbarCommand 12
sB_RIGHT :: ScrollbarCommand
sB_RIGHT = consts_ScrollbarCommand 13
sB_ENDSCROLL :: ScrollbarCommand
sB_ENDSCROLL = consts_ScrollbarCommand 14
primitive showScrollBar "XS_ShowScrollBar" :: HWND -> ScrollbarType -> Bool -> IO ()
primitive setScrollRange "XS_SetScrollRange" :: HWND -> ScrollbarType -> Int  -> Int -> Bool -> IO ()
primitive getScrollPos "XS_getScrollPos" :: HWND -> ScrollbarType -> IO Int
primitive setScrollPos "XS_setScrollPos" :: HWND -> ScrollbarType -> Int -> Bool -> IO ()
type StaticControlStyle = WindowStyle
sS_LEFT :: StaticControlStyle
sS_LEFT = consts_StaticControlStyle 0
sS_CENTER :: StaticControlStyle
sS_CENTER = consts_StaticControlStyle 1
sS_RIGHT :: StaticControlStyle
sS_RIGHT = consts_StaticControlStyle 2
sS_ICON :: StaticControlStyle
sS_ICON = consts_StaticControlStyle 3
sS_BLACKRECT :: StaticControlStyle
sS_BLACKRECT = consts_StaticControlStyle 4
sS_GRAYRECT :: StaticControlStyle
sS_GRAYRECT = consts_StaticControlStyle 5
sS_WHITERECT :: StaticControlStyle
sS_WHITERECT = consts_StaticControlStyle 6
sS_BLACKFRAME :: StaticControlStyle
sS_BLACKFRAME = consts_StaticControlStyle 7
sS_GRAYFRAME :: StaticControlStyle
sS_GRAYFRAME = consts_StaticControlStyle 8
sS_WHITEFRAME :: StaticControlStyle
sS_WHITEFRAME = consts_StaticControlStyle 9
sS_SIMPLE :: StaticControlStyle
sS_SIMPLE = consts_StaticControlStyle 10
sS_LEFTNOWORDWRAP :: StaticControlStyle
sS_LEFTNOWORDWRAP = consts_StaticControlStyle 11
sS_NOPREFIX :: StaticControlStyle
sS_NOPREFIX = consts_StaticControlStyle 12
primitive createStaticWindow "XS_createStaticWindow" :: String -> WindowStyle -> EditStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
type PAINTSTRUCT = (HDC,Bool,RECT)
type LPPAINTSTRUCT = Ptr in
  toLPPAINTSTRUCT   :: Ptr -> LPPAINTSTRUCT,
  fromLPPAINTSTRUCT :: LPPAINTSTRUCT -> Ptr

toLPPAINTSTRUCT   x = x 
fromLPPAINTSTRUCT x = x

instance Pointer LPPAINTSTRUCT where
  toPtr     = fromLPPAINTSTRUCT
  fromPtr   = toLPPAINTSTRUCT
  alloc     = malloc sizeofLPPAINTSTRUCT
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPPAINTSTRUCT))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPPAINTSTRUCT

primitive getLPPAINTSTRUCT        "XS_get_LPPAINTSTRUCT"    :: LPPAINTSTRUCT -> IO PAINTSTRUCT
primitive setLPPAINTSTRUCT        "XS_set_LPPAINTSTRUCT"    :: LPPAINTSTRUCT -> PAINTSTRUCT -> IO ()
sizeofLPPAINTSTRUCT :: Int
sizeofLPPAINTSTRUCT = consts_Int 3
primitive beginPaint "XS_BeginPaint" :: HWND   -> LPPAINTSTRUCT -> IO HDC
primitive endPaint "XS_EndPaint" :: HWND   -> LPPAINTSTRUCT -> IO ()
primitive getClientRect "XS_GetClientRect" :: HWND   -> IO RECT 
primitive invalidateRect "XS_InvalidateRect" :: MbHWND -> MbLPRECT -> Bool -> IO ()
newtype ShowWindowControl = ShowWindowControl INT deriving (Eq)
sW_HIDE :: ShowWindowControl
sW_HIDE = consts_ShowWindowControl 0
sW_SHOWNORMAL :: ShowWindowControl
sW_SHOWNORMAL = consts_ShowWindowControl 1
sW_NORMAL :: ShowWindowControl
sW_NORMAL = consts_ShowWindowControl 2
sW_SHOWMINIMIZED :: ShowWindowControl
sW_SHOWMINIMIZED = consts_ShowWindowControl 3
sW_SHOWMAXIMIZED :: ShowWindowControl
sW_SHOWMAXIMIZED = consts_ShowWindowControl 4
sW_MAXIMIZE :: ShowWindowControl
sW_MAXIMIZE = consts_ShowWindowControl 5
sW_SHOWNOACTIVATE :: ShowWindowControl
sW_SHOWNOACTIVATE = consts_ShowWindowControl 6
sW_SHOW :: ShowWindowControl
sW_SHOW = consts_ShowWindowControl 7
sW_MINIMIZE :: ShowWindowControl
sW_MINIMIZE = consts_ShowWindowControl 8
sW_SHOWMINNOACTIVE :: ShowWindowControl
sW_SHOWMINNOACTIVE = consts_ShowWindowControl 9
sW_SHOWNA :: ShowWindowControl
sW_SHOWNA = consts_ShowWindowControl 10
sW_RESTORE :: ShowWindowControl
sW_RESTORE = consts_ShowWindowControl 11
hIDE_WINDOW :: ShowWindowControl
hIDE_WINDOW = consts_ShowWindowControl 12
sHOW_OPENWINDOW :: ShowWindowControl
sHOW_OPENWINDOW = consts_ShowWindowControl 13
sHOW_ICONWINDOW :: ShowWindowControl
sHOW_ICONWINDOW = consts_ShowWindowControl 14
sHOW_FULLSCREEN :: ShowWindowControl
sHOW_FULLSCREEN = consts_ShowWindowControl 15
sHOW_OPENNOACTIVATE :: ShowWindowControl
sHOW_OPENNOACTIVATE = consts_ShowWindowControl 16
primitive showWindow "XS_ShowWindow" :: HWND -> ShowWindowControl -> IO Bool
primitive updateWindow "XS_UpdateWindow" :: HWND -> IO ()
primitive getModuleHandle "XS_GetModuleHandle" :: MbString -> IO HMODULE
newtype StdHandleId = StdHandleId DWORD deriving (Eq)
sTD_INPUT_HANDLE :: StdHandleId
sTD_INPUT_HANDLE = consts_StdHandleId 0
sTD_OUTPUT_HANDLE :: StdHandleId
sTD_OUTPUT_HANDLE = consts_StdHandleId 1
sTD_ERROR_HANDLE :: StdHandleId
sTD_ERROR_HANDLE = consts_StdHandleId 2
primitive getStdHandle "XS_GetStdHandle" :: StdHandleId -> IO HANDLE
type TimerId = UINT
primitive setWinTimer "XS_setWinTimer" :: HWND -> TimerId -> UINT -> IO TimerId
primitive killTimer "XS_KillTimer" :: MbHWND -> TimerId -> IO ()
type MilliSeconds = DWORD
primitive timeGetTime "XS_timeGetTime" :: IO MilliSeconds
type Duration = DWORD
type MbDuration = Maybe Duration
primitive beep "XS_Beep" :: DWORD -> MbDuration -> IO ()
type MbBeep = Maybe MBStyle
primitive messageBeep "XS_MessageBeep" :: MbBeep -> IO ()
primitive flashWindow "XS_FlashWindow" :: HWND -> Bool -> IO Bool
primitive exitWindows "XS_ExitWindows" :: IO ()
newtype ExitOption = ExitOption UINT deriving (Eq)
eWX_FORCE :: ExitOption
eWX_FORCE = consts_ExitOption 0
eWX_LOGOFF :: ExitOption
eWX_LOGOFF = consts_ExitOption 1
eWX_POWEROFF :: ExitOption
eWX_POWEROFF = consts_ExitOption 2
eWX_REBOOT :: ExitOption
eWX_REBOOT = consts_ExitOption 3
eWX_SHUTDOWN :: ExitOption
eWX_SHUTDOWN = consts_ExitOption 4
primitive exitWindowsEx "XS_exitWindowsEx" :: ExitOption -> IO ()
primitive getCursorPos "XS_GetCursorPos" :: IO POINT 
primitive setCursorPos "XS_SetCursorPos" :: POINT -> IO ()
primitive clipCursor "XS_ClipCursor" :: RECT -> IO () 
primitive getClipCursor "XS_GetClipCursor" :: IO RECT 
primitive getCaretPos "XS_GetCaretPos" :: IO POINT 
primitive setCaretPos "XS_SetCaretPos" :: POINT -> IO ()
primitive hideCaret "XS_HideCaret" :: HWND -> IO ()
primitive showCaret "XS_ShowCaret" :: HWND -> IO ()
primitive createCaret "XS_CreateCaret" :: HWND -> HBITMAP -> MbINT -> MbINT -> IO ()
primitive destroyCaret "XS_DestroyCaret" :: IO ()
primitive screenToClient "XS_ScreenToClient" :: HWND -> POINT -> IO POINT
primitive clientToScreen "XS_ClientToScreen" :: HWND -> POINT -> IO POINT
type MSG = (HWND,UINT,WPARAM,LPARAM,DWORD,POINT)
type LPMSG = Ptr in
  toLPMSG   :: Ptr -> LPMSG,
  fromLPMSG :: LPMSG -> Ptr

toLPMSG   x = x 
fromLPMSG x = x

instance Pointer LPMSG where
  toPtr     = fromLPMSG
  fromPtr   = toLPMSG
  alloc     = malloc sizeofLPMSG
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPMSG))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPMSG

primitive getLPMSG        "XS_get_LPMSG"    :: LPMSG -> IO MSG
primitive setLPMSG        "XS_set_LPMSG"    :: LPMSG -> MSG -> IO ()
sizeofLPMSG :: Int
sizeofLPMSG = consts_Int 4
primitive isGetMsgError "XStest_isGetMsgError" :: IOError -> Bool
primitive getMessage "XS_GetMessage" :: LPMSG -> MbHWND -> IO Bool 
primitive peekMessage "XS_PeekMessage" :: LPMSG -> MbHWND -> IO Bool
primitive translateMessage "XS_TranslateMessage" :: LPMSG -> IO Bool
primitive dispatchMessage "XS_DispatchMessage" :: LPMSG -> IO LONG
primitive get_hugsQuitFlag "XSget_hugsQuitFlag" :: IO Bool
primitive set_hugsQuitFlag "XSset_hugsQuitFlag" :: Bool -> IO ()
eventLoop :: HWND -> IO ()
eventLoop hwnd = do
  set_hugsQuitFlag False
  lpmsg <- alloc
  while (map not get_hugsQuitFlag) (do
    getMessage       lpmsg (Just hwnd)
    translateMessage lpmsg
    dispatchMessage  lpmsg
    )
  free lpmsg
 where
  while :: IO Bool -> IO a -> IO ()
  while p s = do
    c <- p
    if c then s >> while p s else return ()
primitive postMessage "XS_PostMessage" :: MbHWND -> WindowMessage -> WPARAM -> LPARAM -> IO ()
primitive postQuitMessage "XS_PostQuitMessage" :: Int -> IO ()
primitive postThreadMessage "XS_PostThreadMessage" :: DWORD -> WindowMessage -> WPARAM -> LPARAM -> IO ()
primitive sendMessage "XS_SendMessage" :: HWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT
primitive inSendMessage "XS_InSendMessage" :: IO Bool
primitive setFocus "XS_SetFocus" :: HWND -> IO HWND
mA_NOACTIVATE :: LRESULT
mA_NOACTIVATE = consts_LRESULT 0
mA_NOACTIVATEANDEAT :: LRESULT
mA_NOACTIVATEANDEAT = consts_LRESULT 1
mA_ACTIVATE :: LRESULT
mA_ACTIVATE = consts_LRESULT 2
mA_ACTIVATEANDEAT :: LRESULT
mA_ACTIVATEANDEAT = consts_LRESULT 3
primitive eqHWND "XS_eqHWND" :: HWND -> HWND -> Bool
newtype SMSetting = SMSetting UINT deriving (Eq)
primitive getSystemMetrics "XS_GetSystemMetrics" :: SMSetting -> IO Int
sM_ARRANGE :: SMSetting
sM_ARRANGE = consts_SMSetting 0
sM_CLEANBOOT :: SMSetting
sM_CLEANBOOT = consts_SMSetting 1
sM_CMETRICS :: SMSetting
sM_CMETRICS = consts_SMSetting 2
sM_CMOUSEBUTTONS :: SMSetting
sM_CMOUSEBUTTONS = consts_SMSetting 3
sM_CXBORDER :: SMSetting
sM_CXBORDER = consts_SMSetting 4
sM_CYBORDER :: SMSetting
sM_CYBORDER = consts_SMSetting 5
sM_CXCURSOR :: SMSetting
sM_CXCURSOR = consts_SMSetting 6
sM_CYCURSOR :: SMSetting
sM_CYCURSOR = consts_SMSetting 7
sM_CXDLGFRAME :: SMSetting
sM_CXDLGFRAME = consts_SMSetting 8
sM_CYDLGFRAME :: SMSetting
sM_CYDLGFRAME = consts_SMSetting 9
sM_CXDOUBLECLK :: SMSetting
sM_CXDOUBLECLK = consts_SMSetting 10
sM_CYDOUBLECLK :: SMSetting
sM_CYDOUBLECLK = consts_SMSetting 11
sM_CXDRAG :: SMSetting
sM_CXDRAG = consts_SMSetting 12
sM_CYDRAG :: SMSetting
sM_CYDRAG = consts_SMSetting 13
sM_CXEDGE :: SMSetting
sM_CXEDGE = consts_SMSetting 14
sM_CYEDGE :: SMSetting
sM_CYEDGE = consts_SMSetting 15
sM_CXFRAME :: SMSetting
sM_CXFRAME = consts_SMSetting 16
sM_CYFRAME :: SMSetting
sM_CYFRAME = consts_SMSetting 17
sM_CXFULLSCREEN :: SMSetting
sM_CXFULLSCREEN = consts_SMSetting 18
sM_CYFULLSCREEN :: SMSetting
sM_CYFULLSCREEN = consts_SMSetting 19
sM_CXHSCROLL :: SMSetting
sM_CXHSCROLL = consts_SMSetting 20
sM_CYVSCROLL :: SMSetting
sM_CYVSCROLL = consts_SMSetting 21
sM_CXHTHUMB :: SMSetting
sM_CXHTHUMB = consts_SMSetting 22
sM_CXICON :: SMSetting
sM_CXICON = consts_SMSetting 23
sM_CYICON :: SMSetting
sM_CYICON = consts_SMSetting 24
sM_CXICONSPACING :: SMSetting
sM_CXICONSPACING = consts_SMSetting 25
sM_CYICONSPACING :: SMSetting
sM_CYICONSPACING = consts_SMSetting 26
sM_CXMAXIMIZED :: SMSetting
sM_CXMAXIMIZED = consts_SMSetting 27
sM_CYMAXIMIZED :: SMSetting
sM_CYMAXIMIZED = consts_SMSetting 28
sM_CXMAXTRACK :: SMSetting
sM_CXMAXTRACK = consts_SMSetting 29
sM_CYMAXTRACK :: SMSetting
sM_CYMAXTRACK = consts_SMSetting 30
sM_CXMENUCHECK :: SMSetting
sM_CXMENUCHECK = consts_SMSetting 31
sM_CYMENUCHECK :: SMSetting
sM_CYMENUCHECK = consts_SMSetting 32
sM_CXMENUSIZE :: SMSetting
sM_CXMENUSIZE = consts_SMSetting 33
sM_CYMENUSIZE :: SMSetting
sM_CYMENUSIZE = consts_SMSetting 34
sM_CXMIN :: SMSetting
sM_CXMIN = consts_SMSetting 35
sM_CYMIN :: SMSetting
sM_CYMIN = consts_SMSetting 36
sM_CXMINIMIZED :: SMSetting
sM_CXMINIMIZED = consts_SMSetting 37
sM_CYMINIMIZED :: SMSetting
sM_CYMINIMIZED = consts_SMSetting 38
sM_CXMINSPACING :: SMSetting
sM_CXMINSPACING = consts_SMSetting 39
sM_CYMINSPACING :: SMSetting
sM_CYMINSPACING = consts_SMSetting 40
sM_CXMINTRACK :: SMSetting
sM_CXMINTRACK = consts_SMSetting 41
sM_CYMINTRACK :: SMSetting
sM_CYMINTRACK = consts_SMSetting 42
sM_CXSCREEN :: SMSetting
sM_CXSCREEN = consts_SMSetting 43
sM_CYSCREEN :: SMSetting
sM_CYSCREEN = consts_SMSetting 44
sM_CXSIZE :: SMSetting
sM_CXSIZE = consts_SMSetting 45
sM_CYSIZE :: SMSetting
sM_CYSIZE = consts_SMSetting 46
sM_CXSIZEFRAME :: SMSetting
sM_CXSIZEFRAME = consts_SMSetting 47
sM_CYSIZEFRAME :: SMSetting
sM_CYSIZEFRAME = consts_SMSetting 48
sM_CXSMICON :: SMSetting
sM_CXSMICON = consts_SMSetting 49
sM_CYSMICON :: SMSetting
sM_CYSMICON = consts_SMSetting 50
sM_CXSMSIZE :: SMSetting
sM_CXSMSIZE = consts_SMSetting 51
sM_CYSMSIZE :: SMSetting
sM_CYSMSIZE = consts_SMSetting 52
sM_CXVSCROLL :: SMSetting
sM_CXVSCROLL = consts_SMSetting 53
sM_CYHSCROLL :: SMSetting
sM_CYHSCROLL = consts_SMSetting 54
sM_CYVTHUMB :: SMSetting
sM_CYVTHUMB = consts_SMSetting 55
sM_CYCAPTION :: SMSetting
sM_CYCAPTION = consts_SMSetting 56
sM_CYKANJIWINDOW :: SMSetting
sM_CYKANJIWINDOW = consts_SMSetting 57
sM_CYMENU :: SMSetting
sM_CYMENU = consts_SMSetting 58
sM_CYSMCAPTION :: SMSetting
sM_CYSMCAPTION = consts_SMSetting 59
sM_DBCSENABLED :: SMSetting
sM_DBCSENABLED = consts_SMSetting 60
sM_DEBUG :: SMSetting
sM_DEBUG = consts_SMSetting 61
sM_MENUDROPALIGNMENT :: SMSetting
sM_MENUDROPALIGNMENT = consts_SMSetting 62
sM_MIDEASTENABLED :: SMSetting
sM_MIDEASTENABLED = consts_SMSetting 63
sM_MOUSEPRESENT :: SMSetting
sM_MOUSEPRESENT = consts_SMSetting 64
sM_NETWORK :: SMSetting
sM_NETWORK = consts_SMSetting 65
sM_PENWINDOWS :: SMSetting
sM_PENWINDOWS = consts_SMSetting 66
sM_SECURE :: SMSetting
sM_SECURE = consts_SMSetting 67
sM_SHOWSOUNDS :: SMSetting
sM_SHOWSOUNDS = consts_SMSetting 68
sM_SLOWMACHINE :: SMSetting
sM_SLOWMACHINE = consts_SMSetting 69
sM_SWAPBUTTON :: SMSetting
sM_SWAPBUTTON = consts_SMSetting 70
newtype MenuName = MenuName LPCSTR deriving ()
type MenuID = UINT
type MbMenuName = Maybe MenuName
primitive mkMenuName "XS_mkMenuName" :: PString -> MenuName
primitive atomToMenuName "XS_atomToMenuName" :: ATOM -> MenuName
primitive isNoMenuItem "XStest_isNoMenuItem" :: IOError -> Bool
primitive isNoMenuID "XStest_isNoMenuID" :: IOError -> Bool
primitive isBadMenu "XStest_isBadMenu" :: IOError -> Bool
type GMDIFlag = UINT
gMDI_USEDISABLED :: GMDIFlag
gMDI_USEDISABLED = consts_GMDIFlag 0
gMDI_GOINTOPOPUPS :: GMDIFlag
gMDI_GOINTOPOPUPS = consts_GMDIFlag 1
type MenuFlag = UINT
mF_BYCOMMAND :: MenuFlag
mF_BYCOMMAND = consts_MenuFlag 0
mF_BYPOSITION :: MenuFlag
mF_BYPOSITION = consts_MenuFlag 1
newtype MenuItem = MenuItem UINT deriving ()
mF_INSERT :: MenuItem
mF_INSERT = consts_MenuItem 0
mF_CHANGE :: MenuItem
mF_CHANGE = consts_MenuItem 1
mF_APPEND :: MenuItem
mF_APPEND = consts_MenuItem 2
mF_DELETE :: MenuItem
mF_DELETE = consts_MenuItem 3
mF_REMOVE :: MenuItem
mF_REMOVE = consts_MenuItem 4
mF_USECHECKBITMAPS :: MenuItem
mF_USECHECKBITMAPS = consts_MenuItem 5
mF_POPUP :: MenuItem
mF_POPUP = consts_MenuItem 6
mF_SYSMENU :: MenuItem
mF_SYSMENU = consts_MenuItem 7
mF_HELP :: MenuItem
mF_HELP = consts_MenuItem 8
mF_MOUSESELECT :: MenuItem
mF_MOUSESELECT = consts_MenuItem 9
mF_END :: MenuItem
mF_END = consts_MenuItem 10
type MenuFlagT = MenuFlag
mFT_STRING :: MenuFlagT
mFT_STRING = consts_MenuFlagT 0
mFT_BITMAP :: MenuFlagT
mFT_BITMAP = consts_MenuFlagT 1
mFT_MENUBARBREAK :: MenuFlagT
mFT_MENUBARBREAK = consts_MenuFlagT 2
mFT_MENUBREAK :: MenuFlagT
mFT_MENUBREAK = consts_MenuFlagT 3
mFT_OWNERDRAW :: MenuFlagT
mFT_OWNERDRAW = consts_MenuFlagT 4
mFT_RADIOCHECK :: MenuFlagT
mFT_RADIOCHECK = consts_MenuFlagT 5
mFT_SEPARATOR :: MenuFlagT
mFT_SEPARATOR = consts_MenuFlagT 6
mFT_RIGHTORDER :: MenuFlagT
mFT_RIGHTORDER = consts_MenuFlagT 7
mFT_RIGHTJUSTIFY :: MenuFlagT
mFT_RIGHTJUSTIFY = consts_MenuFlagT 8
type MenuState = MenuFlag
mFS_GRAYED :: MenuState
mFS_GRAYED = consts_MenuState 0
mFS_DISABLED :: MenuState
mFS_DISABLED = consts_MenuState 1
mFS_CHECKED :: MenuState
mFS_CHECKED = consts_MenuState 2
mFS_HILITE :: MenuState
mFS_HILITE = consts_MenuState 3
mFS_ENABLED :: MenuState
mFS_ENABLED = consts_MenuState 4
mFS_UNCHECKED :: MenuState
mFS_UNCHECKED = consts_MenuState 5
mFS_UNHILITE :: MenuState
mFS_UNHILITE = consts_MenuState 6
mFS_DEFAULT :: MenuState
mFS_DEFAULT = consts_MenuState 7
type TrackMenuFlag = UINT
tPM_LEFTBUTTON :: TrackMenuFlag
tPM_LEFTBUTTON = consts_TrackMenuFlag 0
tPM_RIGHTBUTTON :: TrackMenuFlag
tPM_RIGHTBUTTON = consts_TrackMenuFlag 1
tPM_LEFTALIGN :: TrackMenuFlag
tPM_LEFTALIGN = consts_TrackMenuFlag 2
tPM_CENTERALIGN :: TrackMenuFlag
tPM_CENTERALIGN = consts_TrackMenuFlag 3
tPM_RIGHTALIGN :: TrackMenuFlag
tPM_RIGHTALIGN = consts_TrackMenuFlag 4
tPM_TOPALIGN :: TrackMenuFlag
tPM_TOPALIGN = consts_TrackMenuFlag 5
tPM_VCENTERALIGN :: TrackMenuFlag
tPM_VCENTERALIGN = consts_TrackMenuFlag 6
tPM_BOTTOMALIGN :: TrackMenuFlag
tPM_BOTTOMALIGN = consts_TrackMenuFlag 7
tPM_HORIZONTAL :: TrackMenuFlag
tPM_HORIZONTAL = consts_TrackMenuFlag 8
tPM_VERTICAL :: TrackMenuFlag
tPM_VERTICAL = consts_TrackMenuFlag 9
tPM_NONOTIFY :: TrackMenuFlag
tPM_NONOTIFY = consts_TrackMenuFlag 10
tPM_RETURNCMD :: TrackMenuFlag
tPM_RETURNCMD = consts_TrackMenuFlag 11
newtype SystemMenuCommand = SystemMenuCommand UINT deriving (Eq)
sC_SIZE :: SystemMenuCommand
sC_SIZE = consts_SystemMenuCommand 0
sC_MOVE :: SystemMenuCommand
sC_MOVE = consts_SystemMenuCommand 1
sC_MINIMIZE :: SystemMenuCommand
sC_MINIMIZE = consts_SystemMenuCommand 2
sC_MAXIMIZE :: SystemMenuCommand
sC_MAXIMIZE = consts_SystemMenuCommand 3
sC_NEXTWINDOW :: SystemMenuCommand
sC_NEXTWINDOW = consts_SystemMenuCommand 4
sC_PREVWINDOW :: SystemMenuCommand
sC_PREVWINDOW = consts_SystemMenuCommand 5
sC_CLOSE :: SystemMenuCommand
sC_CLOSE = consts_SystemMenuCommand 6
sC_VSCROLL :: SystemMenuCommand
sC_VSCROLL = consts_SystemMenuCommand 7
sC_HSCROLL :: SystemMenuCommand
sC_HSCROLL = consts_SystemMenuCommand 8
sC_MOUSEMENU :: SystemMenuCommand
sC_MOUSEMENU = consts_SystemMenuCommand 9
sC_KEYMENU :: SystemMenuCommand
sC_KEYMENU = consts_SystemMenuCommand 10
sC_ARRANGE :: SystemMenuCommand
sC_ARRANGE = consts_SystemMenuCommand 11
sC_RESTORE :: SystemMenuCommand
sC_RESTORE = consts_SystemMenuCommand 12
sC_TASKLIST :: SystemMenuCommand
sC_TASKLIST = consts_SystemMenuCommand 13
sC_SCREENSAVE :: SystemMenuCommand
sC_SCREENSAVE = consts_SystemMenuCommand 14
sC_HOTKEY :: SystemMenuCommand
sC_HOTKEY = consts_SystemMenuCommand 15
sC_DEFAULT :: SystemMenuCommand
sC_DEFAULT = consts_SystemMenuCommand 16
sC_MONITORPOWER :: SystemMenuCommand
sC_MONITORPOWER = consts_SystemMenuCommand 17
sC_CONTEXTHELP :: SystemMenuCommand
sC_CONTEXTHELP = consts_SystemMenuCommand 18
sC_SEPARATOR :: SystemMenuCommand
sC_SEPARATOR = consts_SystemMenuCommand 19
sC_ICON :: SystemMenuCommand
sC_ICON = consts_SystemMenuCommand 20
sC_ZOOM :: SystemMenuCommand
sC_ZOOM = consts_SystemMenuCommand 21
primitive isMenu "XS_IsMenu" :: HMENU -> IO Bool
primitive getSystemMenu "XS_GetSystemMenu" :: HWND  -> Bool ->     IO MbHMENU
primitive getMenu "XS_GetMenu" :: HWND  ->             IO MbHMENU
primitive getSubMenu "XS_GetSubMenu" :: HMENU -> MenuItem -> IO MbHMENU
primitive setMenu "XS_SetMenu" :: HWND -> HMENU -> IO ()
primitive getMenuItemCount "XS_GetMenuItemCount" :: HMENU -> IO Int
primitive getMenuItemID "XS_GetMenuItemID" :: HMENU -> MenuItem -> IO MenuID
primitive getMenuState "XS_GetMenuState" :: HMENU -> MenuItem -> MenuFlag -> IO MenuState
primitive getMenuDefaultItem "XS_GetMenuDefaultItem" :: HMENU -> Bool -> GMDIFlag -> IO MenuItem
primitive setMenuDefaultItem "XS_SetMenuDefaultItem" :: HMENU -> MenuItem -> Bool -> IO ()
primitive checkMenuItem "XS_CheckMenuItem" :: HMENU -> MenuItem -> MenuFlag -> IO DWORD
primitive checkMenuRadioItem "XS_CheckMenuRadioItem" :: HMENU -> MenuItem -> MenuItem -> MenuItem -> MenuFlag -> IO DWORD
primitive enableMenuItem "XS_EnableMenuItem" :: HMENU -> MenuItem -> MenuFlag -> IO MenuState
primitive hiliteMenuItem "XS_HiliteMenuItem" :: HWND  -> HMENU -> MenuItem -> MenuFlag -> IO Bool
primitive setMenuItemBitmaps "XS_SetMenuItemBitmaps" :: HMENU -> MenuItem -> MenuFlag -> MbHBITMAP -> MbHBITMAP -> IO ()
primitive createMenu "XS_CreateMenu" :: IO HMENU
primitive createPopupMenu "XS_CreatePopupMenu" :: IO HMENU
primitive destroyMenu "XS_DestroyMenu" :: HMENU -> IO ()
primitive deleteMenu "XS_DeleteMenu" :: HMENU -> MenuItem -> MenuFlag -> IO ()
primitive removeMenu "XS_RemoveMenu" :: HMENU -> MenuItem -> MenuFlag -> IO ()
primitive appendMenu "XS_AppendMenu" :: HMENU -> MenuFlag -> MenuID -> String -> IO ()
primitive insertMenu "XS_InsertMenu" :: HMENU -> MenuItem -> MenuFlag -> MenuID -> String -> IO ()
primitive modifyMenu "XS_ModifyMenu" :: HMENU -> MenuItem -> MenuFlag -> MenuID -> String -> IO ()
primitive drawMenuBar "XS_DrawMenuBar" :: HWND -> IO ()
primitive trackPopupMenu "XS_TrackPopupMenu" :: HMENU -> TrackMenuFlag -> Int -> Int -> HWND -> RECT -> IO () 
primitive moveToEx "XS_moveToEx" :: HDC -> INT -> INT -> IO POINT 
primitive lineTo "XS_LineTo" :: HDC -> INT -> INT -> IO ()
primitive polyline "XS_polyline" :: HDC -> ListPOINT  -> IO ()    
primitive polylineTo "XS_polylineTo" :: HDC -> ListPOINT  -> IO ()    
primitive polygon "XS_polygon" :: HDC -> ListPOINT  -> IO ()    
primitive polyBezier "XS_polyBezier" :: HDC -> ListPOINT -> IO ()   
primitive polyBezierTo "XS_polyBezierTo" :: HDC -> ListPOINT -> IO ()   
primitive fillRect "XS_fillRect" :: HDC -> RECT -> HBRUSH -> IO () 
primitive frameRect "XS_frameRect" :: HDC -> RECT -> HBRUSH -> IO () 
primitive invertRect "XS_invertRect" :: HDC -> RECT -> IO ()           
primitive rectangle "XS_Rectangle" :: HDC -> INT -> INT -> INT -> INT -> IO ()
primitive roundRect "XS_RoundRect" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
primitive ellipse "XS_Ellipse" :: HDC -> INT -> INT -> INT -> INT -> IO ()
primitive arc "XS_Arc" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
primitive arcTo "XS_ArcTo" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
primitive angleArc "XS_AngleArc" :: HDC -> INT -> INT -> DWORD -> Float -> Float -> IO ()
primitive chord "XS_Chord" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
primitive pie "XS_Pie" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
newtype RegionType = RegionType INT deriving (Eq)
eRROR :: RegionType
eRROR = consts_RegionType 0
nULLREGION :: RegionType
nULLREGION = consts_RegionType 1
sIMPLEREGION :: RegionType
sIMPLEREGION = consts_RegionType 2
cOMPLEXREGION :: RegionType
cOMPLEXREGION = consts_RegionType 3
newtype ClippingMode = ClippingMode INT deriving (Eq)
rGN_AND :: ClippingMode
rGN_AND = consts_ClippingMode 0
rGN_OR :: ClippingMode
rGN_OR = consts_ClippingMode 1
rGN_XOR :: ClippingMode
rGN_XOR = consts_ClippingMode 2
rGN_DIFF :: ClippingMode
rGN_DIFF = consts_ClippingMode 3
rGN_COPY :: ClippingMode
rGN_COPY = consts_ClippingMode 4
newtype StockBrush = StockBrush INT deriving ()
newtype StockPen = StockPen INT deriving ()
newtype StockFont = StockFont INT deriving ()
newtype StockPalette = StockPalette INT deriving ()
wHITE_BRUSH :: StockBrush
wHITE_BRUSH = consts_StockBrush 0
lTGRAY_BRUSH :: StockBrush
lTGRAY_BRUSH = consts_StockBrush 1
gRAY_BRUSH :: StockBrush
gRAY_BRUSH = consts_StockBrush 2
dKGRAY_BRUSH :: StockBrush
dKGRAY_BRUSH = consts_StockBrush 3
bLACK_BRUSH :: StockBrush
bLACK_BRUSH = consts_StockBrush 4
nULL_BRUSH :: StockBrush
nULL_BRUSH = consts_StockBrush 5
hOLLOW_BRUSH :: StockBrush
hOLLOW_BRUSH = consts_StockBrush 6
wHITE_PEN :: StockPen
wHITE_PEN = consts_StockPen 0
bLACK_PEN :: StockPen
bLACK_PEN = consts_StockPen 1
nULL_PEN :: StockPen
nULL_PEN = consts_StockPen 2
oEM_FIXED_FONT :: StockFont
oEM_FIXED_FONT = consts_StockFont 0
aNSI_FIXED_FONT :: StockFont
aNSI_FIXED_FONT = consts_StockFont 1
aNSI_VAR_FONT :: StockFont
aNSI_VAR_FONT = consts_StockFont 2
sYSTEM_FONT :: StockFont
sYSTEM_FONT = consts_StockFont 3
dEVICE_DEFAULT_FONT :: StockFont
dEVICE_DEFAULT_FONT = consts_StockFont 4
sYSTEM_FIXED_FONT :: StockFont
sYSTEM_FIXED_FONT = consts_StockFont 5
dEFAULT_PALETTE :: StockPalette
dEFAULT_PALETTE = consts_StockPalette 0
primitive getStockBrush "XS_getStockBrush" :: StockBrush   -> IO HBRUSH   
primitive getStockFont "XS_getStockFont" :: StockFont    -> IO HFONT    
primitive getStockPalette "XS_getStockPalette" :: StockPalette -> IO HPALETTE 
primitive getStockPen "XS_getStockPen" :: StockPen     -> IO HPEN     
primitive deleteBrush "XS_deleteBrush" :: HBRUSH   -> IO () 
primitive deleteBitmap "XS_deleteBitmap" :: HBITMAP  -> IO () 
primitive deleteFont "XS_deleteFont" :: HFONT    -> IO () 
primitive deletePalette "XS_deletePalette" :: HPALETTE -> IO () 
primitive deletePen "XS_deletePen" :: HPEN     -> IO () 
primitive deleteRegion "XS_deleteRegion" :: HRGN     -> IO () 
primitive getCurrentBitmap "XS_getCurrentBitmap" :: HDC -> IO HBITMAP  
primitive getCurrentBrush "XS_getCurrentBrush" :: HDC -> IO HBRUSH   
primitive getCurrentFont "XS_getCurrentFont" :: HDC -> IO HFONT    
primitive getCurrentPalette "XS_getCurrentPalette" :: HDC -> IO HPALETTE 
primitive getCurrentPen "XS_getCurrentPen" :: HDC -> IO HPEN     
primitive selectBitmap "XS_selectBitmap" :: HDC -> HBITMAP -> IO HBITMAP    
primitive selectBrush "XS_selectBrush" :: HDC -> HBRUSH  -> IO HBRUSH     
primitive selectFont "XS_selectFont" :: HDC -> HFONT   -> IO HFONT      
primitive selectPen "XS_selectPen" :: HDC -> HPEN    -> IO HPEN       
primitive selectRgn "XS_selectRgn" :: HDC -> HRGN    -> IO RegionType 
primitive selectClipRgn "XS_SelectClipRgn" :: HDC -> MbHRGN  -> IO RegionType
primitive selectClipPath "XS_SelectClipPath" :: HDC -> ClippingMode -> IO RegionType
primitive extSelectClipRgn "XS_ExtSelectClipRgn" :: HDC -> HRGN    -> ClippingMode -> IO RegionType
primitive selectPalette "XS_SelectPalette" :: HDC -> HPALETTE -> Bool -> IO HPALETTE
primitive cancelDC "XS_CancelDC" :: HDC -> IO ()
primitive createCompatibleDC "XS_CreateCompatibleDC" :: MbHDC -> IO HDC
primitive deleteDC "XS_DeleteDC" :: HDC   -> IO ()
primitive getDC "XS_GetDC" :: MbHWND -> IO HDC
primitive getWindowDC "XS_GetWindowDC" :: MbHWND -> IO HWND
primitive releaseDC "XS_ReleaseDC" :: MbHWND -> HDC -> IO ()
newtype GetDCExFlags = GetDCExFlags DWORD deriving ()
dCX_WINDOW :: GetDCExFlags
dCX_WINDOW = consts_GetDCExFlags 0
dCX_CACHE :: GetDCExFlags
dCX_CACHE = consts_GetDCExFlags 1
dCX_CLIPCHILDREN :: GetDCExFlags
dCX_CLIPCHILDREN = consts_GetDCExFlags 2
dCX_CLIPSIBLINGS :: GetDCExFlags
dCX_CLIPSIBLINGS = consts_GetDCExFlags 3
dCX_PARENTCLIP :: GetDCExFlags
dCX_PARENTCLIP = consts_GetDCExFlags 4
dCX_EXCLUDERGN :: GetDCExFlags
dCX_EXCLUDERGN = consts_GetDCExFlags 5
dCX_INTERSECTRGN :: GetDCExFlags
dCX_INTERSECTRGN = consts_GetDCExFlags 6
dCX_LOCKWINDOWUPDATE :: GetDCExFlags
dCX_LOCKWINDOWUPDATE = consts_GetDCExFlags 7
primitive getDCEx "XS_GetDCEx" :: HWND -> HRGN -> GetDCExFlags -> IO HDC
primitive getDCOrgEx "XS_GetDCOrgEx" :: HDC  -> IO POINT 
newtype GraphicsMode = GraphicsMode INT deriving (Eq)
type MbGraphicsMode = Maybe GraphicsMode
gM_COMPATIBLE :: GraphicsMode
gM_COMPATIBLE = consts_GraphicsMode 0
gM_ADVANCED :: GraphicsMode
gM_ADVANCED = consts_GraphicsMode 1
primitive getGraphicsMode "XS_GetGraphicsMode" :: HDC -> IO MbGraphicsMode
primitive setGraphicsMode "XS_SetGraphicsMode" :: HDC -> GraphicsMode -> IO MbGraphicsMode
primitive saveDC "XS_SaveDC" :: HDC -> IO INT
primitive restoreDC "XS_RestoreDC" :: HDC -> INT -> IO ()
newtype ArcDirection = ArcDirection INT deriving (Eq)
type MbArcDirection = Maybe ArcDirection
aD_COUNTERCLOCKWISE :: ArcDirection
aD_COUNTERCLOCKWISE = consts_ArcDirection 0
aD_CLOCKWISE :: ArcDirection
aD_CLOCKWISE = consts_ArcDirection 1
primitive setArcDirection "XS_SetArcDirection" :: HDC -> ArcDirection -> IO ArcDirection
newtype PolyFillMode = PolyFillMode INT deriving (Eq)
type MbPolyFillMode = Maybe PolyFillMode
aLTERNATE :: PolyFillMode
aLTERNATE = consts_PolyFillMode 0
wINDING :: PolyFillMode
wINDING = consts_PolyFillMode 1
primitive getPolyFillMode "XS_GetPolyFillMode" :: HDC -> IO PolyFillMode
primitive setPolyFillMode "XS_SetPolyFillMode" :: HDC -> PolyFillMode -> IO PolyFillMode
primitive combineRgn "XS_CombineRgn" :: HRGN -> HRGN -> HRGN -> ClippingMode -> IO RegionType
primitive offsetRgn "XS_OffsetRgn" :: HRGN -> INT -> INT -> IO RegionType
primitive getRgnBox "XS_GetRgnBox" :: HRGN -> LPRECT -> IO RegionType
primitive createEllipticRgn "XS_CreateEllipticRgn" :: INT -> INT -> INT -> INT -> IO HRGN
primitive createEllipticRgnIndirect "XS_CreateEllipticRgnIndirect" :: LPRECT ->                   IO HRGN
primitive createRectRgn "XS_CreateRectRgn" :: INT -> INT -> INT -> INT -> IO HRGN
primitive createRectRgnIndirect "XS_CreateRectRgnIndirect" :: LPRECT ->                   IO HRGN
primitive createRoundRectRgn "XS_CreateRoundRectRgn" :: INT -> INT -> INT -> INT -> INT -> INT -> IO HRGN
primitive createPolygonRgn "XS_CreatePolygonRgn" :: ListPOINT -> PolyFillMode -> IO HRGN 
primitive equalRgn "XS_EqualRgn" :: HRGN -> HRGN -> IO Bool
primitive fillRgn "XS_FillRgn" :: HDC -> HRGN -> HBRUSH -> IO ()
primitive invertRgn "XS_InvertRgn" :: HDC -> HRGN -> IO ()
primitive paintRgn "XS_PaintRgn" :: HDC -> HRGN -> IO ()
primitive frameRgn "XS_FrameRgn" :: HDC -> HRGN -> HBRUSH -> INT -> INT -> IO ()
primitive ptInRegion "XS_PtInRegion" :: HRGN -> INT -> INT -> IO Bool
primitive rectInRegion "XS_RectInRegion" :: HRGN -> RECT -> IO Bool
primitive abortPath "XS_AbortPath" :: HDC -> IO ()
primitive beginPath "XS_BeginPath" :: HDC -> IO ()
primitive closeFigure "XS_CloseFigure" :: HDC -> IO ()
primitive endPath "XS_EndPath" :: HDC -> IO ()
primitive fillPath "XS_FillPath" :: HDC -> IO ()
primitive flattenPath "XS_FlattenPath" :: HDC -> IO ()
primitive pathToRegion "XS_PathToRegion" :: HDC -> IO HRGN
primitive strokeAndFillPath "XS_StrokeAndFillPath" :: HDC -> IO ()
primitive strokePath "XS_StrokePath" :: HDC -> IO ()
primitive widenPath "XS_WidenPath" :: HDC -> IO ()
primitive getMiterLimit "XS_GetMiterLimit" :: HDC ->          IO FLOAT 
primitive setMiterLimit "XS_SetMiterLimit" :: HDC -> FLOAT -> IO FLOAT 
type XFORM = (FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT)
newtype COLORREF = COLORREF Word32 deriving ()
primitive rgb "XS_rgb" :: BYTE -> BYTE -> BYTE -> COLORREF
primitive getRValue "XS_GetRValue" :: COLORREF -> BYTE
primitive getGValue "XS_GetGValue" :: COLORREF -> BYTE
primitive getBValue "XS_GetBValue" :: COLORREF -> BYTE
primitive pALETTERGB "XS_PALETTERGB" :: BYTE -> BYTE -> BYTE -> COLORREF
newtype PaletteIndex = PaletteIndex WORD deriving ()
primitive pALETTEINDEX "XS_PALETTEINDEX" :: PaletteIndex -> COLORREF
newtype HatchStyle = HatchStyle INT deriving (Eq)
hS_HORIZONTAL :: HatchStyle
hS_HORIZONTAL = consts_HatchStyle 0
hS_VERTICAL :: HatchStyle
hS_VERTICAL = consts_HatchStyle 1
hS_FDIAGONAL :: HatchStyle
hS_FDIAGONAL = consts_HatchStyle 2
hS_BDIAGONAL :: HatchStyle
hS_BDIAGONAL = consts_HatchStyle 3
hS_CROSS :: HatchStyle
hS_CROSS = consts_HatchStyle 4
hS_DIAGCROSS :: HatchStyle
hS_DIAGCROSS = consts_HatchStyle 5
primitive setBrushOrgEx "XS_setBrushOrgEx" :: HDC -> INT -> INT -> IO (Maybe POINT)
primitive getBrushOrgEx "XS_getBrushOrgEx" :: HDC        -> IO POINT 
primitive createSolidBrush "XS_CreateSolidBrush" :: COLORREF   -> IO HBRUSH
primitive createHatchBrush "XS_CreateHatchBrush" :: HatchStyle -> COLORREF -> IO HBRUSH
primitive createPatternBrush "XS_CreatePatternBrush" :: HBITMAP    -> IO HBRUSH
newtype PenStyle = PenStyle INT deriving (Eq)
instance BitSet PenStyle where
  toWord32   = \ (PenStyle x) -> x
  fromWord32 = PenStyle
pS_SOLID :: PenStyle
pS_SOLID = consts_PenStyle 0
pS_DASH :: PenStyle
pS_DASH = consts_PenStyle 1
pS_DOT :: PenStyle
pS_DOT = consts_PenStyle 2
pS_DASHDOT :: PenStyle
pS_DASHDOT = consts_PenStyle 3
pS_DASHDOTDOT :: PenStyle
pS_DASHDOTDOT = consts_PenStyle 4
pS_NULL :: PenStyle
pS_NULL = consts_PenStyle 5
pS_INSIDEFRAME :: PenStyle
pS_INSIDEFRAME = consts_PenStyle 6
pS_USERSTYLE :: PenStyle
pS_USERSTYLE = consts_PenStyle 7
pS_ALTERNATE :: PenStyle
pS_ALTERNATE = consts_PenStyle 8
pS_STYLE_MASK :: PenStyle
pS_STYLE_MASK = consts_PenStyle 9
pS_ENDCAP_ROUND :: PenStyle
pS_ENDCAP_ROUND = consts_PenStyle 10
pS_ENDCAP_SQUARE :: PenStyle
pS_ENDCAP_SQUARE = consts_PenStyle 11
pS_ENDCAP_FLAT :: PenStyle
pS_ENDCAP_FLAT = consts_PenStyle 12
pS_ENDCAP_MASK :: PenStyle
pS_ENDCAP_MASK = consts_PenStyle 13
pS_JOIN_ROUND :: PenStyle
pS_JOIN_ROUND = consts_PenStyle 14
pS_JOIN_BEVEL :: PenStyle
pS_JOIN_BEVEL = consts_PenStyle 15
pS_JOIN_MITER :: PenStyle
pS_JOIN_MITER = consts_PenStyle 16
pS_JOIN_MASK :: PenStyle
pS_JOIN_MASK = consts_PenStyle 17
pS_COSMETIC :: PenStyle
pS_COSMETIC = consts_PenStyle 18
pS_GEOMETRIC :: PenStyle
pS_GEOMETRIC = consts_PenStyle 19
pS_TYPE_MASK :: PenStyle
pS_TYPE_MASK = consts_PenStyle 20
primitive createPen "XS_CreatePen" :: PenStyle -> INT -> COLORREF -> IO HPEN
oBM_CLOSE :: Bitmap
oBM_CLOSE = consts_Bitmap 0
oBM_UPARROW :: Bitmap
oBM_UPARROW = consts_Bitmap 1
oBM_DNARROW :: Bitmap
oBM_DNARROW = consts_Bitmap 2
oBM_RGARROW :: Bitmap
oBM_RGARROW = consts_Bitmap 3
oBM_LFARROW :: Bitmap
oBM_LFARROW = consts_Bitmap 4
oBM_REDUCE :: Bitmap
oBM_REDUCE = consts_Bitmap 5
oBM_ZOOM :: Bitmap
oBM_ZOOM = consts_Bitmap 6
oBM_RESTORE :: Bitmap
oBM_RESTORE = consts_Bitmap 7
oBM_REDUCED :: Bitmap
oBM_REDUCED = consts_Bitmap 8
oBM_ZOOMD :: Bitmap
oBM_ZOOMD = consts_Bitmap 9
oBM_RESTORED :: Bitmap
oBM_RESTORED = consts_Bitmap 10
oBM_UPARROWD :: Bitmap
oBM_UPARROWD = consts_Bitmap 11
oBM_DNARROWD :: Bitmap
oBM_DNARROWD = consts_Bitmap 12
oBM_RGARROWD :: Bitmap
oBM_RGARROWD = consts_Bitmap 13
oBM_LFARROWD :: Bitmap
oBM_LFARROWD = consts_Bitmap 14
oBM_MNARROW :: Bitmap
oBM_MNARROW = consts_Bitmap 15
oBM_COMBO :: Bitmap
oBM_COMBO = consts_Bitmap 16
oBM_UPARROWI :: Bitmap
oBM_UPARROWI = consts_Bitmap 17
oBM_DNARROWI :: Bitmap
oBM_DNARROWI = consts_Bitmap 18
oBM_RGARROWI :: Bitmap
oBM_RGARROWI = consts_Bitmap 19
oBM_LFARROWI :: Bitmap
oBM_LFARROWI = consts_Bitmap 20
oBM_OLD_CLOSE :: Bitmap
oBM_OLD_CLOSE = consts_Bitmap 21
oBM_SIZE :: Bitmap
oBM_SIZE = consts_Bitmap 22
oBM_OLD_UPARROW :: Bitmap
oBM_OLD_UPARROW = consts_Bitmap 23
oBM_OLD_DNARROW :: Bitmap
oBM_OLD_DNARROW = consts_Bitmap 24
oBM_OLD_RGARROW :: Bitmap
oBM_OLD_RGARROW = consts_Bitmap 25
oBM_OLD_LFARROW :: Bitmap
oBM_OLD_LFARROW = consts_Bitmap 26
oBM_BTSIZE :: Bitmap
oBM_BTSIZE = consts_Bitmap 27
oBM_CHECK :: Bitmap
oBM_CHECK = consts_Bitmap 28
oBM_CHECKBOXES :: Bitmap
oBM_CHECKBOXES = consts_Bitmap 29
oBM_BTNCORNERS :: Bitmap
oBM_BTNCORNERS = consts_Bitmap 30
oBM_OLD_REDUCE :: Bitmap
oBM_OLD_REDUCE = consts_Bitmap 31
oBM_OLD_ZOOM :: Bitmap
oBM_OLD_ZOOM = consts_Bitmap 32
oBM_OLD_RESTORE :: Bitmap
oBM_OLD_RESTORE = consts_Bitmap 33
primitive createCompatibleBitmap "XS_CreateCompatibleBitmap" :: HDC -> INT -> INT -> IO HBITMAP
newtype RasterOp3 = RasterOp3 DWORD deriving ()
primitive mkRasterOp3 "XS_mkRasterOp3" :: DWORD -> RasterOp3
sRCCOPY :: RasterOp3
sRCCOPY = consts_RasterOp3 0
sRCPAINT :: RasterOp3
sRCPAINT = consts_RasterOp3 1
sRCAND :: RasterOp3
sRCAND = consts_RasterOp3 2
sRCINVERT :: RasterOp3
sRCINVERT = consts_RasterOp3 3
sRCERASE :: RasterOp3
sRCERASE = consts_RasterOp3 4
nOTSRCCOPY :: RasterOp3
nOTSRCCOPY = consts_RasterOp3 5
nOTSRCERASE :: RasterOp3
nOTSRCERASE = consts_RasterOp3 6
mERGECOPY :: RasterOp3
mERGECOPY = consts_RasterOp3 7
mERGEPAINT :: RasterOp3
mERGEPAINT = consts_RasterOp3 8
pATCOPY :: RasterOp3
pATCOPY = consts_RasterOp3 9
pATPAINT :: RasterOp3
pATPAINT = consts_RasterOp3 10
pATINVERT :: RasterOp3
pATINVERT = consts_RasterOp3 11
dSTINVERT :: RasterOp3
dSTINVERT = consts_RasterOp3 12
bLACKNESS :: RasterOp3
bLACKNESS = consts_RasterOp3 13
wHITENESS :: RasterOp3
wHITENESS = consts_RasterOp3 14
primitive bitBlt "XS_BitBlt" :: HDC -> INT -> INT -> INT -> INT -> HDC -> INT -> INT -> RasterOp3 -> IO ()
newtype RasterOp4 = RasterOp4 DWORD deriving ()
primitive mAKEROP4 "XS_MAKEROP4" :: RasterOp3 -> RasterOp3 -> RasterOp4
primitive maskBlt "XS_MaskBlt" :: HDC -> INT -> INT -> INT -> INT -> HDC -> INT -> INT -> HBITMAP -> INT -> INT -> RasterOp4 -> IO ()
primitive stretchBlt "XS_StretchBlt" :: HDC -> INT -> INT -> INT -> INT -> HDC -> INT -> INT -> INT -> INT -> RasterOp3 -> IO ()
primitive patBlt "XS_PatBlt" :: HDC -> INT -> INT -> INT -> INT -> RasterOp3 -> IO ()
primitive plgBlt "XS_plgBlt" :: HDC -> POINT -> POINT -> POINT -> HDC -> INT -> INT -> INT -> INT -> MbHBITMAP -> INT -> INT -> IO ()
primitive getBitmapDimensionEx "XS_getBitmapDimensionEx" :: HBITMAP ->         IO SIZE 
primitive setBitmapDimensionEx "XS_setBitmapDimensionEx" :: HBITMAP -> SIZE -> IO SIZE 
type RGBQUAD = (BYTE,BYTE,BYTE,BYTE)
type LPRGBQUAD = Ptr in
  toLPRGBQUAD   :: Ptr -> LPRGBQUAD,
  fromLPRGBQUAD :: LPRGBQUAD -> Ptr

toLPRGBQUAD   x = x 
fromLPRGBQUAD x = x

instance Pointer LPRGBQUAD where
  toPtr     = fromLPRGBQUAD
  fromPtr   = toLPRGBQUAD
  alloc     = malloc sizeofLPRGBQUAD
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPRGBQUAD))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPRGBQUAD

primitive getLPRGBQUAD        "XS_get_LPRGBQUAD"    :: LPRGBQUAD -> IO RGBQUAD
primitive setLPRGBQUAD        "XS_set_LPRGBQUAD"    :: LPRGBQUAD -> RGBQUAD -> IO ()
sizeofLPRGBQUAD :: Int
sizeofLPRGBQUAD = consts_Int 5
type BITMAP = (LONG,LONG,LONG,LONG,WORD,WORD)
type LPBITMAP = Ptr in
  toLPBITMAP   :: Ptr -> LPBITMAP,
  fromLPBITMAP :: LPBITMAP -> Ptr

toLPBITMAP   x = x 
fromLPBITMAP x = x

instance Pointer LPBITMAP where
  toPtr     = fromLPBITMAP
  fromPtr   = toLPBITMAP
  alloc     = malloc sizeofLPBITMAP
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPBITMAP))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPBITMAP

primitive getLPBITMAP        "XS_get_LPBITMAP"    :: LPBITMAP -> IO BITMAP
primitive setLPBITMAP        "XS_set_LPBITMAP"    :: LPBITMAP -> BITMAP -> IO ()
sizeofLPBITMAP :: Int
sizeofLPBITMAP = consts_Int 6
primitive bmBits "XS_bmBits" :: LPBITMAP -> LPVOID
primitive getBitmapInfo "XS_getBitmapInfo" :: HBITMAP -> IO BITMAP
newtype BitmapCompression = BitmapCompression DWORD deriving (Eq)
bI_RGB :: BitmapCompression
bI_RGB = consts_BitmapCompression 0
bI_RLE8 :: BitmapCompression
bI_RLE8 = consts_BitmapCompression 1
bI_RLE4 :: BitmapCompression
bI_RLE4 = consts_BitmapCompression 2
bI_BITFIELDS :: BitmapCompression
bI_BITFIELDS = consts_BitmapCompression 3
type BITMAPINFOHEADER = (DWORD,LONG,LONG,WORD,WORD,BitmapCompression,DWORD,LONG,LONG,MbDWORD,MbDWORD)
type LPBITMAPINFOHEADER = Ptr in
  toLPBITMAPINFOHEADER   :: Ptr -> LPBITMAPINFOHEADER,
  fromLPBITMAPINFOHEADER :: LPBITMAPINFOHEADER -> Ptr

toLPBITMAPINFOHEADER   x = x 
fromLPBITMAPINFOHEADER x = x

instance Pointer LPBITMAPINFOHEADER where
  toPtr     = fromLPBITMAPINFOHEADER
  fromPtr   = toLPBITMAPINFOHEADER
  alloc     = malloc sizeofLPBITMAPINFOHEADER
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPBITMAPINFOHEADER))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPBITMAPINFOHEADER

primitive getLPBITMAPINFOHEADER        "XS_get_LPBITMAPINFOHEADER"    :: LPBITMAPINFOHEADER -> IO BITMAPINFOHEADER
primitive setLPBITMAPINFOHEADER        "XS_set_LPBITMAPINFOHEADER"    :: LPBITMAPINFOHEADER -> BITMAPINFOHEADER -> IO ()
sizeofLPBITMAPINFOHEADER :: Int
sizeofLPBITMAPINFOHEADER = consts_Int 7
type BITMAPINFO = (BITMAPINFOHEADER)
type LPBITMAPINFO = Ptr in
  toLPBITMAPINFO   :: Ptr -> LPBITMAPINFO,
  fromLPBITMAPINFO :: LPBITMAPINFO -> Ptr

toLPBITMAPINFO   x = x 
fromLPBITMAPINFO x = x

instance Pointer LPBITMAPINFO where
  toPtr     = fromLPBITMAPINFO
  fromPtr   = toLPBITMAPINFO
  alloc     = malloc sizeofLPBITMAPINFO
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPBITMAPINFO))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPBITMAPINFO

primitive getLPBITMAPINFO        "XS_get_LPBITMAPINFO"    :: LPBITMAPINFO -> IO BITMAPINFO
primitive setLPBITMAPINFO        "XS_set_LPBITMAPINFO"    :: LPBITMAPINFO -> BITMAPINFO -> IO ()
sizeofLPBITMAPINFO :: Int
sizeofLPBITMAPINFO = consts_Int 8
primitive bmiColors "XS_bmiColors" :: LPBITMAPINFO -> LPRGBQUAD
type BITMAPFILEHEADER = (UINT,DWORD,UINT,UINT,DWORD)
type LPBITMAPFILEHEADER = Ptr in
  toLPBITMAPFILEHEADER   :: Ptr -> LPBITMAPFILEHEADER,
  fromLPBITMAPFILEHEADER :: LPBITMAPFILEHEADER -> Ptr

toLPBITMAPFILEHEADER   x = x 
fromLPBITMAPFILEHEADER x = x

instance Pointer LPBITMAPFILEHEADER where
  toPtr     = fromLPBITMAPFILEHEADER
  fromPtr   = toLPBITMAPFILEHEADER
  alloc     = malloc sizeofLPBITMAPFILEHEADER
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPBITMAPFILEHEADER))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPBITMAPFILEHEADER

primitive getLPBITMAPFILEHEADER        "XS_get_LPBITMAPFILEHEADER"    :: LPBITMAPFILEHEADER -> IO BITMAPFILEHEADER
primitive setLPBITMAPFILEHEADER        "XS_set_LPBITMAPFILEHEADER"    :: LPBITMAPFILEHEADER -> BITMAPFILEHEADER -> IO ()
sizeofLPBITMAPFILEHEADER :: Int
sizeofLPBITMAPFILEHEADER = consts_Int 9
primitive createBitmap "XS_CreateBitmap" :: INT -> INT -> UINT -> UINT -> MbLPVOID -> IO HBITMAP
primitive createBitmapIndirect "XS_CreateBitmapIndirect" :: LPBITMAP -> IO HBITMAP
type ColorFormat = UINT
dIB_PAL_COLORS :: ColorFormat
dIB_PAL_COLORS = consts_ColorFormat 0
dIB_RGB_COLORS :: ColorFormat
dIB_RGB_COLORS = consts_ColorFormat 1
primitive getDIBits "XS_GetDIBits" :: HDC -> HBITMAP -> UINT -> UINT -> MbLPVOID -> LPBITMAPINFO -> ColorFormat -> IO INT
primitive setDIBits "XS_SetDIBits" :: HDC -> HBITMAP -> UINT -> UINT -> LPVOID -> LPBITMAPINFO -> ColorFormat -> IO INT
primitive setDIBColorTable "XS_SetDIBColorTable" :: HDC -> UINT -> UINT -> LPRGBQUAD -> IO UINT
primitive getDIBColorTable "XS_GetDIBColorTable" :: HDC -> UINT -> UINT -> LPRGBQUAD -> IO UINT
cBM_INIT :: DWORD
cBM_INIT = consts_DWORD 0
primitive createDIBitmap "XS_CreateDIBitmap" :: HDC -> LPBITMAPINFOHEADER -> DWORD -> LPVOID -> LPBITMAPINFO -> ColorFormat -> IO HBITMAP
primitive createDIBPatternBrushPt "XS_CreateDIBPatternBrushPt" :: LPVOID -> ColorFormat -> IO HBRUSH
primitive createDIBSection "XS_CreateDIBSection" :: HDC -> LPBITMAPINFO -> ColorFormat -> MbHANDLE -> DWORD -> IO (HBITMAP, LPVOID) 
primitive setDIBitsToDevice "XS_SetDIBitsToDevice" :: HDC -> INT -> INT -> DWORD -> DWORD -> INT -> INT -> UINT -> UINT -> LPVOID -> LPBITMAPINFO -> ColorFormat -> IO INT
primitive stretchDIBits "XS_StretchDIBits" :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> LPVOID -> LPBITMAPINFO -> ColorFormat -> RasterOp3 -> IO INT
type StretchBltMode = INT
type MbStretchBltMode = Maybe StretchBltMode
bLACKONWHITE :: StretchBltMode
bLACKONWHITE = consts_StretchBltMode 0
wHITEONBLACK :: StretchBltMode
wHITEONBLACK = consts_StretchBltMode 1
cOLORONCOLOR :: StretchBltMode
cOLORONCOLOR = consts_StretchBltMode 2
hALFTONE :: StretchBltMode
hALFTONE = consts_StretchBltMode 3
sTRETCH_ANDSCANS :: StretchBltMode
sTRETCH_ANDSCANS = consts_StretchBltMode 4
sTRETCH_ORSCANS :: StretchBltMode
sTRETCH_ORSCANS = consts_StretchBltMode 5
sTRETCH_DELETESCANS :: StretchBltMode
sTRETCH_DELETESCANS = consts_StretchBltMode 6
primitive setStretchBltMode "XS_SetStretchBltMode" :: HDC -> StretchBltMode -> IO StretchBltMode
primitive getStretchBltMode "XS_GetStretchBltMode" :: HDC -> IO StretchBltMode
type FontWeight = INT
fW_DONTCARE :: FontWeight
fW_DONTCARE = consts_FontWeight 0
fW_THIN :: FontWeight
fW_THIN = consts_FontWeight 1
fW_EXTRALIGHT :: FontWeight
fW_EXTRALIGHT = consts_FontWeight 2
fW_LIGHT :: FontWeight
fW_LIGHT = consts_FontWeight 3
fW_NORMAL :: FontWeight
fW_NORMAL = consts_FontWeight 4
fW_MEDIUM :: FontWeight
fW_MEDIUM = consts_FontWeight 5
fW_SEMIBOLD :: FontWeight
fW_SEMIBOLD = consts_FontWeight 6
fW_BOLD :: FontWeight
fW_BOLD = consts_FontWeight 7
fW_EXTRABOLD :: FontWeight
fW_EXTRABOLD = consts_FontWeight 8
fW_HEAVY :: FontWeight
fW_HEAVY = consts_FontWeight 9
fW_ULTRALIGHT :: FontWeight
fW_ULTRALIGHT = consts_FontWeight 10
fW_REGULAR :: FontWeight
fW_REGULAR = consts_FontWeight 11
fW_DEMIBOLD :: FontWeight
fW_DEMIBOLD = consts_FontWeight 12
fW_ULTRABOLD :: FontWeight
fW_ULTRABOLD = consts_FontWeight 13
fW_BLACK :: FontWeight
fW_BLACK = consts_FontWeight 14
newtype CharSet = CharSet BYTE deriving (Eq)
aNSI_CHARSET :: CharSet
aNSI_CHARSET = consts_CharSet 0
dEFAULT_CHARSET :: CharSet
dEFAULT_CHARSET = consts_CharSet 1
sYMBOL_CHARSET :: CharSet
sYMBOL_CHARSET = consts_CharSet 2
sHIFTJIS_CHARSET :: CharSet
sHIFTJIS_CHARSET = consts_CharSet 3
hANGEUL_CHARSET :: CharSet
hANGEUL_CHARSET = consts_CharSet 4
cHINESEBIG5_CHARSET :: CharSet
cHINESEBIG5_CHARSET = consts_CharSet 5
oEM_CHARSET :: CharSet
oEM_CHARSET = consts_CharSet 6
newtype OutPrecision = OutPrecision BYTE deriving (Eq)
oUT_DEFAULT_PRECIS :: OutPrecision
oUT_DEFAULT_PRECIS = consts_OutPrecision 0
oUT_STRING_PRECIS :: OutPrecision
oUT_STRING_PRECIS = consts_OutPrecision 1
oUT_CHARACTER_PRECIS :: OutPrecision
oUT_CHARACTER_PRECIS = consts_OutPrecision 2
oUT_STROKE_PRECIS :: OutPrecision
oUT_STROKE_PRECIS = consts_OutPrecision 3
oUT_TT_PRECIS :: OutPrecision
oUT_TT_PRECIS = consts_OutPrecision 4
oUT_DEVICE_PRECIS :: OutPrecision
oUT_DEVICE_PRECIS = consts_OutPrecision 5
oUT_RASTER_PRECIS :: OutPrecision
oUT_RASTER_PRECIS = consts_OutPrecision 6
oUT_TT_ONLY_PRECIS :: OutPrecision
oUT_TT_ONLY_PRECIS = consts_OutPrecision 7
newtype ClipPrecision = ClipPrecision BYTE deriving (Eq)
instance BitSet ClipPrecision where
  toWord32   = \ (ClipPrecision x) -> x
  fromWord32 = ClipPrecision
cLIP_DEFAULT_PRECIS :: ClipPrecision
cLIP_DEFAULT_PRECIS = consts_ClipPrecision 0
cLIP_CHARACTER_PRECIS :: ClipPrecision
cLIP_CHARACTER_PRECIS = consts_ClipPrecision 1
cLIP_STROKE_PRECIS :: ClipPrecision
cLIP_STROKE_PRECIS = consts_ClipPrecision 2
cLIP_MASK :: ClipPrecision
cLIP_MASK = consts_ClipPrecision 3
cLIP_LH_ANGLES :: ClipPrecision
cLIP_LH_ANGLES = consts_ClipPrecision 4
cLIP_TT_ALWAYS :: ClipPrecision
cLIP_TT_ALWAYS = consts_ClipPrecision 5
cLIP_EMBEDDED :: ClipPrecision
cLIP_EMBEDDED = consts_ClipPrecision 6
newtype FontQuality = FontQuality BYTE deriving (Eq)
dEFAULT_QUALITY :: FontQuality
dEFAULT_QUALITY = consts_FontQuality 0
dRAFT_QUALITY :: FontQuality
dRAFT_QUALITY = consts_FontQuality 1
pROOF_QUALITY :: FontQuality
pROOF_QUALITY = consts_FontQuality 2
newtype PitchAndFamily = PitchAndFamily BYTE deriving (Eq)
dEFAULT_PITCH :: PitchAndFamily
dEFAULT_PITCH = consts_PitchAndFamily 0
fIXED_PITCH :: PitchAndFamily
fIXED_PITCH = consts_PitchAndFamily 1
vARIABLE_PITCH :: PitchAndFamily
vARIABLE_PITCH = consts_PitchAndFamily 2
fF_DONTCARE :: PitchAndFamily
fF_DONTCARE = consts_PitchAndFamily 3
fF_ROMAN :: PitchAndFamily
fF_ROMAN = consts_PitchAndFamily 4
fF_SWISS :: PitchAndFamily
fF_SWISS = consts_PitchAndFamily 5
fF_MODERN :: PitchAndFamily
fF_MODERN = consts_PitchAndFamily 6
fF_SCRIPT :: PitchAndFamily
fF_SCRIPT = consts_PitchAndFamily 7
fF_DECORATIVE :: PitchAndFamily
fF_DECORATIVE = consts_PitchAndFamily 8
familyMask :: PitchAndFamily
familyMask = consts_PitchAndFamily 9
pitchMask :: PitchAndFamily
pitchMask = consts_PitchAndFamily 10
lF_FACESIZE :: Int
lF_FACESIZE = consts_Int 10
type FaceName = [Char]
type LOGFONT = (INT,INT,INT,INT,FontWeight,Bool,Bool,Bool,CharSet,OutPrecision,ClipPrecision,FontQuality,PitchAndFamily,FaceName)
type LPLOGFONT = Ptr in
  toLPLOGFONT   :: Ptr -> LPLOGFONT,
  fromLPLOGFONT :: LPLOGFONT -> Ptr

toLPLOGFONT   x = x 
fromLPLOGFONT x = x

instance Pointer LPLOGFONT where
  toPtr     = fromLPLOGFONT
  fromPtr   = toLPLOGFONT
  alloc     = malloc sizeofLPLOGFONT
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPLOGFONT))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPLOGFONT

primitive getLPLOGFONT        "XS_get_LPLOGFONT"    :: LPLOGFONT -> IO LOGFONT
primitive setLPLOGFONT        "XS_set_LPLOGFONT"    :: LPLOGFONT -> LOGFONT -> IO ()
sizeofLPLOGFONT :: Int
sizeofLPLOGFONT = consts_Int 11
type MbLPLOGFONT = Maybe LPLOGFONT
primitive createFont "XS_CreateFont" :: INT -> INT -> INT -> INT -> FontWeight -> Bool -> Bool -> Bool -> CharSet -> OutPrecision -> ClipPrecision -> FontQuality -> PitchAndFamily -> FaceName -> IO HFONT
primitive createFontIndirect "XS_CreateFontIndirect" :: LPLOGFONT -> IO HFONT
primitive setBkColor "XS_SetBkColor" :: HDC -> COLORREF -> IO COLORREF
primitive getBkColor "XS_GetBkColor" :: HDC ->             IO COLORREF
primitive setTextColor "XS_SetTextColor" :: HDC -> COLORREF -> IO COLORREF
primitive getTextColor "XS_GetTextColor" :: HDC ->             IO COLORREF
newtype BackgroundMode = BackgroundMode INT deriving (Eq)
tRANSPARENT :: BackgroundMode
tRANSPARENT = consts_BackgroundMode 0
oPAQUE :: BackgroundMode
oPAQUE = consts_BackgroundMode 1
primitive setBkMode "XS_SetBkMode" :: HDC -> BackgroundMode -> IO BackgroundMode
primitive getBkMode "XS_GetBkMode" :: HDC ->                   IO BackgroundMode
newtype TextAlignment = TextAlignment UINT deriving (Eq)
instance BitSet TextAlignment where
  toWord32   = \ (TextAlignment x) -> x
  fromWord32 = TextAlignment
tA_NOUPDATECP :: TextAlignment
tA_NOUPDATECP = consts_TextAlignment 0
tA_UPDATECP :: TextAlignment
tA_UPDATECP = consts_TextAlignment 1
tA_LEFT :: TextAlignment
tA_LEFT = consts_TextAlignment 2
tA_RIGHT :: TextAlignment
tA_RIGHT = consts_TextAlignment 3
tA_CENTER :: TextAlignment
tA_CENTER = consts_TextAlignment 4
tA_TOP :: TextAlignment
tA_TOP = consts_TextAlignment 5
tA_BOTTOM :: TextAlignment
tA_BOTTOM = consts_TextAlignment 6
tA_BASELINE :: TextAlignment
tA_BASELINE = consts_TextAlignment 7
primitive setTextAlign "XS_SetTextAlign" :: HDC -> TextAlignment -> IO TextAlignment
primitive getTextAlign "XS_GetTextAlign" :: HDC ->                  IO TextAlignment
primitive isCharSpacingFailed "XStest_isCharSpacingFailed" :: IOError -> Bool
primitive setTextCharacterExtra "XS_SetTextCharacterExtra" :: HDC -> INT -> IO INT
primitive getTextCharacterExtra "XS_GetTextCharacterExtra" :: HDC ->        IO INT
primitive getTextExtentPoint32 "XS_getTextExtentPoint32" :: HDC -> String -> IO SIZE 
primitive getTabbedTextExtent "XS_getTabbedTextExtent" :: HDC -> String -> ListINT -> IO (WORD,WORD)
primitive setTextJustification "XS_SetTextJustification" :: HDC -> MbINT -> INT -> IO ()
type TechAndFamily = PitchAndFamily
techMask :: TechAndFamily
techMask = consts_TechAndFamily 0
tMPF_FIXED_PITCH :: TechAndFamily
tMPF_FIXED_PITCH = consts_TechAndFamily 1
tMPF_VECTOR :: TechAndFamily
tMPF_VECTOR = consts_TechAndFamily 2
tMPF_DEVICE :: TechAndFamily
tMPF_DEVICE = consts_TechAndFamily 3
tMPF_TRUETYPE :: TechAndFamily
tMPF_TRUETYPE = consts_TechAndFamily 4
type TEXTMETRIC = (INT,INT,INT,INT,INT,INT,INT,FontWeight,Bool,Bool,Bool,Char,Char,Char,Char,TechAndFamily,BYTE,INT,INT,INT)
primitive getTextMetrics "XS_GetTextMetrics" :: HDC -> IO TEXTMETRIC 
primitive textOut "XS_TextOut" :: HDC -> INT -> INT -> String -> IO () 
primitive tabbedTextOut "XS_TabbedTextOut" :: HDC -> INT -> INT -> String -> ListINT -> INT -> IO (WORD,WORD)
newtype DrawTextFormat = DrawTextFormat UINT deriving (Eq)
instance BitSet DrawTextFormat where
  toWord32   = \ (DrawTextFormat x) -> x
  fromWord32 = DrawTextFormat
dT_TOP :: DrawTextFormat
dT_TOP = consts_DrawTextFormat 0
dT_LEFT :: DrawTextFormat
dT_LEFT = consts_DrawTextFormat 1
dT_CENTER :: DrawTextFormat
dT_CENTER = consts_DrawTextFormat 2
dT_RIGHT :: DrawTextFormat
dT_RIGHT = consts_DrawTextFormat 3
dT_VCENTER :: DrawTextFormat
dT_VCENTER = consts_DrawTextFormat 4
dT_BOTTOM :: DrawTextFormat
dT_BOTTOM = consts_DrawTextFormat 5
dT_WORDBREAK :: DrawTextFormat
dT_WORDBREAK = consts_DrawTextFormat 6
dT_SINGLELINE :: DrawTextFormat
dT_SINGLELINE = consts_DrawTextFormat 7
dT_EXPANDTABS :: DrawTextFormat
dT_EXPANDTABS = consts_DrawTextFormat 8
dT_TABSTOP :: DrawTextFormat
dT_TABSTOP = consts_DrawTextFormat 9
dT_NOCLIP :: DrawTextFormat
dT_NOCLIP = consts_DrawTextFormat 10
dT_EXTERNALLEADING :: DrawTextFormat
dT_EXTERNALLEADING = consts_DrawTextFormat 11
dT_CALCRECT :: DrawTextFormat
dT_CALCRECT = consts_DrawTextFormat 12
dT_NOPREFIX :: DrawTextFormat
dT_NOPREFIX = consts_DrawTextFormat 13
dT_INTERNAL :: DrawTextFormat
dT_INTERNAL = consts_DrawTextFormat 14
primitive drawText "XS_DrawText" :: HDC -> String -> RECT -> DrawTextFormat -> IO INT
type AccessMode = DWORD
gENERIC_NONE :: AccessMode
gENERIC_NONE = consts_AccessMode 0
gENERIC_READ :: AccessMode
gENERIC_READ = consts_AccessMode 1
gENERIC_WRITE :: AccessMode
gENERIC_WRITE = consts_AccessMode 2
gENERIC_EXECUTE :: AccessMode
gENERIC_EXECUTE = consts_AccessMode 3
gENERIC_ALL :: AccessMode
gENERIC_ALL = consts_AccessMode 4
dELETE :: AccessMode
dELETE = consts_AccessMode 5
rEAD_CONTROL :: AccessMode
rEAD_CONTROL = consts_AccessMode 6
wRITE_DAC :: AccessMode
wRITE_DAC = consts_AccessMode 7
wRITE_OWNER :: AccessMode
wRITE_OWNER = consts_AccessMode 8
sYNCHRONIZE :: AccessMode
sYNCHRONIZE = consts_AccessMode 9
sTANDARD_RIGHTS_REQUIRED :: AccessMode
sTANDARD_RIGHTS_REQUIRED = consts_AccessMode 10
sTANDARD_RIGHTS_READ :: AccessMode
sTANDARD_RIGHTS_READ = consts_AccessMode 11
sTANDARD_RIGHTS_WRITE :: AccessMode
sTANDARD_RIGHTS_WRITE = consts_AccessMode 12
sTANDARD_RIGHTS_EXECUTE :: AccessMode
sTANDARD_RIGHTS_EXECUTE = consts_AccessMode 13
sTANDARD_RIGHTS_ALL :: AccessMode
sTANDARD_RIGHTS_ALL = consts_AccessMode 14
sPECIFIC_RIGHTS_ALL :: AccessMode
sPECIFIC_RIGHTS_ALL = consts_AccessMode 15
aCCESS_SYSTEM_SECURITY :: AccessMode
aCCESS_SYSTEM_SECURITY = consts_AccessMode 16
mAXIMUM_ALLOWED :: AccessMode
mAXIMUM_ALLOWED = consts_AccessMode 17
type ShareMode = DWORD
fILE_SHARE_NONE :: ShareMode
fILE_SHARE_NONE = consts_ShareMode 0
fILE_SHARE_READ :: ShareMode
fILE_SHARE_READ = consts_ShareMode 1
fILE_SHARE_WRITE :: ShareMode
fILE_SHARE_WRITE = consts_ShareMode 2
newtype CreateMode = CreateMode DWORD deriving ()
cREATE_NEW :: CreateMode
cREATE_NEW = consts_CreateMode 0
cREATE_ALWAYS :: CreateMode
cREATE_ALWAYS = consts_CreateMode 1
oPEN_EXISTING :: CreateMode
oPEN_EXISTING = consts_CreateMode 2
oPEN_ALWAYS :: CreateMode
oPEN_ALWAYS = consts_CreateMode 3
tRUNCATE_EXISTING :: CreateMode
tRUNCATE_EXISTING = consts_CreateMode 4
type FileAttributeOrFlag = DWORD
fILE_ATTRIBUTE_READONLY :: FileAttributeOrFlag
fILE_ATTRIBUTE_READONLY = consts_FileAttributeOrFlag 0
fILE_ATTRIBUTE_HIDDEN :: FileAttributeOrFlag
fILE_ATTRIBUTE_HIDDEN = consts_FileAttributeOrFlag 1
fILE_ATTRIBUTE_SYSTEM :: FileAttributeOrFlag
fILE_ATTRIBUTE_SYSTEM = consts_FileAttributeOrFlag 2
fILE_ATTRIBUTE_DIRECTORY :: FileAttributeOrFlag
fILE_ATTRIBUTE_DIRECTORY = consts_FileAttributeOrFlag 3
fILE_ATTRIBUTE_ARCHIVE :: FileAttributeOrFlag
fILE_ATTRIBUTE_ARCHIVE = consts_FileAttributeOrFlag 4
fILE_ATTRIBUTE_NORMAL :: FileAttributeOrFlag
fILE_ATTRIBUTE_NORMAL = consts_FileAttributeOrFlag 5
fILE_ATTRIBUTE_TEMPORARY :: FileAttributeOrFlag
fILE_ATTRIBUTE_TEMPORARY = consts_FileAttributeOrFlag 6
fILE_ATTRIBUTE_COMPRESSED :: FileAttributeOrFlag
fILE_ATTRIBUTE_COMPRESSED = consts_FileAttributeOrFlag 7
fILE_FLAG_WRITE_THROUGH :: FileAttributeOrFlag
fILE_FLAG_WRITE_THROUGH = consts_FileAttributeOrFlag 8
fILE_FLAG_OVERLAPPED :: FileAttributeOrFlag
fILE_FLAG_OVERLAPPED = consts_FileAttributeOrFlag 9
fILE_FLAG_NO_BUFFERING :: FileAttributeOrFlag
fILE_FLAG_NO_BUFFERING = consts_FileAttributeOrFlag 10
fILE_FLAG_RANDOM_ACCESS :: FileAttributeOrFlag
fILE_FLAG_RANDOM_ACCESS = consts_FileAttributeOrFlag 11
fILE_FLAG_SEQUENTIAL_SCAN :: FileAttributeOrFlag
fILE_FLAG_SEQUENTIAL_SCAN = consts_FileAttributeOrFlag 12
fILE_FLAG_DELETE_ON_CLOSE :: FileAttributeOrFlag
fILE_FLAG_DELETE_ON_CLOSE = consts_FileAttributeOrFlag 13
fILE_FLAG_BACKUP_SEMANTICS :: FileAttributeOrFlag
fILE_FLAG_BACKUP_SEMANTICS = consts_FileAttributeOrFlag 14
fILE_FLAG_POSIX_SEMANTICS :: FileAttributeOrFlag
fILE_FLAG_POSIX_SEMANTICS = consts_FileAttributeOrFlag 15
sECURITY_ANONYMOUS :: FileAttributeOrFlag
sECURITY_ANONYMOUS = consts_FileAttributeOrFlag 16
sECURITY_IDENTIFICATION :: FileAttributeOrFlag
sECURITY_IDENTIFICATION = consts_FileAttributeOrFlag 17
sECURITY_IMPERSONATION :: FileAttributeOrFlag
sECURITY_IMPERSONATION = consts_FileAttributeOrFlag 18
sECURITY_DELEGATION :: FileAttributeOrFlag
sECURITY_DELEGATION = consts_FileAttributeOrFlag 19
sECURITY_CONTEXT_TRACKING :: FileAttributeOrFlag
sECURITY_CONTEXT_TRACKING = consts_FileAttributeOrFlag 20
sECURITY_EFFECTIVE_ONLY :: FileAttributeOrFlag
sECURITY_EFFECTIVE_ONLY = consts_FileAttributeOrFlag 21
sECURITY_SQOS_PRESENT :: FileAttributeOrFlag
sECURITY_SQOS_PRESENT = consts_FileAttributeOrFlag 22
sECURITY_VALID_SQOS_FLAGS :: FileAttributeOrFlag
sECURITY_VALID_SQOS_FLAGS = consts_FileAttributeOrFlag 23
newtype LPSECURITY_ATTRIBUTES = LPSECURITY_ATTRIBUTES Ptr deriving ()
type MbLPSECURITY_ATTRIBUTES = Maybe LPSECURITY_ATTRIBUTES
primitive createFile "XS_CreateFile" :: String -> AccessMode -> ShareMode -> MbLPSECURITY_ATTRIBUTES -> CreateMode -> FileAttributeOrFlag -> MbHANDLE -> IO HANDLE
primitive closeHandle "XS_CloseHandle" :: HANDLE -> IO ()
primitive deleteFile "XS_DeleteFile" :: String -> IO ()
primitive copyFile "XS_CopyFile" :: String -> String -> Bool -> IO ()
primitive moveFile "XS_MoveFile" :: String -> String -> IO ()
type MoveFileFlag = DWORD
mOVEFILE_REPLACE_EXISTING :: MoveFileFlag
mOVEFILE_REPLACE_EXISTING = consts_MoveFileFlag 0
mOVEFILE_COPY_ALLOWED :: MoveFileFlag
mOVEFILE_COPY_ALLOWED = consts_MoveFileFlag 1
mOVEFILE_DELAY_UNTIL_REBOOT :: MoveFileFlag
mOVEFILE_DELAY_UNTIL_REBOOT = consts_MoveFileFlag 2
primitive moveFileEx "XS_MoveFileEx" :: String -> String -> MoveFileFlag -> IO ()
newtype FilePtrDirection = FilePtrDirection DWORD deriving ()
fILE_BEGIN :: FilePtrDirection
fILE_BEGIN = consts_FilePtrDirection 0
fILE_CURRENT :: FilePtrDirection
fILE_CURRENT = consts_FilePtrDirection 1
fILE_END :: FilePtrDirection
fILE_END = consts_FilePtrDirection 2
primitive setFilePointer "XS_SetFilePointer" :: HANDLE -> Word64 -> FilePtrDirection -> IO Word64
type OVERLAPPED = (DWORD,DWORD,DWORD,DWORD,HANDLE)
type LPOVERLAPPED = Ptr in
  toLPOVERLAPPED   :: Ptr -> LPOVERLAPPED,
  fromLPOVERLAPPED :: LPOVERLAPPED -> Ptr

toLPOVERLAPPED   x = x 
fromLPOVERLAPPED x = x

instance Pointer LPOVERLAPPED where
  toPtr     = fromLPOVERLAPPED
  fromPtr   = toLPOVERLAPPED
  alloc     = malloc sizeofLPOVERLAPPED
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofLPOVERLAPPED))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofLPOVERLAPPED

primitive getLPOVERLAPPED        "XS_get_LPOVERLAPPED"    :: LPOVERLAPPED -> IO OVERLAPPED
primitive setLPOVERLAPPED        "XS_set_LPOVERLAPPED"    :: LPOVERLAPPED -> OVERLAPPED -> IO ()
sizeofLPOVERLAPPED :: Int
sizeofLPOVERLAPPED = consts_Int 12
type MbLPOVERLAPPED = Maybe LPOVERLAPPED
primitive readFile "XS_ReadFile" :: HANDLE -> LPVOID -> DWORD -> MbLPOVERLAPPED -> IO DWORD 
primitive writeFile "XS_WriteFile" :: HANDLE -> LPVOID -> DWORD -> MbLPOVERLAPPED -> IO DWORD 
primitive setEndOfFile "XS_SetEndOfFile" :: HANDLE -> IO ()
primitive flushFileBuffers "XS_FlushFileBuffers" :: HANDLE -> IO ()
primitive getFileAttributes "XS_GetFileAttributes" :: String -> IO FileAttributeOrFlag
primitive setFileAttributes "XS_SetFileAttributes" :: String -> FileAttributeOrFlag -> IO ()
newtype FileType = FileType DWORD deriving (Eq)
fILE_TYPE_UNKNOWN :: FileType
fILE_TYPE_UNKNOWN = consts_FileType 0
fILE_TYPE_DISK :: FileType
fILE_TYPE_DISK = consts_FileType 1
fILE_TYPE_CHAR :: FileType
fILE_TYPE_CHAR = consts_FileType 2
fILE_TYPE_PIPE :: FileType
fILE_TYPE_PIPE = consts_FileType 3
fILE_TYPE_REMOTE :: FileType
fILE_TYPE_REMOTE = consts_FileType 4
primitive getFileType "XS_GetFileType" :: HANDLE -> IO FileType
primitive createDirectory "XS_CreateDirectory" :: String -> MbLPSECURITY_ATTRIBUTES -> IO ()
primitive createDirectoryEx "XS_CreateDirectoryEx" :: String -> String -> MbLPSECURITY_ATTRIBUTES -> IO ()
primitive removeDirectory "XS_RemoveDirectory" :: String -> IO ()
primitive setCurrentDirectory "XS_SetCurrentDirectory" :: String -> IO ()
newtype FileNotificationFlag = FileNotificationFlag DWORD deriving (Eq)
instance BitSet FileNotificationFlag where
  toWord32   = \ (FileNotificationFlag x) -> x
  fromWord32 = FileNotificationFlag
fILE_NOTIFY_CHANGE_FILE_NAME :: FileNotificationFlag
fILE_NOTIFY_CHANGE_FILE_NAME = consts_FileNotificationFlag 0
fILE_NOTIFY_CHANGE_DIR_NAME :: FileNotificationFlag
fILE_NOTIFY_CHANGE_DIR_NAME = consts_FileNotificationFlag 1
fILE_NOTIFY_CHANGE_ATTRIBUTES :: FileNotificationFlag
fILE_NOTIFY_CHANGE_ATTRIBUTES = consts_FileNotificationFlag 2
fILE_NOTIFY_CHANGE_SIZE :: FileNotificationFlag
fILE_NOTIFY_CHANGE_SIZE = consts_FileNotificationFlag 3
fILE_NOTIFY_CHANGE_LAST_WRITE :: FileNotificationFlag
fILE_NOTIFY_CHANGE_LAST_WRITE = consts_FileNotificationFlag 4
fILE_NOTIFY_CHANGE_SECURITY :: FileNotificationFlag
fILE_NOTIFY_CHANGE_SECURITY = consts_FileNotificationFlag 5
primitive findFirstChangeNotification "XS_FindFirstChangeNotification" :: String -> Bool -> FileNotificationFlag -> IO HANDLE
primitive findNextChangeNotification "XS_FindNextChangeNotification" :: HANDLE -> IO ()
primitive findCloseChangeNotification "XS_FindCloseChangeNotification" :: HANDLE -> IO ()
primitive setVolumeLabel "XS_SetVolumeLabel" :: String -> String -> IO ()
primitive getDiskFreeSpace "XS_GetDiskFreeSpace" :: MbString -> IO (DWORD, DWORD, DWORD, DWORD) 
newtype DriveType = DriveType UINT deriving (Eq)
dRIVE_UNKNOWN :: DriveType
dRIVE_UNKNOWN = consts_DriveType 0
dRIVE_NO_ROOT_DIR :: DriveType
dRIVE_NO_ROOT_DIR = consts_DriveType 1
dRIVE_REMOVABLE :: DriveType
dRIVE_REMOVABLE = consts_DriveType 2
dRIVE_FIXED :: DriveType
dRIVE_FIXED = consts_DriveType 3
dRIVE_REMOTE :: DriveType
dRIVE_REMOTE = consts_DriveType 4
dRIVE_CDROM :: DriveType
dRIVE_CDROM = consts_DriveType 5
dRIVE_RAMDISK :: DriveType
dRIVE_RAMDISK = consts_DriveType 6
primitive getDriveType "XS_GetDriveType" :: MbString -> IO DriveType
primitive getLogicalDrives "XS_GetLogicalDrives" :: IO DWORD
primitive areFileApisANSI "XS_AreFileApisANSI" :: IO Bool
primitive setFileApisToOEM "XS_SetFileApisToOEM" :: IO ()
primitive setFileApisToANSI "XS_SetFileApisToANSI" :: IO ()
primitive setHandleCount "XS_SetHandleCount" :: UINT -> IO UINT
newtype DefineDosDeviceFlags = DefineDosDeviceFlags DWORD deriving ()
instance BitSet DefineDosDeviceFlags where
  toWord32   = \ (DefineDosDeviceFlags x) -> x
  fromWord32 = DefineDosDeviceFlags
dDD_RAW_TARGET_PATH :: DefineDosDeviceFlags
dDD_RAW_TARGET_PATH = consts_DefineDosDeviceFlags 0
dDD_REMOVE_DEFINITION :: DefineDosDeviceFlags
dDD_REMOVE_DEFINITION = consts_DefineDosDeviceFlags 1
dDD_EXACT_MATCH_ON_REMOVE :: DefineDosDeviceFlags
dDD_EXACT_MATCH_ON_REMOVE = consts_DefineDosDeviceFlags 2
primitive defineDosDevice "XS_DefineDosDevice" :: DefineDosDeviceFlags -> String -> String -> IO ()
newtype BinaryType = BinaryType DWORD deriving (Eq)
sCS_32BIT_BINARY :: BinaryType
sCS_32BIT_BINARY = consts_BinaryType 0
sCS_DOS_BINARY :: BinaryType
sCS_DOS_BINARY = consts_BinaryType 1
sCS_WOW_BINARY :: BinaryType
sCS_WOW_BINARY = consts_BinaryType 2
sCS_PIF_BINARY :: BinaryType
sCS_PIF_BINARY = consts_BinaryType 3
sCS_POSIX_BINARY :: BinaryType
sCS_POSIX_BINARY = consts_BinaryType 4
sCS_OS216_BINARY :: BinaryType
sCS_OS216_BINARY = consts_BinaryType 5
primitive getBinaryType "XS_GetBinaryType" :: String -> IO BinaryType 
newtype HKEY = HKEY NearPtr deriving (Eq)
type MbHKEY = Maybe HKEY
hKEY_CLASSES_ROOT :: HKEY
hKEY_CLASSES_ROOT = consts_HKEY 0
hKEY_CURRENT_USER :: HKEY
hKEY_CURRENT_USER = consts_HKEY 1
hKEY_LOCAL_MACHINE :: HKEY
hKEY_LOCAL_MACHINE = consts_HKEY 2
hKEY_USERS :: HKEY
hKEY_USERS = consts_HKEY 3
hKEY_PERFORMANCE_DATA :: HKEY
hKEY_PERFORMANCE_DATA = consts_HKEY 4
hKEY_CURRENT_CONFIG :: HKEY
hKEY_CURRENT_CONFIG = consts_HKEY 5
hKEY_DYN_DATA :: HKEY
hKEY_DYN_DATA = consts_HKEY 6
newtype KeyRestoreFlags = KeyRestoreFlags DWORD deriving (Eq)
rEG_WHOLE_HIVE_VOLATILE :: KeyRestoreFlags
rEG_WHOLE_HIVE_VOLATILE = consts_KeyRestoreFlags 0
rEG_REFRESH_HIVE :: KeyRestoreFlags
rEG_REFRESH_HIVE = consts_KeyRestoreFlags 1
rEG_NO_LAZY_FLUSH :: KeyRestoreFlags
rEG_NO_LAZY_FLUSH = consts_KeyRestoreFlags 2
newtype RegType = RegType DWORD deriving (Eq)
rEG_NONE :: RegType
rEG_NONE = consts_RegType 0
rEG_SZ :: RegType
rEG_SZ = consts_RegType 1
rEG_EXPAND_SZ :: RegType
rEG_EXPAND_SZ = consts_RegType 2
rEG_BINARY :: RegType
rEG_BINARY = consts_RegType 3
rEG_DWORD :: RegType
rEG_DWORD = consts_RegType 4
rEG_DWORD_LITTLE_ENDIAN :: RegType
rEG_DWORD_LITTLE_ENDIAN = consts_RegType 5
rEG_DWORD_BIG_ENDIAN :: RegType
rEG_DWORD_BIG_ENDIAN = consts_RegType 6
rEG_LINK :: RegType
rEG_LINK = consts_RegType 7
rEG_MULTI_SZ :: RegType
rEG_MULTI_SZ = consts_RegType 8
rEG_RESOURCE_LIST :: RegType
rEG_RESOURCE_LIST = consts_RegType 9
rEG_FULL_RESOURCE_DESCRIPTOR :: RegType
rEG_FULL_RESOURCE_DESCRIPTOR = consts_RegType 10
rEG_RESOURCE_REQUIREMENTS_LIST :: RegType
rEG_RESOURCE_REQUIREMENTS_LIST = consts_RegType 11
newtype RegistryOptions = RegistryOptions DWORD deriving (Eq)
instance BitSet RegistryOptions where
  toWord32   = \ (RegistryOptions x) -> x
  fromWord32 = RegistryOptions
rEG_OPTION_RESERVED :: RegistryOptions
rEG_OPTION_RESERVED = consts_RegistryOptions 0
rEG_OPTION_NON_VOLATILE :: RegistryOptions
rEG_OPTION_NON_VOLATILE = consts_RegistryOptions 1
rEG_OPTION_VOLATILE :: RegistryOptions
rEG_OPTION_VOLATILE = consts_RegistryOptions 2
rEG_OPTION_CREATE_LINK :: RegistryOptions
rEG_OPTION_CREATE_LINK = consts_RegistryOptions 3
rEG_OPTION_BACKUP_RESTORE :: RegistryOptions
rEG_OPTION_BACKUP_RESTORE = consts_RegistryOptions 4
rEG_OPTION_OPEN_LINK :: RegistryOptions
rEG_OPTION_OPEN_LINK = consts_RegistryOptions 5
rEG_LEGAL_OPTION :: RegistryOptions
rEG_LEGAL_OPTION = consts_RegistryOptions 6
newtype KeyDisposition = KeyDisposition DWORD deriving (Eq)
rEG_CREATED_NEW_KEY :: KeyDisposition
rEG_CREATED_NEW_KEY = consts_KeyDisposition 0
rEG_OPENED_EXISTING_KEY :: KeyDisposition
rEG_OPENED_EXISTING_KEY = consts_KeyDisposition 1
newtype REGSAM = REGSAM AccessMode deriving (Eq)
instance BitSet REGSAM where
  toWord32   = \ (REGSAM x) -> x
  fromWord32 = REGSAM
kEY_QUERY_VALUE :: REGSAM
kEY_QUERY_VALUE = consts_REGSAM 0
kEY_SET_VALUE :: REGSAM
kEY_SET_VALUE = consts_REGSAM 1
kEY_CREATE_SUB_KEY :: REGSAM
kEY_CREATE_SUB_KEY = consts_REGSAM 2
kEY_ENUMERATE_SUB_KEYS :: REGSAM
kEY_ENUMERATE_SUB_KEYS = consts_REGSAM 3
kEY_NOTIFY :: REGSAM
kEY_NOTIFY = consts_REGSAM 4
kEY_CREATE_LINK :: REGSAM
kEY_CREATE_LINK = consts_REGSAM 5
kEY_READ :: REGSAM
kEY_READ = consts_REGSAM 6
kEY_WRITE :: REGSAM
kEY_WRITE = consts_REGSAM 7
kEY_EXECUTE :: REGSAM
kEY_EXECUTE = consts_REGSAM 8
kEY_ALL_ACCESS :: REGSAM
kEY_ALL_ACCESS = consts_REGSAM 9
newtype RegistryNotifyFilter = RegistryNotifyFilter DWORD deriving ()
instance BitSet RegistryNotifyFilter where
  toWord32   = \ (RegistryNotifyFilter x) -> x
  fromWord32 = RegistryNotifyFilter
rEG_NOTIFY_CHANGE_NAME :: RegistryNotifyFilter
rEG_NOTIFY_CHANGE_NAME = consts_RegistryNotifyFilter 0
rEG_NOTIFY_CHANGE_ATTRIBUTES :: RegistryNotifyFilter
rEG_NOTIFY_CHANGE_ATTRIBUTES = consts_RegistryNotifyFilter 1
rEG_NOTIFY_CHANGE_LAST_SET :: RegistryNotifyFilter
rEG_NOTIFY_CHANGE_LAST_SET = consts_RegistryNotifyFilter 2
rEG_NOTIFY_CHANGE_SECURITY :: RegistryNotifyFilter
rEG_NOTIFY_CHANGE_SECURITY = consts_RegistryNotifyFilter 3
rEG_LEGAL_CHANGE_FILTER :: RegistryNotifyFilter
rEG_LEGAL_CHANGE_FILTER = consts_RegistryNotifyFilter 4
primitive regCloseKey "XS_RegCloseKey" :: HKEY -> IO ()
primitive regConnectRegistry "XS_RegConnectRegistry" :: MbString -> HKEY -> IO MbHKEY 
primitive regCreateKey "XS_RegCreateKey" :: HKEY -> String -> IO HKEY 
primitive regCreateKeyEx "XS_RegCreateKeyEx" :: HKEY -> String -> String -> RegistryOptions -> REGSAM -> MbLPSECURITY_ATTRIBUTES -> IO (HKEY, KeyDisposition) 
primitive regDeleteKey "XS_RegDeleteKey" :: HKEY -> String -> IO ()
primitive regDeleteValue "XS_RegDeleteValue" :: HKEY -> String -> IO ()
primitive regFlushKey "XS_RegFlushKey" :: HKEY -> IO ()
primitive regNotifyChangeKeyValue "XS_RegNotifyChangeKeyValue" :: HKEY -> Bool -> RegistryNotifyFilter -> MbHANDLE -> IO () 
primitive regOpenKey "XS_RegOpenKey" :: HKEY -> String ->           IO HKEY 
primitive regOpenKeyEx "XS_RegOpenKeyEx" :: HKEY -> String -> REGSAM -> IO HKEY 
primitive regQueryValueEx "XS_RegQueryValueEx" :: HKEY -> String -> LPBYTE -> IO (RegType, DWORD) 
primitive regSetValueEx "XS_RegSetValueEx" :: HKEY -> String -> RegType -> LPBYTE -> DWORD -> IO () 
primitive regLoadKey "XS_RegLoadKey" :: HKEY -> String -> String -> IO ()
primitive regReplaceKey "XS_RegReplaceKey" :: HKEY -> String -> String -> String -> IO ()
primitive regRestoreKey "XS_RegRestoreKey" :: HKEY -> String -> KeyRestoreFlags -> IO ()
primitive regSaveKey "XS_RegSaveKey" :: HKEY -> String -> LPSECURITY_ATTRIBUTES -> IO ()
primitive regUnLoadKey "XS_RegUnLoadKey" :: HKEY -> String -> IO ()
newtype VERSION = VERSION WORD deriving (Eq, Ord, Show)
newtype Manufacturer = Manufacturer UINT deriving (Eq)
newtype Product = Product UINT deriving (Eq)
mIDIERR_UNPREPARED :: WindowsError
mIDIERR_UNPREPARED = consts_WindowsError 0
mIDIERR_STILLPLAYING :: WindowsError
mIDIERR_STILLPLAYING = consts_WindowsError 1
mIDIERR_NOMAP :: WindowsError
mIDIERR_NOMAP = consts_WindowsError 2
mIDIERR_NOTREADY :: WindowsError
mIDIERR_NOTREADY = consts_WindowsError 3
mIDIERR_NODEVICE :: WindowsError
mIDIERR_NODEVICE = consts_WindowsError 4
mIDIERR_INVALIDSETUP :: WindowsError
mIDIERR_INVALIDSETUP = consts_WindowsError 5
mIDIERR_LASTERROR :: WindowsError
mIDIERR_LASTERROR = consts_WindowsError 6
newtype MMRESULT = MMRESULT UINT deriving (Eq)
mMSYSERR_NOERROR :: MMRESULT
mMSYSERR_NOERROR = consts_MMRESULT 0
mMSYSERR_ERROR :: MMRESULT
mMSYSERR_ERROR = consts_MMRESULT 1
mMSYSERR_BADDEVICEID :: MMRESULT
mMSYSERR_BADDEVICEID = consts_MMRESULT 2
mMSYSERR_NOTENABLED :: MMRESULT
mMSYSERR_NOTENABLED = consts_MMRESULT 3
mMSYSERR_ALLOCATED :: MMRESULT
mMSYSERR_ALLOCATED = consts_MMRESULT 4
mMSYSERR_INVALHANDLE :: MMRESULT
mMSYSERR_INVALHANDLE = consts_MMRESULT 5
mMSYSERR_NODRIVER :: MMRESULT
mMSYSERR_NODRIVER = consts_MMRESULT 6
mMSYSERR_NOMEM :: MMRESULT
mMSYSERR_NOMEM = consts_MMRESULT 7
mMSYSERR_NOTSUPPORTED :: MMRESULT
mMSYSERR_NOTSUPPORTED = consts_MMRESULT 8
mMSYSERR_BADERRNUM :: MMRESULT
mMSYSERR_BADERRNUM = consts_MMRESULT 9
mMSYSERR_INVALFLAG :: MMRESULT
mMSYSERR_INVALFLAG = consts_MMRESULT 10
mMSYSERR_INVALPARAM :: MMRESULT
mMSYSERR_INVALPARAM = consts_MMRESULT 11
mMSYSERR_HANDLEBUSY :: MMRESULT
mMSYSERR_HANDLEBUSY = consts_MMRESULT 12
mMSYSERR_INVALIDALIAS :: MMRESULT
mMSYSERR_INVALIDALIAS = consts_MMRESULT 13
mMSYSERR_BADDB :: MMRESULT
mMSYSERR_BADDB = consts_MMRESULT 14
mMSYSERR_KEYNOTFOUND :: MMRESULT
mMSYSERR_KEYNOTFOUND = consts_MMRESULT 15
mMSYSERR_READERROR :: MMRESULT
mMSYSERR_READERROR = consts_MMRESULT 16
mMSYSERR_WRITEERROR :: MMRESULT
mMSYSERR_WRITEERROR = consts_MMRESULT 17
mMSYSERR_DELETEERROR :: MMRESULT
mMSYSERR_DELETEERROR = consts_MMRESULT 18
mMSYSERR_VALNOTFOUND :: MMRESULT
mMSYSERR_VALNOTFOUND = consts_MMRESULT 19
mMSYSERR_NODRIVERCB :: MMRESULT
mMSYSERR_NODRIVERCB = consts_MMRESULT 20
mMSYSERR_LASTERROR :: MMRESULT
mMSYSERR_LASTERROR = consts_MMRESULT 21
newtype HMIDI = HMIDI HANDLE deriving ()
newtype HMIDIIN = HMIDIIN HANDLE deriving ()
newtype HMIDIOUT = HMIDIOUT HANDLE deriving ()
mIM_OPEN :: WindowMessage
mIM_OPEN = consts_WindowMessage 131
mIM_CLOSE :: WindowMessage
mIM_CLOSE = consts_WindowMessage 132
mIM_DATA :: WindowMessage
mIM_DATA = consts_WindowMessage 133
mIM_LONGDATA :: WindowMessage
mIM_LONGDATA = consts_WindowMessage 134
mIM_ERROR :: WindowMessage
mIM_ERROR = consts_WindowMessage 135
mIM_LONGERROR :: WindowMessage
mIM_LONGERROR = consts_WindowMessage 136
mOM_OPEN :: WindowMessage
mOM_OPEN = consts_WindowMessage 137
mOM_CLOSE :: WindowMessage
mOM_CLOSE = consts_WindowMessage 138
mOM_DONE :: WindowMessage
mOM_DONE = consts_WindowMessage 139
newtype MidiCachePolicy = MidiCachePolicy UINT deriving (Eq)
mIDI_CACHE_ALL :: MidiCachePolicy
mIDI_CACHE_ALL = consts_MidiCachePolicy 0
mIDI_CACHE_BESTFIT :: MidiCachePolicy
mIDI_CACHE_BESTFIT = consts_MidiCachePolicy 1
mIDI_CACHE_QUERY :: MidiCachePolicy
mIDI_CACHE_QUERY = consts_MidiCachePolicy 2
mIDI_UNCACHE :: MidiCachePolicy
mIDI_UNCACHE = consts_MidiCachePolicy 3
newtype MidiTechnology = MidiTechnology UINT deriving (Eq)
mOD_MIDIPORT :: MidiTechnology
mOD_MIDIPORT = consts_MidiTechnology 0
mOD_SYNTH :: MidiTechnology
mOD_SYNTH = consts_MidiTechnology 1
mOD_SQSYNTH :: MidiTechnology
mOD_SQSYNTH = consts_MidiTechnology 2
mOD_FMSYNTH :: MidiTechnology
mOD_FMSYNTH = consts_MidiTechnology 3
mOD_MAPPER :: MidiTechnology
mOD_MAPPER = consts_MidiTechnology 4
newtype MidiOutSupport = MidiOutSupport UINT deriving ()
instance BitSet MidiOutSupport where
  toWord32   = \ (MidiOutSupport x) -> x
  fromWord32 = MidiOutSupport
mIDICAPS_VOLUME :: MidiOutSupport
mIDICAPS_VOLUME = consts_MidiOutSupport 0
mIDICAPS_LRVOLUME :: MidiOutSupport
mIDICAPS_LRVOLUME = consts_MidiOutSupport 1
mIDICAPS_CACHE :: MidiOutSupport
mIDICAPS_CACHE = consts_MidiOutSupport 2
type MIDIOUTCAPS = (Manufacturer,Product,VERSION,String,MidiTechnology,UINT,UINT,UINT,MidiOutSupport)
type MIDIINCAPS = (Manufacturer,Product,VERSION,String)
type MidiDevice = UINT
mIDIMAPPER :: MidiDevice
mIDIMAPPER = consts_MidiDevice 0
mIDI_MAPPER :: MidiDevice
mIDI_MAPPER = consts_MidiDevice 1
primitive isMidiError "XStest_isMidiError" :: IOError -> Maybe (MMRESULT )
primitive midiOutGetNumDevs "XS_midiOutGetNumDevs" :: IO UINT
primitive midiOutGetDevCaps "XS_midiOutGetDevCaps" :: MidiDevice -> IO MIDIOUTCAPS
primitive midiOutGetVolume "XS_midiOutGetVolume" :: HMIDIOUT -> IO (WORD, WORD)
primitive midiOutSetVolume "XS_midiOutSetVolume" :: HMIDIOUT -> WORD -> WORD -> IO ()
primitive midiOutOpenNoCB "XS_midiOutOpenNoCB" :: MidiDevice -> IO HMIDIOUT
primitive midiOutClose "XS_midiOutClose" :: HMIDIOUT -> IO ()
primitive midiOutShortMsg "XS_midiOutShortMsg" :: HMIDIOUT -> DWORD -> IO ()
primitive midiOutReset "XS_midiOutReset" :: HMIDIOUT -> IO ()
primitive midiOutMessage "XS_midiOutMessage" :: HMIDIOUT -> UINT -> DWORD -> DWORD -> IO ()
primitive midiInGetNumDevs "XS_midiInGetNumDevs" :: IO ()
primitive midiInGetDevCaps "XS_midiInGetDevCaps" :: MidiDevice -> IO MIDIINCAPS
primitive midiInClose "XS_midiInClose" :: HMIDIIN -> IO ()
primitive midiInStart "XS_midiInStart" :: HMIDIIN -> IO ()
primitive midiInStop "XS_midiInStop" :: HMIDIIN -> IO ()
primitive midiInReset "XS_midiInReset" :: HMIDIIN -> IO ()
primitive midiInMessage "XS_midiInMessage" :: HMIDIIN -> UINT -> DWORD -> DWORD -> IO ()
primitive delay "XS_delay" :: Int -> Int -> IO ()
newtype DialogBoxStyle = DialogBoxStyle Int deriving (Eq)
dS_ABSALIGN :: DialogBoxStyle
dS_ABSALIGN = consts_DialogBoxStyle 0
dS_SYSMODAL :: DialogBoxStyle
dS_SYSMODAL = consts_DialogBoxStyle 1
dS_LOCALEDIT :: DialogBoxStyle
dS_LOCALEDIT = consts_DialogBoxStyle 2
dS_SETFONT :: DialogBoxStyle
dS_SETFONT = consts_DialogBoxStyle 3
dS_MODALFRAME :: DialogBoxStyle
dS_MODALFRAME = consts_DialogBoxStyle 4
dS_NOIDLEMSG :: DialogBoxStyle
dS_NOIDLEMSG = consts_DialogBoxStyle 5
primitive createDialogBox "XS_createDialogBox" :: String -> WindowStyle -> DialogBoxStyle -> MbPos -> MbPos -> MbPos -> MbPos -> HWND -> MbHMENU -> HANDLE -> IO HWND
newtype CommDlgError = CommDlgError DWORD deriving (Show)
primitive isCommDlgError "XStest_isCommDlgError" :: IOError -> Maybe (CommDlgError )
cDERR_DIALOGFAILURE :: CommDlgError
cDERR_DIALOGFAILURE = consts_CommDlgError 0
cDERR_GENERALCODES :: CommDlgError
cDERR_GENERALCODES = consts_CommDlgError 1
cDERR_STRUCTSIZE :: CommDlgError
cDERR_STRUCTSIZE = consts_CommDlgError 2
cDERR_INITIALIZATION :: CommDlgError
cDERR_INITIALIZATION = consts_CommDlgError 3
cDERR_NOTEMPLATE :: CommDlgError
cDERR_NOTEMPLATE = consts_CommDlgError 4
cDERR_NOHINSTANCE :: CommDlgError
cDERR_NOHINSTANCE = consts_CommDlgError 5
cDERR_LOADSTRFAILURE :: CommDlgError
cDERR_LOADSTRFAILURE = consts_CommDlgError 6
cDERR_FINDRESFAILURE :: CommDlgError
cDERR_FINDRESFAILURE = consts_CommDlgError 7
cDERR_LOADRESFAILURE :: CommDlgError
cDERR_LOADRESFAILURE = consts_CommDlgError 8
cDERR_LOCKRESFAILURE :: CommDlgError
cDERR_LOCKRESFAILURE = consts_CommDlgError 9
cDERR_MEMALLOCFAILURE :: CommDlgError
cDERR_MEMALLOCFAILURE = consts_CommDlgError 10
cDERR_MEMLOCKFAILURE :: CommDlgError
cDERR_MEMLOCKFAILURE = consts_CommDlgError 11
cDERR_NOHOOK :: CommDlgError
cDERR_NOHOOK = consts_CommDlgError 12
cDERR_REGISTERMSGFAIL :: CommDlgError
cDERR_REGISTERMSGFAIL = consts_CommDlgError 13
pDERR_PRINTERCODES :: CommDlgError
pDERR_PRINTERCODES = consts_CommDlgError 14
pDERR_SETUPFAILURE :: CommDlgError
pDERR_SETUPFAILURE = consts_CommDlgError 15
pDERR_PARSEFAILURE :: CommDlgError
pDERR_PARSEFAILURE = consts_CommDlgError 16
pDERR_RETDEFFAILURE :: CommDlgError
pDERR_RETDEFFAILURE = consts_CommDlgError 17
pDERR_LOADDRVFAILURE :: CommDlgError
pDERR_LOADDRVFAILURE = consts_CommDlgError 18
pDERR_GETDEVMODEFAIL :: CommDlgError
pDERR_GETDEVMODEFAIL = consts_CommDlgError 19
pDERR_INITFAILURE :: CommDlgError
pDERR_INITFAILURE = consts_CommDlgError 20
pDERR_NODEVICES :: CommDlgError
pDERR_NODEVICES = consts_CommDlgError 21
pDERR_NODEFAULTPRN :: CommDlgError
pDERR_NODEFAULTPRN = consts_CommDlgError 22
pDERR_DNDMMISMATCH :: CommDlgError
pDERR_DNDMMISMATCH = consts_CommDlgError 23
pDERR_CREATEICFAILURE :: CommDlgError
pDERR_CREATEICFAILURE = consts_CommDlgError 24
pDERR_PRINTERNOTFOUND :: CommDlgError
pDERR_PRINTERNOTFOUND = consts_CommDlgError 25
pDERR_DEFAULTDIFFERENT :: CommDlgError
pDERR_DEFAULTDIFFERENT = consts_CommDlgError 26
cFERR_CHOOSEFONTCODES :: CommDlgError
cFERR_CHOOSEFONTCODES = consts_CommDlgError 27
cFERR_NOFONTS :: CommDlgError
cFERR_NOFONTS = consts_CommDlgError 28
cFERR_MAXLESSTHANMIN :: CommDlgError
cFERR_MAXLESSTHANMIN = consts_CommDlgError 29
fNERR_FILENAMECODES :: CommDlgError
fNERR_FILENAMECODES = consts_CommDlgError 30
fNERR_SUBCLASSFAILURE :: CommDlgError
fNERR_SUBCLASSFAILURE = consts_CommDlgError 31
fNERR_INVALIDFILENAME :: CommDlgError
fNERR_INVALIDFILENAME = consts_CommDlgError 32
fNERR_BUFFERTOOSMALL :: CommDlgError
fNERR_BUFFERTOOSMALL = consts_CommDlgError 33
fRERR_FINDREPLACECODES :: CommDlgError
fRERR_FINDREPLACECODES = consts_CommDlgError 34
fRERR_BUFFERLENGTHZERO :: CommDlgError
fRERR_BUFFERLENGTHZERO = consts_CommDlgError 35
cCERR_CHOOSECOLORCODES :: CommDlgError
cCERR_CHOOSECOLORCODES = consts_CommDlgError 36
primitive createBMPFile "XS_CreateBMPFile" :: String -> HBITMAP -> HDC -> IO ()
type EZATTR = DWORD
eZ_ATTR_BOLD :: EZATTR
eZ_ATTR_BOLD = consts_EZATTR 0
eZ_ATTR_ITALIC :: EZATTR
eZ_ATTR_ITALIC = consts_EZATTR 1
eZ_ATTR_UNDERLINE :: EZATTR
eZ_ATTR_UNDERLINE = consts_EZATTR 2
eZ_ATTR_STRIKEOUT :: EZATTR
eZ_ATTR_STRIKEOUT = consts_EZATTR 3
primitive ezCreateFont "XS_EzCreateFont" :: HDC -> String -> Int -> Int -> EZATTR -> Bool -> IO HFONT
primitive loWord "XS_loWord" :: LPARAM -> Int
primitive hiWord "XS_hiWord" :: LPARAM -> Int
primitive reallyTimeGetTime "XS_reallyTimeGetTime" :: Int -> IO MilliSeconds
primitive timeBeginPeriod "XS_timeBeginPeriod" :: UINT -> IO UINT
primitive timeEndPeriod "XS_timeEndPeriod" :: UINT -> IO UINT
primitive transformedEllipse "XS_TransformedEllipse" :: HDC -> POINT -> POINT -> POINT -> IO ()
primitive consts_FontQuality "XSconsts_FontQuality" :: Int -> FontQuality
primitive consts_Word32 "XSconsts_Word32" :: Int -> Word32
primitive consts_RasterOp3 "XSconsts_RasterOp3" :: Int -> RasterOp3
primitive consts_MenuFlagT "XSconsts_MenuFlagT" :: Int -> MenuFlagT
primitive consts_ScrollbarCommand "XSconsts_ScrollbarCommand" :: Int -> ScrollbarCommand
primitive consts_StretchBltMode "XSconsts_StretchBltMode" :: Int -> StretchBltMode
primitive consts_MenuItem "XSconsts_MenuItem" :: Int -> MenuItem
primitive consts_RegistryOptions "XSconsts_RegistryOptions" :: Int -> RegistryOptions
primitive consts_ColorFormat "XSconsts_ColorFormat" :: Int -> ColorFormat
primitive consts_StockBrush "XSconsts_StockBrush" :: Int -> StockBrush
primitive consts_Icon "XSconsts_Icon" :: Int -> Icon
primitive consts_DialogBoxStyle "XSconsts_DialogBoxStyle" :: Int -> DialogBoxStyle
primitive consts_CharSet "XSconsts_CharSet" :: Int -> CharSet
primitive consts_ComboBoxStyle "XSconsts_ComboBoxStyle" :: Int -> ComboBoxStyle
primitive consts_PolyFillMode "XSconsts_PolyFillMode" :: Int -> PolyFillMode
primitive consts_HKEY "XSconsts_HKEY" :: Int -> HKEY
primitive consts_MidiDevice "XSconsts_MidiDevice" :: Int -> MidiDevice
primitive consts_ExitOption "XSconsts_ExitOption" :: Int -> ExitOption
primitive consts_ClipPrecision "XSconsts_ClipPrecision" :: Int -> ClipPrecision
primitive consts_ScrollbarStyle "XSconsts_ScrollbarStyle" :: Int -> ScrollbarStyle
primitive consts_ArcDirection "XSconsts_ArcDirection" :: Int -> ArcDirection
primitive consts_EditStyle "XSconsts_EditStyle" :: Int -> EditStyle
primitive consts_ShareMode "XSconsts_ShareMode" :: Int -> ShareMode
primitive consts_SystemMenuCommand "XSconsts_SystemMenuCommand" :: Int -> SystemMenuCommand
primitive consts_KeyRestoreFlags "XSconsts_KeyRestoreFlags" :: Int -> KeyRestoreFlags
primitive consts_ShowWindowControl "XSconsts_ShowWindowControl" :: Int -> ShowWindowControl
primitive consts_StaticControlStyle "XSconsts_StaticControlStyle" :: Int -> StaticControlStyle
primitive consts_ClippingMode "XSconsts_ClippingMode" :: Int -> ClippingMode
primitive consts_DriveType "XSconsts_DriveType" :: Int -> DriveType
primitive consts_DrawTextFormat "XSconsts_DrawTextFormat" :: Int -> DrawTextFormat
primitive consts_VKey "XSconsts_VKey" :: Int -> VKey
primitive consts_RegType "XSconsts_RegType" :: Int -> RegType
primitive consts_DWORD "XSconsts_DWORD" :: Int -> DWORD
primitive consts_TextAlignment "XSconsts_TextAlignment" :: Int -> TextAlignment
primitive consts_TrackMenuFlag "XSconsts_TrackMenuFlag" :: Int -> TrackMenuFlag
primitive consts_ScrollbarType "XSconsts_ScrollbarType" :: Int -> ScrollbarType
primitive consts_ClassStyle "XSconsts_ClassStyle" :: Int -> ClassStyle
primitive consts_Int "XSconsts_Int" :: Int -> Int
primitive consts_CommDlgError "XSconsts_CommDlgError" :: Int -> CommDlgError
primitive consts_GetDCExFlags "XSconsts_GetDCExFlags" :: Int -> GetDCExFlags
primitive consts_GraphicsMode "XSconsts_GraphicsMode" :: Int -> GraphicsMode
primitive consts_ListBoxStyle "XSconsts_ListBoxStyle" :: Int -> ListBoxStyle
primitive consts_FilePtrDirection "XSconsts_FilePtrDirection" :: Int -> FilePtrDirection
primitive consts_MMRESULT "XSconsts_MMRESULT" :: Int -> MMRESULT
primitive consts_TechAndFamily "XSconsts_TechAndFamily" :: Int -> TechAndFamily
primitive consts_Cursor "XSconsts_Cursor" :: Int -> Cursor
primitive consts_SMSetting "XSconsts_SMSetting" :: Int -> SMSetting
primitive consts_FileNotificationFlag "XSconsts_FileNotificationFlag" :: Int -> FileNotificationFlag
primitive consts_SystemColor "XSconsts_SystemColor" :: Int -> SystemColor
primitive consts_StdHandleId "XSconsts_StdHandleId" :: Int -> StdHandleId
primitive consts_BackgroundMode "XSconsts_BackgroundMode" :: Int -> BackgroundMode
primitive consts_RegionType "XSconsts_RegionType" :: Int -> RegionType
primitive consts_MidiCachePolicy "XSconsts_MidiCachePolicy" :: Int -> MidiCachePolicy
primitive consts_BinaryType "XSconsts_BinaryType" :: Int -> BinaryType
primitive consts_WindowStyle "XSconsts_WindowStyle" :: Int -> WindowStyle
primitive consts_LRESULT "XSconsts_LRESULT" :: Int -> LRESULT
primitive consts_PitchAndFamily "XSconsts_PitchAndFamily" :: Int -> PitchAndFamily
primitive consts_DefineDosDeviceFlags "XSconsts_DefineDosDeviceFlags" :: Int -> DefineDosDeviceFlags
primitive consts_MenuFlag "XSconsts_MenuFlag" :: Int -> MenuFlag
primitive consts_MoveFileFlag "XSconsts_MoveFileFlag" :: Int -> MoveFileFlag
primitive consts_MBStatus "XSconsts_MBStatus" :: Int -> MBStatus
primitive consts_MenuState "XSconsts_MenuState" :: Int -> MenuState
primitive consts_AccessMode "XSconsts_AccessMode" :: Int -> AccessMode
primitive consts_MidiOutSupport "XSconsts_MidiOutSupport" :: Int -> MidiOutSupport
primitive consts_RegistryNotifyFilter "XSconsts_RegistryNotifyFilter" :: Int -> RegistryNotifyFilter
primitive consts_EZATTR "XSconsts_EZATTR" :: Int -> EZATTR
primitive consts_GMDIFlag "XSconsts_GMDIFlag" :: Int -> GMDIFlag
primitive consts_WindowMessage "XSconsts_WindowMessage" :: Int -> WindowMessage
primitive consts_KeyDisposition "XSconsts_KeyDisposition" :: Int -> KeyDisposition
primitive consts_MBStyle "XSconsts_MBStyle" :: Int -> MBStyle
primitive consts_FileType "XSconsts_FileType" :: Int -> FileType
primitive consts_OutPrecision "XSconsts_OutPrecision" :: Int -> OutPrecision
primitive consts_StockPen "XSconsts_StockPen" :: Int -> StockPen
primitive consts_ButtonStyle "XSconsts_ButtonStyle" :: Int -> ButtonStyle
primitive consts_BitmapCompression "XSconsts_BitmapCompression" :: Int -> BitmapCompression
primitive consts_FileAttributeOrFlag "XSconsts_FileAttributeOrFlag" :: Int -> FileAttributeOrFlag
primitive consts_StockPalette "XSconsts_StockPalette" :: Int -> StockPalette
primitive consts_PenStyle "XSconsts_PenStyle" :: Int -> PenStyle
primitive consts_WindowsError "XSconsts_WindowsError" :: Int -> WindowsError
primitive consts_HatchStyle "XSconsts_HatchStyle" :: Int -> HatchStyle
primitive consts_CreateMode "XSconsts_CreateMode" :: Int -> CreateMode
primitive consts_StockFont "XSconsts_StockFont" :: Int -> StockFont
primitive consts_Bitmap "XSconsts_Bitmap" :: Int -> Bitmap
primitive consts_MidiTechnology "XSconsts_MidiTechnology" :: Int -> MidiTechnology
primitive consts_REGSAM "XSconsts_REGSAM" :: Int -> REGSAM
primitive consts_FontWeight "XSconsts_FontWeight" :: Int -> FontWeight

needPrims_hugs
